#ifndef __AIFLEET_H__
#define __AIFLEET_H__
#pragma once









#include "Log.h"


class Craft;
class AIFleet;
class STGoal;
class FileWriter;
class FileReader;
class CombatSituation;
class STDamage;


LOGSOURCE_REFERENCE(FLEET_LOG);


class AIFleetFactory;



class AIFleet
{
public:

	

	
	static std::vector <AIFleet *> s_AIFleetList;

	// Unique IDs for fleets
	static int s_nextFleetID;

	
	static bool s_shuttingDown;

	
	static const int sk_NO_AI_FLEET;

	

	
	static void InitAIFleetSystem();


	
	static void ShutDownAIFleetSystem();


	
	static void ProcessAIForAllFleets(float dt);


	
	static AIFleet * GetFleetByUniqueID
	(
		int fleetIDQuery				
	);

	int GetFleetID() const {
		return m_fleetID;
	}


	

	


	
	AIFleet();


	
	virtual ~AIFleet();

	
	virtual const AIFleetFactory* GetFactory() const = 0;

	
	virtual void GetDebugString
	(
		char *buffer,					
		const int maxSize				
	) = 0;


	
	virtual void TryToWarp
	(
		bool tryWarp
	);


	//
	// GetTryToWarp
	//
	// Should we try to use warp speed?
	//
	bool GetTryToWarp()
	{
		
		return m_tryToWarp;
	}


	
	virtual void SetStayInFormationWhileMoving
	(
		bool stayInFormationWhileMoving
	);


	
	bool GetStayInFormationWhileMoving()
	{
		
		return m_stayInFormationWhileMoving;
	}


	
	int GetFleetFormationLeader()
	{
		
		return m_leadShip;
	}


	
	void SetFleetFormationLeader
	(
		int newLeadShip					
	);


	
	const Vector3 &GetFleetFormationOrientation()
	{
		
		return m_formationOrientation;
	}


	
	void SetFleetFormationOrientation
	(
		Vector3 newOrientation
	);

	
	
	float GetMaxFleetFormationImpulseSpeed()
	{
		
		return m_maxFormationImpulseSpeed;
	}


	
	float GetMaxFleetFormationWarpSpeed()
	{
		
		return m_maxFormationWarpSpeed;
	}


	
	virtual void AddShip
	(
		int newShipHandle				
	);

	
	void RemoveShip
	(
		int badShipHandle				
	);

	
	virtual void Disband();

	//
	// SetFleetDestination
	//
	// Give us an ultimate destination
	//
	virtual void SetFleetDestination
	(
		Vector3 ultimateDestination		
	);

	
	virtual void SetTargetHandle
	(
		int targetHandle
	);


	
	virtual void ProcessFleetAI(float dt);


	
	void ChooseANewLeader();


	
	bool IsAnyoneOutOfRange
	(
		int theTarget
	);

		
	bool IsAnyoneOutOfRange
	(
		const float weaponRange, const float targetRange
	);


	
	virtual bool IsFusingFleet()
	{
		return false;
	}

	
	virtual bool IsBiopulseFleet()
	{
		return false;
	}

	
	virtual bool SplitDamage(STDamage *stDamage);


	
	static bool SaveAll(FileWriter &fp);
	static bool LoadAll(FileReader &fp);
	
	static bool PostLoadAll();

	
	virtual bool Save(FileWriter &fp);
	virtual bool Load(FileReader &fp);
	virtual bool PostLoad();

protected:

	

	
	int m_targetHandle;

	

	
	int m_fleetID;

	// What team we're on
	int m_teamID;

	
	bool m_newShipsAdded;

	
	float m_fleetPower;

	

	
	int m_leadShip;

	
	int m_formation;

	
	Vector3 m_formationOrientation;

	
	float m_maxFormationImpulseSpeed;

	
	float m_maxFormationWarpSpeed;

	

	
	Vector3 m_ultimateDestination;

	
	bool m_destinationSet;

	

	// Dynamic array of all the ships in the fleet
	std::vector <int> m_shipsInFleet;

	
	std::vector <int> m_artilleryShips;

	
	std::vector <int> m_fleetDefenseShips;

	
	std::vector <int> m_scoutShips;

	
	std::vector <int> m_combatShips;

	// Non-combat ships in the fleet
	std::vector <int> m_nonCombatShips;

	
	std::vector <int> m_repairShips;

	

	
	int m_lastHelplessShip;

	
	int m_lastBruteWhoShotHim;

	
	float m_lastFleetDefenseTime;

	


	
	void SendFleet
	(
		
		const Vector3 &destination,

		
		int formation
	);

	
	void SendFleetWarp
	(
		
		const Vector3 &destination,

		
		int formation
	);

	
	void SetFleetFormation
	(
		int whichFormation				
	);


	
	virtual std::vector<int>::iterator RemoveShip
	(
		std::vector<int>::iterator removeMe	
	);

	
	Craft * FindFirstLiveShipInList
	(
		std::vector <int> * shipList,	
		bool (*QualifyFcn)(Craft *) = NULL	
	);


	
	bool SendDefenders
	(
		std::vector <int> * defenders,	
		Craft * helplessShip,			
		bool onlyIdleShips				
	);


	
	bool DefendTheseShips
	(
		std::vector <int> * defenders,	
		std::vector <int> * shipsToProtect,	
		float underAttackTime,			
		bool onlyIdleShips = false		// Only send idle defenders
	);


	
	bool SendRepairShip
	(
		Craft * damagedShip				
	);


	
	bool RepairTheseShips
	(
		std::vector <int> * shipsToRepair	
	);


	
	void WeedOutDeadShips();


	
	virtual bool mShouldRemoveShip(Craft*)
	{
		return false;
	}


	
	void RecomputeMaxSpeeds();

	
	void UpdateMaxSpeeds
	(
		Craft *pCraft
	);

	
	virtual void ReplaceLeader();


private:

	

	
	bool m_tryToWarp;

	
	bool m_stayInFormationWhileMoving;


};



class AIFleetFactory
{
public:
	// Auto-adds me to the factory list
	AIFleetFactory();

	
	virtual const char *GetName() const = 0;
	
	
	virtual AIFleet *Build() const = 0;

	
	AIFleetFactory *m_pNext;
};



#endif 
