#ifndef _AiPath_
#define _AiPath_
#pragma once






class AiPath;
class FileReader;
class FileWriter;
class NebulaParams;

typedef std::list<AiPath *> AiPathList;

typedef enum {
	ONE_WAY_PATH,
	ROUND_TRIP_PATH,
	LOOP_PATH,
	ASTEROID_BELT,
	NEBULA_FIELD,
	BAD_PATH
} PathType;




class AiPath {
public:

	

	PathType pathType;

	
	float asteroid_belt_speed;

	
	int m_nebula_type;


	

	AiPathList::iterator me;
	static AiPathList pathList;	
	static AiPath *Find(const char *label);
	static void Cleanup(void);
	static bool LoadAll(FileReader &fp);
	static bool SaveAll(FileWriter &fp);
	static void Release(AiPath * & path, bool async = false);

	AiPath(void);
	AiPath(const char *lbl);
 	AiPath(const Vector3 &p1, const Vector3 &p2, const char *lbl = NULL);
	AiPath(const Vector3 &the_point);

	//this copy constructor is made explicit (to prevent implicit
	//copies) since it performs a "deep" copy (copying all points),
	//which is expensive
	explicit AiPath(const AiPath &path);
	~AiPath();

	bool Load(FileReader &fp);
	bool Save(FileWriter &fp);

	const char *GetLabel() const { 
		return m_pLabel; 
	}
	void SetLabel(const char *lbl);

	int GetNumPoints() const 
	{ 
		int pointCount = m_points.GetSize();
		ASSERT(pointCount >= 0);
		ASSERT(pointCount < 10000);
		return pointCount; 
	}
	Vector3 GetFirstPoint() const 
	{ 
		int pointCount = m_points.GetSize();
		ASSERT(pointCount >= 0);
		ASSERT(pointCount < 10000);

		return m_points[0]; 
	}
	Vector3 GetPoint(int index) const
	{
		ASSERT(index >= 0 && index < m_points.GetSize());
		return m_points[index];
	}
	
	Vector3 GetLastPoint() const 
	{ 
		int pointCount = m_points.GetSize();
		ASSERT(pointCount >= 0);
		ASSERT(pointCount < 10000);

		return m_points[pointCount - 1]; 
	}
	
	
	LSizedArray<Vector3> &GetPointList()
	{
		return m_points;	
	}
	
	const LSizedArray<Vector3> &GetPoints() const 
	{ 
		return m_points; 
	}
	
	
	void AppendPathPoint(const Vector3& newPoint)
	{
		m_points.Append(newPoint);
		mClearOutDistances();
	}
	
	
	void InsertPathPoint(int atIndex, const Vector3& pNewPosition);
	
	
	void RemovePathPoint(int index);
	
	
	void ClearPoints()
	{
		m_points.Clear();
		
		mClearOutDistances();
	}
	
	
	void ChangeLastPoint(const Vector3 &new_last_point)
	{
		ASSERT(fabs(new_last_point.y)<1e20);
		
		
		int pointCount = m_points.GetSize();
		m_points[pointCount - 1] = new_last_point;
		if (m_pDistances != NULL) 
		{
			m_pDistances[pointCount - 1] = m_pDistances[pointCount - 2] 
				+ GetDistance(m_points[pointCount - 2], m_points[pointCount - 1]);
		}
	}
	void AddPointAtBegining(const Vector3 &new_point);

	
	void AppendPathToEnd(const AiPath* path);
	
	
	float TotalDistanceTo(int index) const
	{
		
		if (!m_pDistances)
			mSetDistances();

		ASSERT (m_pDistances != NULL);
		return m_pDistances[index];
	}

	
	float GetPathLength();

	
	Vector3 PointAtDistance(float total_distance, int &segment_number) const;


	
	float DistanceAlongSegment(int segmentStartpoint, float percentAlongSegment) const;

	
	
	float NearestPointOnPathToQuery
	(
		const Vector3& query, 
		Vector3* nearestPoint, 
		int* segmentStartpoint, 
		float* percentageAlongSegment 
	) const;


	//
	// Set_Up_Pathing_Around_Asteroid_Belt
	//
	// If this path is an asteroid belt, and the pathing restrictions around
	// it have not yet been set up, set them up!
	void Set_Up_Pathing_Around_Asteroid_Belt();

	
	void AddAsteroidsToBelt();

	
	void BlowAwayAsteroids();


	//
	// BlowAwayNebulae
	//
	// Blow away all nebulae who belong to this field
	//
	void BlowAwayNebulae( NebulaParams*out_pParams);


	
	bool PointInPolygon
	(
		const Vector2 &queryPoint		
	);


private:

	
	
	
	LDynArray<Vector3> m_points;
	
	
	bool m_pathPlanningInited;

	char *m_pLabel;
	mutable float *m_pDistances;

	

	
	void mClearOutDistances() const;
	
	
	void mSetDistances() const;

	
	float mFindNearestPointToSegment(const Vector3& query, 
		int segmentStartpoint,
		float* percentageAlongSegment,
		Vector3* result) const;

};

#endif
