#ifndef __CINERACTIVE_H__
#define __CINERACTIVE_H__
#pragma once




class CameraTrack;		// This is a support class only used internally.
class GameObject;
class AiPath;
class hot_int;


class script;


typedef int CinHandle;			




extern hot_int isCineractiveDone;


extern hot_int isCineractiveCancel;




class Cineractive {
public:

	
	static void Start(script * calledFromScript = NULL);
	
	
	static void End();


	// Constructor.
	Cineractive(script * calledFromScript = NULL);

	
	~Cineractive();


	
	void CameraOffset(float x, float y, float z, bool isOrientationRelative = true);
	void CameraOffset(const char* path, float offsetSpeed, bool isOrientationRelative = true);
	void TargetOffset(float x, float y, float z, bool isOrientationRelative = true);
	void TargetOffset(const char* path, float offsetSpeed, bool isOrientationRelative = true);

	
	void Shot(const char* path, int index, bool useOffsetDirection = false);
	void Shot(const char* path, int index, const char* targetPath, int targetIndex);
	void Shot(const char* path, int index, CinHandle targetHandle);
	void Shot(const char* path, int index, const char* targetPath, float targetSpeed);
	void Shot(CinHandle handle, bool useOffsetDirection = false);
	void Shot(CinHandle handle, const char* targetPath, int targetIndex);
	void Shot(CinHandle handle, CinHandle targetHandle);
	void Shot(CinHandle handle, const char* targetPath, float targetSpeed);
	void Shot(const char* path, float speed, bool useOffsetDirection = false);
	void Shot(const char* path, float speed, const char* targetPath, int targetIndex);
	void Shot(const char* path, float speed, CinHandle targetHandle);
	void Shot(const char* path, float speed, const char* targetPath, float targetSpeed);
	void ShotTrackLeft(const char* path, int index, const char* targetPath, int targetIndex, CinHandle targetHandle);


	
	void ConstantSpeed(bool constantSpeed);

	
	void SetPathSegmentSpeedModifier(int segment, float segmentSpeed);

	
	void Update(float dt);

	
	void SetStepScript(script * calledFromScript = NULL);

private:

	
	bool m_stepCountAdded;
	void addScriptStepCount(void);
	void removeScriptStepCount(void);

	
	void mLookAtOffset(Vector3 &point, bool isOrientationRelative = true);
	void mLookAtOffset(GameObject *gameObject, bool isOrientationRelative = true);
	void mLookAtOffset(AiPath *path, bool isOrientationRelative = true);

	
	void mSetLocationSpeed(float speed) {
		m_locationSpeed = speed;
	}
	void mSetTargetSpeed(float speed) {
		m_targetSpeed = speed;
	}

	void mSetLocationOffsetSpeed(float speed) {
		m_locationOffsetSpeed = speed;
	}
	void mSetTargetOffsetSpeed(float speed) {
		m_targetOffsetSpeed = speed;
	}

	
	void mLookFromOffset(Vector3 &point, bool isOrientationRelative = true);
	void mLookFromOffset(GameObject *gameObject, bool isOrientationRelative = true);
	void mLookFromOffset(AiPath *path, bool isOrientationRelative = true);


	
	void mLookFrom(const Vector3 &point);
	void mLookFrom(GameObject *gameObject);
	void mLookFrom(AiPath *path);

	
	enum TrackingType {
		TRACK_LEFT
	};
	void mLookAt(const Vector3 &point);
	void mLookAt(GameObject *gameObject);
	void mLookAt(GameObject *gameObject, const Vector3 &point, enum TrackingType type);
	void mLookAt(AiPath *path);
	void mLookAt(bool doWeUseOffsetDirection = false);	

	
	void mSetPointSpeedModifiers();

	
	void mSetSegmentSpeed(int segment, float segmentSpeed);

	
	void mSetConstantSpeed(bool constantSpeed)
	{
		m_constantSpeed = constantSpeed;
	}

	// Resets the camera to the start time.  For now the main effect of this
	// is to paths because it puts the camera at the beginning of the path.
	// This should be called anytime LookFrom or LookAt is called.
	void mReset();

	
	bool mIsDone();

	
	void mAdvance(float deltaTime);

	
	void mUpdate();

	
	script * m_calledFromScript;

	
	bool m_constantSpeed;

	
	bool m_isDirectional;
	bool m_useOffsetDirection;

	
	bool m_isCameraOffsetOrientationRelative;
	bool m_isTargetOffsetOrientationRelative;

	float m_oldFarClippingPlane;

	
	std::vector<float> m_speedModifier;

	
	CameraTrack *m_pLocation;

	
	CameraTrack *m_pTarget;

	// Offset track for camera position.
	CameraTrack *m_pLocationOffset;

	
	CameraTrack *m_pTargetOffset;

	
	Matrix34 m_cameraMatrix;

	
	float m_locationSpeed;
	float m_targetSpeed;
	float m_locationOffsetSpeed;
	float m_targetOffsetSpeed;
	
	//These variables are used to remember the user's audio settings.  We kinda take control of 
	//them during a cinerative
	int m_cineractiveVolume_Music;
	int m_cineractiveVolume_Voice;
	int m_cineractiveVolume_FX;
	
	float m_elapsedTime;
	
	
	LDynArray<int> m_selectedEntityIds;
};


extern Cineractive* g_pCineractiveCamera;







class CineractiveCancelMonitor
{
public:
	
	CineractiveCancelMonitor();
	~CineractiveCancelMonitor();

	
	void Reset();

	
	void CancelRequested();

	
	void CancelRequested (int slot);

	
	bool IsCancelled();

	
	bool MyCancelPending();

private:
	
	bool *m_cancel_requested;

	// did my local node request a cancel?  (which hasn't yet been transmitted through the network system)
	bool m_cancel_pending;
};	
	

extern CineractiveCancelMonitor *g_pCineractiveCancelMonitor;


#endif // __CINERACTIVE_H__
