#ifndef __DefendFleet_H__
#define __DefendFleet_H__
#pragma once

///////////////////////////////////////////////////////////////////////////////
//
// $Header: //StarTrek/RTM/PC/src/Source/AI/FleetAI/DefendFleet.h#1 $
//
// FILE:          DefendFleet.h
//
// DESCRIPTION:   Manages fleets sent to lay Defend to bases
//
// AUTHOR:        Mad Doctor I
//
// Copyright 2001 Mad Doc Software, LLC.  All Rights Reserved.
//
///////////////////////////////////////////////////////////////////////////////




#include "AIFleet.h"







class DefendFleet : public AIFleet
{
public:

	///////////////////////////////////////////////////////////////////////////
	//
	// TYPES
	//
	///////////////////////////////////////////////////////////////////////////

	
	enum eDefendStage
	{
		k_DEFEND_IDLE = 0,				
		k_DEFEND_RALLY,					
		k_DEFEND_DEFENSE					
	};

	///////////////////////////////////////////////////////////////////////////
	//
	// PUBLIC VARIABLES
	//
	///////////////////////////////////////////////////////////////////////////

	


	//
	// DefendFleet
	//
	// Construct an AI fleet used for laying Defend
	//
	DefendFleet();


	
	virtual ~DefendFleet();

	
	virtual const AIFleetFactory* GetFactory() const;

	
	void GetDebugString
	(
		char *buffer,					
		const int maxSize				
	);



 
	
	void SetDefendStage
	(
		eDefendStage newDefendStage		
	);

	
	virtual void ProcessFleetAI(float dt);


	
	virtual bool Save(FileWriter &fp);
	virtual bool Load(FileReader &fp);


protected:

	

	

	
	eDefendStage m_DefendStage;

	

	
	float m_rallyStartTime;

	
	Vector3 m_rallyLocation;

	

	
	float m_lastMoveTime;

	
	Vector3 m_defensiveLocation;

	
	float m_stationAddTime;

	
	std::vector <int> m_stationsToDefend;
	

	


	
	void StartDefendIdle();


	
	void DefendIdle();


	
	void ChooseRallyLocation();


	
	void SendFleetToRallyLocation();


	
	void StartDefendRally();


	
	bool RallyIsSuccessful();
	
	
	
	bool RallyTimeIsOver();


	
	void DefendRally();


	
	void AddStationsToDefendList
	(
		bool forceRecompute = false		
	);


	
	bool AreWeFighting();


	
	void ChooseRandomDefenseLocation();


	
	void SendShipsToDefensiveLocation();


	
	void StartDefendDefense();


	//
	// SeeIfWeShouldMoveWithinDefensiveRegion
	//
	// Description: If enough time has passed since we were last attacked,
	// juggle us to a new defensive position
	// Written: 5/6/2001 by Dr. Ian Lane Davis
	//
	void SeeIfWeShouldMoveWithinDefensiveRegion();


	
	void DefendAgainstAttacks();


	
	void DefendDefense();


};
//
// CLASS DefendFleet
//
///////////////////////////////////////////////////////////////////////////////


#endif 
