

#ifndef LDEBUGMEMORY_H
#define LDEBUGMEMORY_H


namespace LDebugMemory
{
	
	class CleanupCallback
	{
	public:
		
		CleanupCallback();

		~CleanupCallback();

		
		virtual void Cleanup(void) = 0;

		// cleanup all the auto-registered cleanup callbacks
		static void CleanupAll();
	};

	
	void RegisterCleanupCallback(void (*pfnCleanup)());

	
	void Init();

	
	void RegisterUninitializedEnable(int** in_ppUninitializedCheckEnable);

	
	void Term(void);
	
	
	uint32 GetTotalAllocationSize();
};

class AutoCleanup
{
public:
	AutoCleanup(void (*pfnCleanup)())
	{
		LDebugMemory::RegisterCleanupCallback(pfnCleanup);
	}
};


#define AUTO_CLEANUP(name, fnCleanup) static AutoCleanup name(&fnCleanup)




#ifdef MEMORY_FAST

#define VALIDATE_BLOCK(b)	(true)
#define VALIDATE_ALL()		(true)
#define DISABLE_UNINITIALIZED_CHECK()

#else if MEMORY_LOGGED

int   DebugMemory_ValidateAll	 (const char *file, int line);
int   DebugMemory_ValidateBlock	 (const char *file, int line, void *p);

namespace LDebugMemory
{
	
	void LogAllocations(const char *in_pFunctionName);

	
	bool SetUninitializedCheck(bool bEnable);

	class UnitializedAutoDisable
	{
	public:
		UnitializedAutoDisable()
		{
			m_bWasEnabled = SetUninitializedCheck(false);
		}

		~UnitializedAutoDisable()
		{
			SetUninitializedCheck(m_bWasEnabled);
		}
	private:
		bool m_bWasEnabled;
	};
}

#define VALIDATE_BLOCK(b)	DebugMemory_ValidateBlock  ((__FILE__), (__LINE__), b)
#define VALIDATE_ALL()		DebugMemory_ValidateAll	   ((__FILE__), (__LINE__))
#define DISABLE_UNINITIALIZED_CHECK() LDebugMemory::UnitializedAutoDisable autoDisable

#endif




#endif 
