
#ifndef FASTFLOAT_H
#define FASTFLOAT_H

#include <float.h>  


//---------------------------------------------------------------------------
//Abs - fast absolute value;  about 7x faster than fabs
//---------------------------------------------------------------------------
inline float Abs( float f )
{
	__asm and f, 0x7FFFFFFF  
	return f;
}


inline float FABS( float r )  { return Abs(r); }
inline float  ABS( float r )  { return Abs(r); }



// SHANNON: I changed this so that it's no longer inline.
/*inline int Round2( double d )
{
	int retval;

	__asm fld d
	__asm fistp retval

	return retval;
} 
*/int Round2( double d );



inline int Floor( double d )
{
	ASSERT( (_controlfp(0,0) & MCW_PC) == _PC_53 );  //FPU must be in 64-bit mode
	static const unsigned SHIFT = 16;  
	static const double BIAS = 68719476736.0 * 1.5;  
	d += BIAS;  
	return (int)((*(__int64*)&d) >> SHIFT);  
} 


inline int Round( double d )
{
	return Floor(d + 0.5f);
} 


inline int Ceil( double d )
{
	int i = Round2(d);  
	if( (float)i < d )
		return ++i;
	else
		return i;
} 



inline int Trunc( double d )
{
	if( d >= 0 )
		return Floor(d);
	else
		return Ceil(d);
} 


#endif FASTFLOAT_H


 