// Copyright 2002 Mad Doc Software, LLC.  All rights reserved.
// $Header: //StarTrek/RTM/PC/src/lib/dstruct/LSizedArray.h#1 $
// $Modtime: 1/25/05 12:55p $
//
#ifndef LSIZEDARRAY_H
#define LSIZEDARRAY_H



template<class DATATYPE>
class LSizedArray
{
public:
	
	LSizedArray() 
	{
	}

	
	LSizedArray(int size, DATATYPE *pData) 
	{
		SetData(size, pData);
	}

	
	int GetSize(void) const
	{
		return m_size;
	}

	
	static unsigned GetElementSize()
	{
		return sizeof(DATATYPE);
	}

	
	unsigned GetLengthBytes() const
	{
		return m_size * sizeof(DATATYPE);
	}
    
    
    LSizedArray<DATATYPE> GetOffsetArray(int n) const
    {
        ASSERT(n <= m_size);
        return LSizedArray<DATATYPE>(m_size - n, m_pData + n);
    }

	
	DATATYPE* GetPtr(unsigned index = 0) const
	{
		return (m_pData + index);
	}

	
	const DATATYPE& operator [](int i) const
	{
		ASSERT((0<=i) && (i<m_size));
		return m_pData[i];
	}

	// return or assign an element
	DATATYPE& operator [](int i) 
	{
		ASSERT((0<=i) && (i<m_size));
		return m_pData[i];
	}
	
	
	const DATATYPE& Last() const
	{
		ASSERT(m_size > 0);
		return m_pData[m_size-1];
	}

	
	DATATYPE& Last() 
	{
		ASSERT(m_size > 0);
		return m_pData[m_size-1];
	}
	
	
    void Fill(const DATATYPE& in_value)
    {
        for (int i = 0; i < m_size; i++)
            m_pData[i] = in_value;
    }
	
	
	template <class COMPARABLEDATATYPE>
	bool Contains(COMPARABLEDATATYPE const &x) const
	{
		for( int i = m_size-1; i >= 0; --i )
		{
			if( m_pData[i] == x )
			{
				return true;
			}
		}
		return false;
	}

	
	template <class COMPARABLEDATATYPE>
	int Find(const COMPARABLEDATATYPE &findMe) const
	{
		for (int i = 0; i < m_size; ++i)
		{
			if (findMe == m_pData[i])
				return i;
		}
		return -1;
	}
	
	template <class COMPARABLEDATATYPE, class COMPARECLASS>
	int FindCustomCompare(const COMPARABLEDATATYPE &findMe, const COMPARECLASS &comparer) const
	{
		for (int i = 0; i < m_size; ++i)
		{
			if (comparer.Equal(findMe, m_pData[i]))
				return i;
		}
		return -1;
	}
	
	
	template <class COMPARABLEDATATYPE>
	int BinarySearch(const COMPARABLEDATATYPE &findMe, 
					 int *out_insertIndex = NULL)  
		const
	{
		
		int iMin=0, iMax=m_size;
		while ( iMin < iMax )
		{
			int iCur = (iMin + iMax) >> 1;
			if( m_pData[iCur] == findMe )
			{
				
				return iCur;
			}
			else if( m_pData[iCur] < findMe )
			{
				iMin = iCur + 1;
			}
			else 
			{
				iMax = iCur;
			}
		}
		if (out_insertIndex)
			*out_insertIndex = iMin;
		return -1;
	}
	
	
	template <class COMPARABLEDATATYPE, class COMPARECLASS>
	int BinarySearchCustomCompare(const COMPARABLEDATATYPE &findMe, const COMPARECLASS &comparer,
						 int *out_insertIndex = NULL)  // if not found, this will be filled with the index to insert 'findMe' at, can be NULL
		const
	{
		
		int iMin=0, iMax=m_size;
		while ( iMin < iMax )
		{
			int iCur = (iMin + iMax) >> 1;
			if( comparer.Equal(m_pData[iCur], findMe ))
			{
				
				return iCur;
			}
			else if( comparer.Less(m_pData[iCur], findMe ))
			{
				iMin = iCur + 1;
			}
			else // ( m_pData[iCur] > findMe )
			{
				iMax = iCur;
			}
		}
		if (out_insertIndex)
			*out_insertIndex = iMin;
		return -1;
	}

	
	bool operator == (const LSizedArray<DATATYPE> &in_rhs) const
	{
		if( GetSize() == in_rhs.GetSize() )
		{
			for( int i=0; i<GetSize(); i++ )
			{
				if( !(m_pData[i] == in_rhs.m_pData[i]) )
				{
					
					return false;
				}
			}
			
			return true;
		}
		
		return false;
	}

protected:
	
	void SetData(int size, DATATYPE *pData)
	{
		m_size = size;
		m_pData = pData;
	}

	DATATYPE *m_pData;		
	int m_size;		
};


typedef LSizedArray<int8> LSizedArray_int8;
typedef LSizedArray<uint8> LSizedArray_uint8;
typedef LSizedArray<int16> LSizedArray_int16;
typedef LSizedArray<uint16> LSizedArray_uint16;
typedef LSizedArray<int32> LSizedArray_int32;
typedef LSizedArray<uint32> LSizedArray_uint32;
typedef LSizedArray<float> LSizedArray_float;
typedef LSizedArray<int> LSizedArray_int;
typedef LSizedArray<uint> LSizedArray_uint;
typedef LSizedArray<char> LSizedArray_char;
typedef LSizedArray<bool> LSizedArray_bool;
typedef LSizedArray<char*> LSizedArray_string;

#endif 
