#ifndef ATVI_MATH_SCALAR_H
#define ATVI_MATH_SCALAR_H



const float DEG2RAD = 1.74532922e-2f;		
const float RAD2DEG = 57.29578049f;			



const float S0      =0.0f;
const float QUARTER =0.25f;
const float S1      =1.0f;
const float S2      =2.0f;
const float S3      =3.0f;

const float PI2     =S2*PI;

const float HUGE_NUMBER	=1.0e30f;		

const float E2=EPSILON*EPSILON;

const float EQUALS_EPSILON = 0.001f;


const float k_SigmoidEpsilon = .001f;

double rsqrt (double x);




inline long Double2Long(const double f)
{

	return long(f);
}


inline long Float2Long_Nearest(const float f)
{
// buggy on PPC
//	FLOAT2INT v;
//	v.f = f + Float2Int;
//	return v.i;
	return long(f);
}

inline bool NonZero(float x)
{
	return (x < -EPSILON) || (x > EPSILON);
}


long Signed_Div (long Num, long Den);
long GetInt (float Value);
float Atan2 (float x, float y);


float Fmod (float x, float Period);



inline float Fmod1 (float x, float Period)
{
   x = Fmod (x, Period);
   if (x < 0.0f) x += Period;
   return x;
}


// square the argument.
inline float sqr(float x)
{
	return x * x;
}



inline float Sigmoid(float x, float transitionSharpness=1.0f)
{
	return 1.0f/(1.0f+exp(-transitionSharpness*x));
}


inline float ln(float x) { return log(x); }


inline float SigmoidalInterpolator(float t, float transitionSharpness=1.0f)
{
	
	float xmin = ln(1.0f/k_SigmoidEpsilon - 1.0f)/transitionSharpness;
	
	float x = 2.0f * xmin * t - xmin;
	
	return Sigmoid(x, transitionSharpness);
}



#endif 