#ifndef __SIEGEFLEET_H__
#define __SIEGEFLEET_H__
#pragma once






#include "AIFleet.h"






class cDefensiveStructure
{
public:

	
	int m_turretID;

	
	float m_distanceFromRally;

	
	cDefensiveStructure() : m_turretID(0), m_distanceFromRally(-1) {}
	
};





bool IsValidSpotter
(
	Craft *potentialSpotter			
);



class SiegeFleet : public AIFleet
{
public:

	

	
	enum eSiegeStage
	{
		k_SIEGE_IDLE = 0,				
		k_SIEGE_RALLY,					
		k_SIEGE_BREACH,					
		k_SIEGE_ATTACK,					
		k_SIEGE_DEFENSE,				
		k_SIEGE_RETREAT					
	};

	

	///////////////////////////////////////////////////////////////////////////
	//
	// PUBLIC MEMBER FUNCTIONS
	//
	///////////////////////////////////////////////////////////////////////////


	
	SiegeFleet();


	
	virtual ~SiegeFleet();

	
	virtual const AIFleetFactory* GetFactory() const;

	
	void GetDebugString
	(
		char *buffer,					
		const int maxSize				
	);


 
	
	void SetSiegeStage
	(
		eSiegeStage newSiegeStage		
	);

	
	virtual void ProcessFleetAI(float dt);


	
	virtual bool Save(FileWriter &fp);
	virtual bool Load(FileReader &fp);


protected:

	

	

	
	eSiegeStage m_siegeStage;

	

	
	float m_rallyStartTime;

	
	Vector3 m_rallyLocation;

	

	
	std::vector <cDefensiveStructure> m_defensesToBreach;

	
	int m_targetTurretIndex;

	
	bool m_breachOrdersGiven;

	
	std::vector <int> m_targetList;

	
	float m_breachCheckTime;

	

	
	int m_chumpWeSentToSpot;

	
	int m_targetWereSpotting;

	

	
	int m_orbitingShip;

	
	int m_planetWeWant;


	


	
	void StartSiegeIdle();


	
	void SiegeIdle();


	
	void ChooseRallyLocation();


	
	void SendFleetToRallyLocation();


	
	void StartSiegeRally();


	
	bool RallyIsSuccessful();
	
	
	//
	// RallyTimeIsOver
	//
	// Has enough time passed that we skip the rally phase?
	//
	bool RallyTimeIsOver();

	
	void SiegeRally();


	
	void AddDefensesToBreachList
	(
		bool forceRecompute = false		
	);


	
	void StartSiegeBreach();


	
	void SpotTheTarget
	(
		Craft * target					
	);


	
	void TakeOutNextDefense
	(
		std::vector <int> * breachingShips	
	);



	//
	// DefendAgainstAttacks
	//
	// If our ships are being attack, defend them
	//
	void DefendAgainstAttacks();


	
	void SiegeBreach();

	//
	// StartSiegeAttack
	//
	// Make sure we're attacking in a coordinated fashion
	//
	void StartSiegeAttack();

	
	void SiegeAttack();

	
	void StartSiegeDefense();

	
	void SiegeDefense();

	//
	// StartSiegeRetreat
	//
	// Get out of dodge
	//
	void StartSiegeRetreat();

	
	void SiegeRetreat();

	
	void FindPlanetToTakeOver();

	
	void TryToOrbitPlanet();
};



#endif 
