

#ifndef _A1_SPECIAL_FLAG_H_
#define _A1_SPECIAL_FLAG_H_
#pragma once


#ifdef __common_h__
#undef is
#endif 



#ifdef __common_h__
#define is ==
#endif 


class GameObject;
class FileReader;
class FileWriter;


class SpecialFlag {
public:

	
	enum Restriction {
		NO_RESTRICTIONS			= 0,	
		ALLOW_SAME_TEAM			= 1,	
		ALLOW_ALLY				= 2,	
		ALLOW_ENEMY				= 4,	
		ALLOW_NEUTRAL			= 8,	
		ALLOW_ODF				= 16,	
		ALLOW_SHIP				= 32,	
		RESTRICT_TO_SAME_TEAM	= 64,	
		RESTRICT_TO_ALLY		= 128,	
		RESTRICT_TO_ENEMY		= 256,	
		RESTRICT_TO_NEUTRAL		= 512,	
		RESTRICT_ALL			= 1024	
	};

	// Define an invalid handle.
	static const int INVALID_HANDLE;

	
	SpecialFlag();

	
	SpecialFlag(int handle);

	
	virtual ~SpecialFlag();

	
	static bool SaveClassInfo(FileWriter &fp);
	static bool LoadClassInfo(FileReader &fp);

	
	bool Load(FileReader &fp);
	bool Save(FileWriter &fp);

	

	
	static SpecialFlag *GetFlag(int handle);

	
	int GetHandle() { return(m_handle); }

	
	char *GetName() { return(m_name); }

	
	void SetName(char *name);

	
	GameObject *GetLocation() { return(m_location); }

	///////////////////////////////////////////////////////////////////////////
	// Simulation
	///////////////////////////////////////////////////////////////////////////

	
	virtual void Simulate(float dt);

	

	
	virtual bool CanTransport(GameObject *requestedDestination);

	
	virtual void Arrival(GameObject *object);

	
	virtual void Departure(GameObject *object);

	

	
	void AddRestriction(Restriction restriction);

	
	void AddRestriction(Restriction restriction, char *odfName);

	
	void AddRestriction(Restriction restriction, int handle);

	
	void RemoveRestriction(Restriction restriction);

	
	void ClearRestrictions();

	
	bool IsRestricted(Restriction restriction);

private:
	
	static int m_nextHandle;

	
	static std::map<int, SpecialFlag*> m_handleMap;

	
	int m_handle;

	
	GameObject *m_location;

	
	char *m_name;

	
	int m_restrictions;

	
	std::list<std::string> m_odfRestrictions;

	// List of handles for ALLOW_SHIP.
	std::list<int> m_handleRestrictions;
};

#endif 
