


#ifndef VEC3F_H
#define VEC3F_H

#include "LMathConsts.h"

class NiPoint3;
class Vec2f;

// 3d float Vector
class Vec3f 
{
public:
	float x;
	float y;
	float z;

	Vec3f() {}	

	Vec3f(float inX, float inY, float inZ)
		: x(inX)
		, y(inY)
		, z(inZ) 
	{}
	
	Vec3f(const Vec2f &inVector, float inZ) 
		: x( ((float*)(&inVector))[0] )
		, y( ((float*)(&inVector))[1] )
		, z(inZ)
	{}

	operator NiPoint3&() { 
		return *((NiPoint3*)this); 
	};

	operator const NiPoint3&() const { 
		return *((NiPoint3*)this); 
	};

	
	NiPoint3& GetNiPoint3( ) { 
		return *((NiPoint3*)this); 
	};

	const NiPoint3& GetNiPoint3() const { 
		return *((NiPoint3*)this); 
	};

	
	operator Vec2f&() {
		return *((Vec2f*)this);
	}

	operator const Vec2f&() const {
		return *((Vec2f*)this);
	}

	
	Vec2f& GetVec2f() {
		return *((Vec2f*)this);
	}
	const Vec2f& GetVec2f() const {
		return *((Vec2f*)this);
	}


	
	void Set(float newX, float newY, float newZ) {
		x = newX; y = newY; z = newZ;
	}

	
	void Set(const Vec2f &xy, float zin) {
		x = ((float*)(&xy))[0];
		y = ((float*)(&xy))[1];
		z = zin;
	}

	
	void SetZero() {
		x = y = z = 0;
	}

	
	bool operator== (const Vec3f& inVector) const { 
		return (inVector.x == x && inVector.y == y && inVector.z == z); 
	}

	bool operator!= (const Vec3f& inVector) const { 
		return (inVector.x != x || inVector.y != y || inVector.z != z); 
	}

	
	bool IsEqual(const Vec3f& in_vector, float in_epsilon = EPSILON) const;

	Vec3f operator+ (const Vec3f& inVector) const { 
		return Vec3f(x+inVector.x, y+inVector.y, z+inVector.z); 
	}

	Vec3f operator- (const Vec3f& inVector) const { 
		return Vec3f(x-inVector.x, y-inVector.y, z-inVector.z); 
	}

    Vec3f operator- () const {
		return Vec3f(-x,-y,-z);
	}

	Vec3f operator* (const float scale) const
		{ return Vec3f(x*scale, y*scale, z*scale); }

	Vec3f &operator*= (const float scale) { 
		x *= scale; 
		y *= scale; 
		z *= scale; 
		return *this; 
	}

	Vec3f operator/ (const float scale) const
		{ return Vec3f(x/scale, y/scale, z/scale); }

	Vec3f &operator/= (const float scale) { 
		x /= scale; 
		y /= scale; 
		z /= scale; 
		return *this; 
	}

	Vec3f &operator+=(const Vec3f &rhs)
	{
		x += rhs.x;
		y += rhs.y;
		z += rhs.z;
		return *this;
	}

	Vec3f &operator-=(const Vec3f &rhs) 
	{
		x -= rhs.x;
		y -= rhs.y;
		z -= rhs.z;
		return *this;
	}

	float& operator[] (int dir)	{ 
		ASSERT(dir >=0 && dir < 3);
		return *(&x + dir);
	}

	const float& operator[] (int dir) const	{ 
		ASSERT(dir >= 0 && dir < 3);
		return *(&x + dir);
	}

	float Length() const { return sqrt(x*x+y*y+z*z); }

	float LengthSqr() const { return (x*x+y*y+z*z); }

	
	float Length2D() const { return sqrt(x*x+y*y); }

	
	float LengthSqr2D() const { return (x*x+y*y); }

	float Distance(const Vec3f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		const float dz = z-inVector.z;
		return sqrt(dx*dx + dy*dy + dz*dz);
	}

	float DistanceSqr(const Vec3f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		const float dz = z-inVector.z;
		return dx*dx + dy*dy + dz*dz;
	}
	
	static float DistanceSqr(Vec3f& inVector1, Vec3f& inVector2) {
		const float dx = inVector1.x-inVector2.x;
		const float dy = inVector1.y-inVector2.y;
		const float dz = inVector1.z-inVector2.z;
		return dx*dx + dy*dy + dz*dz;
	}

	
	float Distance2D(const Vec3f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		return sqrt(dx*dx + dy*dy);
	}

	float DistanceSqr2D(const Vec3f& inVector) const {
		const float dx = x-inVector.x;
		const float dy = y-inVector.y;
		return dx*dx + dy*dy;
	}


	float Dot(const Vec3f& inVector) const { 
		return x*inVector.x + y*inVector.y + z*inVector.z; 
	}

	Vec3f Cross(const Vec3f& inVector) const { 
		return Vec3f(y*inVector.z - inVector.y*z, z*inVector.x - inVector.z * x, x*inVector.y - inVector.x*y); 
	}

	// Gives you a normalized cross product.
	Vec3f CrossNormalize(const Vec3f & inVector) const {
		Vec3f a = Cross(inVector);
		a.Normalize();
		return a;
	}

	// Normalizes, returning the length.
	float Normalize() {
		float len = Length();
		if (len > 0)
		{
			float fReciprocal = 1.0f/len;
			x *= fReciprocal;  y *= fReciprocal; z *= fReciprocal;
		}
		return len;
	}

	// rotate around the Z axis by fAngle, returns it.
	// Very bad name.  Since you'd think it acted in place,
	// rather than returning something...
	Vec3f RotateZ(double fAngle) const;

	
	float GetAngleZ() const;
	

	

	
	void GetPolarCoords(float *out_pTheta, float *out_pPhi) const;

	
	void SetFromPolarCoords(float in_fTheta, float in_fPhi);

	
	void GetAzimuthAltitudeCoords(float *out_pAzimuth, float *out_pAltitude) const;
	void SetFromAzimuthAltitudeCoords(float in_fAzimuth, float in_fAltitude);
	
	
	float GetAngleBetween( const Vec3f &in_vector) const;
};


// We separate these so we can see the damn vector in the debugger.
class Vec3fConst 
{
public:
	
	static const Vec3f ZERO;
	static const Vec3f UNIT_X;
	static const Vec3f UNIT_Y;
	static const Vec3f UNIT_Z;
	static const Vec3f UNIT_ALL;	
};



inline Vec3f operator* (float fScalar,const Vec3f &rhs) 
{
	return rhs * fScalar;
}



#endif 
