

#ifndef __linked_list_H__
#define __linked_list_H__
#pragma once


#ifndef _ASSERTE
#define _ASSERTE(x)
#define _UNDEF_ASSERTE
#endif
//
#include "Pointer_Pool.h"

#ifdef  _UNDEF_ASSERTE
#undef _ASSERTE
#endif


#ifndef ABS
template< typename t_Type >
inline t_Type ABS( t_Type value ) {
	if( value < 0 )
		return -value;
	return value;
}
#endif





typedef void * data_ptr;
typedef class list_cell * list_cell_ptr;






class list_cell 
{
public:

	// create an invalid list cell (pointer pool requires this)
	inline list_cell() :
		cell_data (NULL),
		next (NULL),
		prev (NULL)
	{
	}

	
	inline list_cell(data_ptr cell_data_in, list_cell* next_in, list_cell* prev_in) :
		cell_data (cell_data_in),
		next (next_in),
		prev (prev_in)
	{
	}

	
	inline ~list_cell()
	{
	}

#ifndef DONT_USE_POOLS

	void* operator new(size_t  )
	{
		
		if (s_list_cell_pool == NULL)
		{
			s_list_cell_pool = new Pointer_Pool<list_cell> (LIST_CELL_POOL_CHUNK_SIZE);
		}

		
		return s_list_cell_pool->Get_Next_Pointer();
	}

	void operator delete(void *list_cell_i_dont_need_anymore) 
	{
		
		ASSERT(s_list_cell_pool);

		
		if (s_list_cell_pool == NULL)
			return;

		
		s_list_cell_pool->Release_Pointer(static_cast <list_cell *> (list_cell_i_dont_need_anymore));

		
		if (s_list_cell_pool->There_Are_No_Outstanding_Allocated_Pointers())
		{
			
			delete s_list_cell_pool;

			
			s_list_cell_pool = NULL;
		} 
	}

#endif 


	
	data_ptr cell_data;
	list_cell* next;
	list_cell* prev;

private:
	
	static Pointer_Pool<list_cell> * s_list_cell_pool;
	enum {LIST_CELL_POOL_CHUNK_SIZE = 1000};
};




typedef class linked_list * linked_list_ptr;



class linked_list 
{

public:

	
	inline linked_list() :
		count (0),
		head (NULL),
		tail (NULL),
		last_cell_accessed (NULL),
		last_number_accessed (-999)		
	{
	}

	
	inline ~linked_list()
	{
		Empty();
	}

	// how many entries are there in the list?
	int size() const
	{
		return count;
	}

	
	inline void Empty()
	{
		while (head != NULL)
		{
			list_cell *current_cell = head;
			head = head->next;
			delete current_cell;
		}

		count = 0;
		head = NULL;
		tail = NULL;
		last_cell_accessed = NULL;		
		last_number_accessed = -999;	// Set the number of the last cell accessed to some impossible negative number
	}

#ifndef DONT_USE_POOLS

	void* operator new(size_t);
	void operator delete(void *);

#endif 




	
	inline list_cell_ptr linked_list::Add_Data (data_ptr new_data)
	{
		ASSERT (new_data); 

		
		head = new list_cell (new_data, head, NULL);

		
		if (tail == NULL)
		{
			tail = head;
		}

		
		else
		{
			head->next->prev = head;
		}


		
		last_number_accessed = -999;
		++count;

		
		return head;
	}




	
	inline list_cell_ptr linked_list::Append_Data (data_ptr new_data)
	{
		ASSERT (new_data);

		
		tail = new list_cell(new_data, NULL, tail);

		
		if (head == NULL)
		{
			head = tail;
		}

		
		else
		{
			tail->prev->next = tail;
		}

		
		last_number_accessed = -999;
		++count;

		// return the newly created cell, in the event that the caller is interested
		return tail;
	}




	void Remove_Cell
	(
		list_cell_ptr dead_cell
	);

	int Remove_Data
	(
		data_ptr dead_data
	);

	
	int Remove_Data_Multiple 
	(
		data_ptr dead_data
	);

	
    inline list_cell_ptr Random_Access_Lookup
    (
        int data_num
    )
    {
        int i;
        int df_head;
        int df_point;
        int df_tail;
        int tA, tB, tC;

		list_cell_ptr p;

        df_head = data_num;
        df_point = data_num - last_number_accessed;
        df_tail = count - data_num - 1;

        tA = df_head < ABS(df_point);
        tB = df_head < df_tail;
        tC = ABS(df_point) < df_tail;

        if (tA && tB) {
             
            p = head;
            for(i=0; i<data_num; i++, p = p->next);

        } else if (!tB && !tC) {
             
            p = tail;
            for(i=df_tail; i; i--, p = p->prev);

        } else {
             
            p = last_cell_accessed;
            if (df_point < 0) {
                 
                for(i= -df_point; i; i--, p = p->prev);
            } else {
                 
                for(i=0; i < df_point; i++, p = p->next);
            }
        }

        return(p);
    }

	
	inline data_ptr Return_Data_By_Number
	(
		int data_num
	)
	{
		
		list_cell_ptr current_cell;
		

		if ((data_num <0) || (data_num >= count))
			return(NULL);

		

		
		if (data_num == (last_number_accessed - 1)) 
		{
			current_cell = last_cell_accessed->prev;

		} 

		
		else if (data_num == (last_number_accessed + 1)) 
		{
			current_cell = last_cell_accessed->next;

		} 

		
		else if (data_num == (count-1))
		{
			current_cell = tail;
		} 

		
		else 
		{
            current_cell = Random_Access_Lookup(data_num);

		}

		
		last_number_accessed = data_num;
		last_cell_accessed = current_cell;


		return(current_cell->cell_data);
	};
	


	int Already_In_List
	(
		data_ptr check_data
	);

	list_cell_ptr Return_Cell_By_Number
	(
		int cell_num
	);

	
	int Return_Index_Of_Data 
	(
		data_ptr search_data
	);

	
	void Append_Whole_List(linked_list * other_list);

	
	void Append_Whole_List(const linked_list &other_list)
	{
		for (list_cell_ptr other_cell (other_list.head); other_cell; other_cell = other_cell->next)
		{
			
			Append_Data(other_cell->cell_data);
		}
	}


#ifdef _DEBUG
	void Validate(); 
#endif _DEBUG


	
	int count;			
	list_cell_ptr head;	

private:

	int last_number_accessed;
	list_cell_ptr last_cell_accessed;
	list_cell_ptr tail;

	
	static Pointer_Pool<linked_list> * s_linked_list_pool;

	friend list_cell;
};





class unique_ID_list
{
public:

	
	unique_ID_list() :
		m_next_ID (0)
	{
	}

	
	int Add_Data(data_ptr new_data)
	{
		m_data[m_next_ID] = new_data;
		return m_next_ID++;
	}

	
	void Remove_By_ID (int id)
	{
		m_data.erase (id);
	}

	
	data_ptr Return_Data_By_ID (int num) const
	{
		IdListType::const_iterator i = m_data.find (num);
		if (i != m_data.end()) 
			return i->second;
		else
			return NULL;
	}

	
	int size() const
	{
		return m_data.size();
	}

	
	typedef void (*f) (int id);
	void for_each (f function)
	{
		for (IdListType::reverse_iterator i = m_data.rbegin(); i != m_data.rend(); ++i)
		{
			function (i->first);
		}
	}

private:

	int m_next_ID;
	typedef std::map <int, data_ptr> IdListType;
	IdListType m_data;
};



#endif 
