#ifndef __MATH3D_H__
#define __MATH3D_H__




#define __MULTIPLY_INLINE__

#include <algorithm>
#include "math_scalar.h"
#include "LMiscMath.h"



Matrix34 SetRollingBall(scalar dx, scalar dy, scalar radius);


Matrix34 Spinner(const Matrix34 &mat, const Vector3 &omega, float dt);

Matrix34 Spinner(const Matrix34 &mat, const Vector3 &deltaAng);

//------------------------------------------------------------
// Vector Utilities
//------------------------------------------------------------


#if _ATVI_HIGH_PRECISION_

inline Vector3Double operator* (const Vector3Double &v, const scalar_double a)
{
	Vector3Double res;

	res.x = v.x * a;
	res.y = v.y * a;
	res.z = v.z * a;

	return res;
}

inline Vector3Double operator* (const scalar_double a, const Vector3Double &v)
{
	return v * a;
}

inline scalar_double operator* (const Vector3Double &v1, const Vector3Double &v2)
{
	return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

inline Vector3Double operator/ (const Vector3Double &v, const scalar_double a)
{
	return v * (1.0 / a);
}

inline scalar_double GetLength (const Vector3Double &v) 
{
	return sqrt(v * v);
}

inline Vector3Double Normalize (const Vector3Double &v)
{
	ASSERT( v != Zero_Vector3);
	return v / GetLength(v);
}

inline Vector3Double V3DL_GetFront(const Matrix34 &m)
{
	Vector3Double v;

	v.x = (scalar_double)m.front.x;
	v.y = (scalar_double)m.front.y;
	v.z = (scalar_double)m.front.z;
	return v;
}

inline Vector3Double GetPositionDouble(const Matrix34 &m)
{
	Vector3Double v;

   v.x = m.position.x;
   v.y = m.position.y;
   v.z = m.position.z;
	return v;
}

inline void TransformPoint(Vector3Double& result, const Vector3Double &Vector, const Matrix34 &Mat)
{
  Vector3Double result;

   result.x = (Vector.x * Mat.right.x + Vector.y * Mat.up.x + Vector.z * Mat.front.x + Mat.position.x);
   result.y = (Vector.x * Mat.right.y + Vector.y * Mat.up.y + Vector.z * Mat.front.y + Mat.position.y);
   result.z = (Vector.x * Mat.right.z + Vector.y * Mat.up.z + Vector.z * Mat.front.z + Mat.position.z);
}

#endif

inline void GetPosition(const Matrix34 &m, scalar *x, scalar *y, scalar *z)
{
   *x = (scalar)m.position.x;
   *y = (scalar)m.position.y;
   *z = (scalar)m.position.z;
}


void Matrix34SetRotateXYZ(Matrix34 &Mat, float Pitch, float Yaw, float Roll);


void Matrix34GetRotateXYZ(const Matrix34 &mat,float *pPitch,float *pYaw,float *pRoll);

Matrix34 Matrix34SetPositionAndRotation( scalar Pitch, scalar Yaw, scalar Roll, scalar x, scalar y, scalar z);


inline bool operator ==(const Matrix34 &m1, const Matrix34 &m2)
{
	bool result = 
		(
		(m1.right.x - EQUALS_EPSILON <= m2.right.x) &&
		(m1.right.y - EQUALS_EPSILON <= m2.right.y) &&
		(m1.right.z - EQUALS_EPSILON <= m2.
		right.z) &&
		(m1.up.x - EQUALS_EPSILON <= m2.up.x) &&
		(m1.up.y - EQUALS_EPSILON <= m2.up.y) &&
		(m1.up.z - EQUALS_EPSILON <= m2.up.z) &&
		(m1.front.x - EQUALS_EPSILON <= m2.front.x) &&
		(m1.front.y - EQUALS_EPSILON <= m2.front.y) &&
		(m1.front.z - EQUALS_EPSILON <= m2.front.z) &&
		(m1.position.x - EQUALS_EPSILON <= m2.position.x) &&
		(m1.position.y - EQUALS_EPSILON <= m2.position.y) &&
		(m1.position.z - EQUALS_EPSILON <= m2.position.z) &&
		(m1.right.x + EQUALS_EPSILON >= m2.right.x) &&
		(m1.right.y + EQUALS_EPSILON >= m2.right.y) &&
		(m1.right.z + EQUALS_EPSILON >= m2.right.z) &&
		(m1.up.x + EQUALS_EPSILON >= m2.up.x) &&
		(m1.up.y + EQUALS_EPSILON >= m2.up.y) &&
		(m1.up.z + EQUALS_EPSILON >= m2.up.z) &&
		(m1.front.x + EQUALS_EPSILON >= m2.front.x) &&
		(m1.front.y + EQUALS_EPSILON >= m2.front.y) &&
		(m1.front.z + EQUALS_EPSILON >= m2.front.z) &&
		(m1.position.x + EQUALS_EPSILON >= m2.position.x) &&
		(m1.position.y + EQUALS_EPSILON >= m2.position.y) &&
		(m1.position.z + EQUALS_EPSILON >= m2.position.z)
		);

	return result;
}

extern const Matrix34 Identity_Matrix;


inline Vector3 GetFront(const Matrix34 &m)
{
	Vector3 v;

	v.x = m.front.x;
	v.y = m.front.y;
	v.z = m.front.z;
	return v;
}


inline void SetPositionV(Matrix34 &m, const Vector3 &p)
{
   m.position.x = (scalar_double)p.x;
   m.position.y = (scalar_double)p.y;
   m.position.z = (scalar_double)p.z;
}

inline void V3DL_SetPosition(Matrix34 &m, const Vector3Double &p)
{
   m.position.x = p.x;
   m.position.y = p.y;
   m.position.z = p.z;
}

inline void SetPosition(Matrix34 &m, scalar_double x, scalar_double y, scalar_double z)
{
   m.position.x = x;
   m.position.y = y;
   m.position.z = z;
}

inline void SetUp(Matrix34 &m, const Vector3 &p)
{
	m.up.x = p.x;
	m.up.y = p.y;
	m.up.z = p.z;
}

inline void SetRight(Matrix34 &m, const Vector3 &p)
{
	m.right.x = p.x;
	m.right.y = p.y;
	m.right.z = p.z;
}

inline void SetFront(Matrix34 &m, const Vector3 &p)
{
	m.front.x = p.x;
	m.front.y = p.y;
	m.front.z = p.z;
}

inline void Matrix_Set(Matrix34 &mat, const Vector3 &up, const Vector3 &right,
						 const Vector3 &front, const Vector3 &posit)
{
	mat.up.x = up.x;
	mat.up.y = up.y;
	mat.up.z = up.z;

	mat.right.x = right.x;
	mat.right.y = right.y;
	mat.right.z = right.z;

	mat.front.x = front.x;
	mat.front.y = front.y;
	mat.front.z = front.z;

   mat.position.x = (scalar_double)posit.x;
   mat.position.y = (scalar_double)posit.y;
   mat.position.z = (scalar_double)posit.z;
}

inline void MatrixTranspose(Matrix34 &mat) {
	scalar scratch;
	scratch = mat.right.y;
	mat.right.y = mat.up.x;
	mat.up.x = scratch;
	scratch = mat.right.z;
	mat.right.z = mat.front.x;
	mat.front.x = scratch;
	scratch = mat.up.z;
	mat.up.z = mat.front.y;
	mat.front.y = scratch;	
}

inline int ZeroVector(const Vector3 v)	
{
	return (v.x*v.x < E2) && (v.y*v.y < E2) && (v.z*v.z < E2);
}

inline Vector3Double V3DL_Add(Vector3Double A, Vector3Double B)
{
	Vector3Double v;

	v.x = A.x + B.x;
	v.y = A.y + B.y;
	v.z = A.z + B.z;
	return v;
}

inline Vector3Double V3DL_AddMult(Vector3Double A, scalar_double s, Vector3Double B)
{
	Vector3Double v;

	v.x = A.x + s * B.x;
	v.y = A.y + s * B.y;
	v.z = A.z + s * B.z;
	return v;
}

inline Vector3Double V3DL_Expand(Vector3 v1)
{
	Vector3Double v2;

	v2.x = (scalar_double)v1.x;
	v2.y = (scalar_double)v1.y;
	v2.z = (scalar_double)v1.z;
	return v2;
}

inline scalar Get_Quaternion_Angle (QUAT *QA, QUAT *QB)
{
   scalar Dot;
   Dot = QA->n.x * QB->n.x + QA->n.y * QB->n.y + QA->n.z * QB->n.z + QA->w * QB->w;
   if (Dot > 1.0f) return 0.0f;

   return acosf (Dot);
}

inline scalar find_small_ang(scalar a, scalar b)
{
    scalar d;

    d =  b - a;
    if (PI < d)
       return d - PI2;
    else if (d < -PI)
       return d + PI2;
    else
       return d;
}





Matrix34 __fastcall Quaternion_to_Matrix (QUAT *Q);
QUAT __fastcall Matrix_to_Quaternion (Matrix34 *Mat);
Matrix34 __fastcall Matrix_Inverse (const Matrix34 &Matrix);
Vector3Double __fastcall Long_Vector_Transform (const Vector3 &Vector, const Matrix34 &Mat);


Matrix34 __fastcall BuildOrthogonalMatrix( const Vector3 &Up, const Vector3 &Heading);
Matrix34 __fastcall BuildDirectionalMatrix( const Vector3 &Position, const Vector3 &Destination);


inline void Multiply(Matrix34 &result,	const Matrix34 &A, const Matrix34 &B)
{
   result.right.x = A.right.x * B.right.x + A.right.y * B.up.x + A.right.z * B.front.x;
   result.right.y = A.right.x * B.right.y + A.right.y * B.up.y + A.right.z * B.front.y;
   result.right.z = A.right.x * B.right.z + A.right.y * B.up.z + A.right.z * B.front.z;

   result.up.x = A.up.x * B.right.x + A.up.y * B.up.x + A.up.z * B.front.x;
   result.up.y = A.up.x * B.right.y + A.up.y * B.up.y + A.up.z * B.front.y;
   result.up.z = A.up.x * B.right.z + A.up.y * B.up.z + A.up.z * B.front.z;

   result.front.x = A.front.x * B.right.x + A.front.y * B.up.x + A.front.z * B.front.x;
   result.front.y = A.front.x * B.right.y + A.front.y * B.up.y + A.front.z * B.front.y;
   result.front.z = A.front.x * B.right.z + A.front.y * B.up.z + A.front.z * B.front.z;

   result.position.x = A.position.x * B.right.x + A.position.y * B.up.x + A.position.z * B.front.x + B.position.x;
   result.position.y = A.position.x * B.right.y + A.position.y * B.up.y + A.position.z * B.front.y + B.position.y;
   result.position.z = A.position.x * B.right.z + A.position.y * B.up.z + A.position.z * B.front.z + B.position.z;
}

//
// MultiplyOrientation.
// Ignores position, just multiplying the orientation.
//
inline void MultiplyOrientation(Matrix34 &result, const Matrix34 &A, const Matrix34 &B)
{
   result.right.x = A.right.x * B.right.x + A.right.y * B.up.x + A.right.z * B.front.x;
   result.right.y = A.right.x * B.right.y + A.right.y * B.up.y + A.right.z * B.front.y;
   result.right.z = A.right.x * B.right.z + A.right.y * B.up.z + A.right.z * B.front.z;

   result.up.x = A.up.x * B.right.x + A.up.y * B.up.x + A.up.z * B.front.x;
   result.up.y = A.up.x * B.right.y + A.up.y * B.up.y + A.up.z * B.front.y;
   result.up.z = A.up.x * B.right.z + A.up.y * B.up.z + A.up.z * B.front.z;

   result.front.x = A.front.x * B.right.x + A.front.y * B.up.x + A.front.z * B.front.x;
   result.front.y = A.front.x * B.right.y + A.front.y * B.up.y + A.front.z * B.front.y;
   result.front.z = A.front.x * B.right.z + A.front.y * B.up.z + A.front.z * B.front.z;
}


inline void TransposeMultiplyOrientation(Matrix34 *pR,const Matrix34 &a,const Matrix34 &b)
{
	Matrix34 atmp = a;
	MatrixTranspose(atmp);
	MultiplyOrientation(*pR,atmp,b);


 }

inline Matrix34 operator* (const Matrix34 &m1, const Matrix34 &m2)
{
	Matrix34 tmp;
	Multiply(tmp, m1,m2);
	return tmp;
}


const Vector3 Zero_Vector3 = Vector3(0, 0, 0);
const Sphere Zero_Sphere(Zero_Vector3, 0.0f);

extern const Matrix34 Identity_Matrix;

inline Matrix34 Matrix34_Build_Translation(Vector3 translation)
{
	Matrix34 m ;
	m = Identity_Matrix ;
	m.position.x = translation.x ;
	m.position.y = translation.y ;
	m.position.z = translation.z ;

	return m ;
}


inline Matrix34 GetRotation(const Matrix34 &m)
{
    Matrix34 tmp ;
    
    tmp = m ;
    tmp.position.x = 0.0f ;
    tmp.position.y = 0.0f ;
    tmp.position.z = 0.0f ;

    return tmp ;
}

inline Matrix34 GetTranslation(const Matrix34 &m)
{
    Matrix34 tmp = Identity_Matrix ;

    tmp.position.x = m.position.x ;
    tmp.position.y = m.position.y ;
    tmp.position.z = m.position.z ;

    return tmp ;
}


inline scalar GetLength(const Vector3 &v) 
{
    return (scalar) sqrt(v.x*v.x + v.y*v.y + v.z*v.z);
}


inline scalar GetLengthSquared(const Vector3 &v) 
{
    return (v.x*v.x) + (v.y*v.y) + (v.z*v.z); 
}

inline scalar Get2DLengthSquared(const Vector3 &v)
{
	return v.x*v.x + v.z*v.z;
}




inline scalar GetDistanceSquared(const Vector3 &v1, const Vector3 &v2) 
{ 
    scalar diff = v1.x - v2.x ;
	scalar total = diff * diff;
	diff = v1.y - v2.y ;
	total += diff * diff;
    diff = v1.z - v2.z ;
 
    return total + diff*diff;
}

inline scalar GetDistance(const Vector3 &v1, const Vector3 &v2)
{
	

	

	double val = GetDistanceSquared(v1, v2);

	if (val <= 0)
		return 0;
	else 
		return sqrt(val);
}



inline bool operator == (const Vector3 &v1, const Vector3 &v2)
{
	return v1.x == v2.x && v1.y == v2.y && v1.z == v2.z;
}

inline bool operator != (const Vector3 &v1, const Vector3 &v2)
{
	return v1.x != v2.x || v1.y != v2.y || v1.z != v2.z;
}

// Equality operator
inline bool operator == (const Vector2 &v1, const Vector2 &v2)
{
	return v1.u == v2.u && v1.v == v2.v ;
}


inline scalar operator* (const Vector3 &v1, const Vector3 &v2)
{
	return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

#if _ATVI_HIGH_PRECISION_
inline Vector3Double operator+ (const Vector3Double &v1, const Vector3Double &v2)
{
	Vector3Double v(v1);
	v += v2;
	return v;
}

inline Vector3Double operator- (const Vector3Double &v1, const Vector3Double &v2)
{
	Vector3Double res;

	res.x = v1.x - v2.x;
	res.y = v1.y - v2.y;
	res.z = v1.z - v2.z;

	return res;
}

inline Vector3Double operator- (Vector3Double v)
{
	v.x = -v.x;
	v.y = -v.y;
	v.z = -v.z;
	return v;
}

inline Vector3Double UntransformDirection(const Matrix34 &Mat, const Vector3Double &Vector)
{
   Vector3Double result;

   result.x = Vector.x * Mat.right.x + Vector.y * Mat.right.y + Vector.z * Mat.right.z;
   result.y = Vector.x * Mat.up.x    + Vector.y * Mat.up.y +    Vector.z * Mat.up.z;
   result.z = Vector.x * Mat.front.x + Vector.y * Mat.front.y + Vector.z * Mat.front.z;
   return result;
}

inline Vector3Double UntransformPoint(const Matrix34 &Mat, Vector3Double v)
{
	v.x -= Mat.position.x;
	v.y -= Mat.position.y;
	v.z -= Mat.position.z;

	return UntransformDirection(Mat, v);
}

inline void operator *= (Vector3Double &vec, const scalar_double s)
{
	vec.x *= s;
	vec.y *= s;
	vec.z *= s;
}
#endif

inline Vector3 operator- (const Vector3 &v)
{
	Vector3 res;
	res.x = -v.x;
	res.y = -v.y;
	res.z = -v.z;
	return res;
}


inline Vector3 operator* (const Vector3 &v, const scalar a)
{
	Vector3 vr;
	vr.x = v.x * a;
	vr.y = v.y * a;
	vr.z = v.z * a;
	return vr;
}

// "scaler * vector"
inline Vector3 operator* (const scalar a, const Vector3 &v)
{
	return v * a;
}



inline void operator *= (Vector3 &v, const scalar a)
{
	v.x *= a;
	v.y *= a;
	v.z *= a;
}


inline Vector3 operator/ (const Vector3 &v, const scalar a)
{
	Vector3 res;
	
	res.x = v.x / a;
	res.y = v.y / a;
	res.z = v.z / a;

	return res;
}

inline void MakeZero(Vector3 &v)
{
	v.x = v.y = v.z = 0;
}


inline scalar GetAbsNorm(const Vector3 &v)
{
	return (scalar) (fabs(v.x) + fabs(v.y) + fabs(v.z));
}


inline scalar Get2DAbsNorm(const Vector3 &v)
{
	return (scalar) (fabs(v.x) + fabs(v.z));
}



inline Vector3 RemoveComponentAlong(const Vector3 &vec, const Vector3 &direction)
{
	
	const scalar norm_direction_squared = GetLengthSquared(direction);
	if (norm_direction_squared < 0.0001) 
		return vec;

	
	return vec - ((vec * direction) / norm_direction_squared) * direction;
}


inline Vector3 SetVector(scalar x, scalar y, scalar z)
{
	Vector3 v;

	v.x = x; v.y = y; v.z = z;
	return v;
}


inline Vector3 ScaleVector(scalar s, Vector3 v)
{
	v.x *= s; v.y *= s; v.z *= s;
	return v;
}


inline Vector3 AddVectors(const Vector3 &A, const Vector3 &B)
{
	Vector3 v;

	v.x = A.x + B.x;
	v.y = A.y + B.y;
	v.z = A.z + B.z;
	return v;
}

inline Vector3 SubVectors(const Vector3 &A, const Vector3 &B)
{
	Vector3 v;

	v.x = A.x - B.x;
	v.y = A.y - B.y;
	v.z = A.z - B.z;
	return v;
}

inline Vector3 AddMultVectors(const Vector3 &A, scalar s, const Vector3 &B)
{
	Vector3 v;

	v.x = A.x + s * B.x;
	v.y = A.y + s * B.y;
	v.z = A.z + s * B.z;
	return v;
}

inline Vector3 CombineVectors(scalar a, const Vector3 &A, scalar b, const Vector3 &B)
{
	Vector3 v;

	v.x = a * A.x + b * B.x;
	v.y = a * A.y + b * B.y;
	v.z = a * A.z + b * B.z;
	return v;
}



inline float UnsignedAngleBetweenVectors(const Vector3 &a,const Vector3 &b)
{
	

	
	if(a == Zero_Vector3 || b == Zero_Vector3)
	{
		//if one of these vectors is the zero vector, return 0.
		return 0;
	}

	
	float a_dot_b = (a*b);

	// What is the sign of the dot product of a and b?
	float a_dot_b_sign = 1.0;

	
	if (a_dot_b < 0)
		a_dot_b_sign = -1.0;

	
	const float val = a_dot_b_sign * sqrt( ((a_dot_b)*(a_dot_b)) / ((a*a)*(b*b)));
	if (val <= -1.0f)
		return PI;
	else if (val >= 1.0f)
		return 0;
	return acos(val);
}


inline float SafePitchFromVector(const Vector3 &a)
{
	
	return atan2( a.y , sqrt(a.z*a.z + a.x*a.x) );
}


inline scalar DotProduct( const Vector3 &v1, const Vector3 &v2)
{
   return v1.x * v2.x + v1.y * v2.y + v1.z * v2.z;
}

inline Vector3 CrossProduct( const Vector3 &A, const Vector3 &B)
{
   Vector3 Dst;

   Dst.x = A.y * B.z - A.z * B.y;
   Dst.y = A.z * B.x - A.x * B.z;
   Dst.z = A.x * B.y - A.y * B.x;
   return Dst;
}



inline const Vector3 UpVector(const Matrix34 &m)
{
	return m.up;
}

inline const Vector3 FrontVector(const Matrix34 &m)
{
	return m.front;
}

inline const Vector3 RightVector(const Matrix34 &m)
{
	return m.right;
}

inline const Vector3 &GetPosition(const Matrix34 &m)
{
	return m.position;
}


inline Vector3 Normalize(const Vector3& a)
{
	

	
	if(!IsZero(a.x) || !IsZero(a.y) || !IsZero(a.z))
	{
		const float mag2 = a.x*a.x + a.y*a.y + a.z*a.z;
		return a/sqrt(mag2);
	}
	else
	{
		
		return Zero_Vector3;
	}
}



inline Vector3 SafeNormalize(const Vector3& a,const float epsilon)
{
	const float mag2 = a.x*a.x + a.y*a.y + a.z*a.z;
	if (mag2 < epsilon)
		return Vector3(1,0,0);
	return a/sqrt(mag2);
}



inline void TransformPoint(Vec3f &result, const Vec3f &point, const Matrix34 &transform)
{
   result.x = scalar(point.x * transform.right.x + point.y * transform.up.x + point.z * transform.front.x + transform.position.x);
   result.y = scalar(point.x * transform.right.y + point.y * transform.up.y + point.z * transform.front.y + transform.position.y);
   result.z = scalar(point.x * transform.right.z + point.y * transform.up.z + point.z * transform.front.z + transform.position.z);
}


inline void TransformPoint(Vec3f *result, const Vec3f &point, const Matrix34 &transform)
{
   result->x = scalar(point.x * transform.right.x + point.y * transform.up.x + point.z * transform.front.x + transform.position.x);
   result->y = scalar(point.x * transform.right.y + point.y * transform.up.y + point.z * transform.front.y + transform.position.y);
   result->z = scalar(point.x * transform.right.z + point.y * transform.up.z + point.z * transform.front.z + transform.position.z);
}


inline Vector3 TransformDirection(const Matrix34 &Mat, const Vector3 &Vector)
{
  Vector3 result;

   result.x = Vector.x * Mat.right.x + Vector.y * Mat.up.x + Vector.z * Mat.front.x;
   result.y = Vector.x * Mat.right.y + Vector.y * Mat.up.y + Vector.z * Mat.front.y;
   result.z = Vector.x * Mat.right.z + Vector.y * Mat.up.z + Vector.z * Mat.front.z;
   return result;
}

inline Vector3 UntransformDirection(const Matrix34 &Mat, const Vector3 &Vector)
{
   Vector3 result;

   result.x = Vector.x * Mat.right.x + Vector.y * Mat.right.y + Vector.z * Mat.right.z;
   result.y = Vector.x * Mat.up.x    + Vector.y * Mat.up.y +    Vector.z * Mat.up.z;
   result.z = Vector.x * Mat.front.x + Vector.y * Mat.front.y + Vector.z * Mat.front.z;
   return result;
}

inline Vector3 UntransformPoint(const Matrix34 &transform, Vector3 v)
{
	v -= transform.position;
	return UntransformDirection(transform, v);
}

inline void ExpandBoundingBox(BBox &bounding_box, const Vector3 &v)
{
	
	bounding_box.min.x = MIN(bounding_box.min.x, v.x);
	bounding_box.min.y = MIN(bounding_box.min.y, v.y);
	bounding_box.min.z = MIN(bounding_box.min.z, v.z);

	
	bounding_box.max.x = MAX(bounding_box.max.x, v.x);
	bounding_box.max.y = MAX(bounding_box.max.y, v.y);
	bounding_box.max.z = MAX(bounding_box.max.z, v.z);
}

inline bool SphereIntersection(const Sphere &a, const Sphere &b)
{
	const float distance_squared = GetDistanceSquared(a.origin, b.origin);

	return distance_squared < (a.radius + b.radius) * (a.radius + b.radius);
}



Matrix34 Build_Axis_Rotation_Matrix (scalar Angle, scalar x, scalar y, scalar z);
QUAT     Slerp(QUAT *A, QUAT *B, scalar t);

Vector3 Normal_Surface(Vector3 *A, Vector3 *B, Vector3 *C);


Matrix34 Build_Pitch_Yaw_Matrix(scalar Pitch, scalar Yaw);


Matrix34 Build_Yaw_Matrix(scalar yaw);
Matrix34 Build_Pitch_Matrix(scalar pitch);
Matrix34 Build_Roll_Matrix(scalar roll);

void TransformPoints(Vector3 *transformed_points, const Vector3 *untransformed_points, long npoints, const Matrix34 &transform);
bool IsOrthogonal(const Matrix34 &mat);
bool IsScaledOrthogonal(const Matrix34 &mat, float *pScale);
bool IsScaledOrthogonal(const Matrix34 &mat, Vector3 *pScale);


Vector3 FindIntersection(const Vector3 &a, const Vector3 &b, const Vector3 &c, const Vector3 &d );
Vector2 FindIntersection(Vector2 &a, Vector2 &b, Vector2 &c, Vector2 &d);

Plane Transform(const Matrix34 &Matrix, const Plane &plane);
Plane PlaneEquation(const Vector3 &A, const Vector3 &B, const Vector3 &C);

Plane PlaneEquationNormalize(const Vector3 &A, const Vector3 &B, const Vector3 &C);


// Renormalize a matrix based on the front and right vectors.
// Assumes front and right are non degenerate and have nondegenerate
// cross product.  Ignores position.
inline void RenormalizeMatrix(Matrix34& b,const Matrix34& a)
{
	b.front = Normalize(a.front);
	b.up = Normalize(CrossProduct(b.front,a.right));
	b.right = CrossProduct(b.up,b.front);
}

bool SegmentIntersectsSphere(const Vector3 &p0, const Vector3 &p1, const Sphere &s);


inline float SegmentIntersectsPlane(const Vector3 &source, const Vector3 &dest, const Plane &plane)
{
	float r0 = plane.normal * source + plane.offset;
	float r1 = plane.normal * dest + plane.offset;
	if ((r0 < 0.0f) || (r1 > 0.0f))
		return -1;

	
	if (r0 == r1)
		return 0;

	
	float r = r0 / (r0 - r1);
	ASSERT((r>=0) && (r<=1.0));
	return r;
}


inline bool PointInTriangle(const Vector3 &test, const Vector3 &p0, const Vector3 &p1, const Vector3 &p2, const Vector3 &normal)
{
	if (DotProduct(normal, CrossProduct(test - p0, p1 - p0)) < 0)
		return false;

	if (DotProduct(normal, CrossProduct(test - p1, p2 - p1)) < 0)
		return false;

	if (DotProduct(normal, CrossProduct(test - p2, p0 - p2)) < 0)
		return false;

	return true;
}




template <class T>
inline T NormalizeRadian0_2Pi(T rad)
{
	// I mean, really.  You've got a problem.
	ASSERT(rad<1000 && rad>-1000);

	const T k2Pi = T(6.28318530718);
	while(rad < 0)
		rad += k2Pi;
	while(rad > k2Pi)
		rad -= k2Pi;
	return rad;
}

template <class T>
inline T NormalizeRadianNegPi_Pi(T rad)
{
	ASSERT(rad<1000 && rad>-1000);

	const T kPi = T(3.141592653589f);
	const T k2Pi = 2*kPi;
	while(rad < -kPi)
		rad += k2Pi;
	while(rad > kPi)
		rad -= k2Pi;
	return rad;
}


template <class T>
inline T NormalizeRadianRelativeNegPi_Pi(T rad, T relativeTo)
{
	return relativeTo + NormalizeRadianNegPi_Pi(rad - relativeTo);
}

#endif 
