#ifndef __MATH_TYPES_H__
#define __MATH_TYPES_H__





#include "Vec3f.h"

#include "Vec2f.h"

template<typename Type>
Type MIN(Type a, Type b) {
	if( a < b )
		return a;
	return b;
}

template<typename Type>
Type MAX(Type a, Type b) {
	if( a < b )
		return b;
	return a;
}


#define _ATVI_HIGH_PRECISION_ 0


typedef signed char			sint8;
typedef signed short int	sint16;
typedef signed long int		sint32;

typedef signed char			int8;
typedef signed short int	int16;
typedef signed long int		int32;

typedef unsigned int		uint;
typedef unsigned char		uint8;
typedef unsigned short int	uint16;
typedef unsigned long int	uint32;


typedef float				scalar;


typedef float				Angle;



class Vector2 
{
public:
	Vector2()
	{
	}
	
	Vector2(float a, float b):u(a),v(b)
	{
	}

	// Saves typing, guaranteed fast
	Vector2& Set(float a,float b) {
		u = a;
		v = b;
		return *this;
	}
	
	Vector2(Vec2f in_vec)
	{
		u = in_vec.x;
		v = in_vec.y;
	}

	const Vector2& operator+=(const Vector2& rhs)
	{
		u += rhs.u; v += rhs.v; return *this;
	}
	
	const Vector2& operator-=(const Vector2& rhs)
	{
		u -= rhs.u; v -= rhs.v; return *this;
	}

	inline scalar &operator [](int i)
	{
		return (&u)[i];
	}

	inline scalar operator [](int i) const
	{
		return (&u)[i];
	}

	scalar u, v; 
};

#if _ATVI_HIGH_PRECISION_
class Vector3Double;
typedef double scalar_double;
#endif

class Vector3 : public Vec3f
{
public:
	Vector3()
	{
	}

	Vector3(float inX, float inY, float inZ) : Vec3f(inX, inY, inZ)
	{
	}

	Vector3(Vec3f& inVal) : Vec3f(inVal)
	{
	}
	

	
	void Set(scalar a,scalar b,scalar c)
	{
		x = a;
		y = b;
		z = c;
	}

	const Vector3& operator+=(const Vector3& rhs)
	{
		x += rhs.x; y+= rhs.y; z += rhs.z; return *this;
	}

	const Vector3& operator-=(const Vector3& rhs)
	{
		x -= rhs.x; y-= rhs.y; z -= rhs.z; return *this;
	}

	// element wise
	const Vector3& operator*=(const Vector3& rhs)
	{
		x *= rhs.x; y*= rhs.y; z *= rhs.z; return *this;
	}

	
	const Vector3& operator/=(const Vector3& rhs)
	{
		x /= rhs.x; y/= rhs.y; z /= rhs.z; return *this;
	}

	inline scalar &operator [](int i)
	{
		return (&x)[i];
	}

	inline scalar operator [](int i) const
	{
		return (&x)[i];
	}

#if _ATVI_HIGH_PRECISION_
	explicit Vector3(const Vector3Double &rhs);
#endif
};

class Vector4 
{ 
public:
	Vector4()
	{
	}

	Vector4( const Vector3& v3 , float fW )
	{
		x = v3.x;
		y = v3.y;
		z = v3.z;
		w = fW;
	}

	Vector4(scalar a, scalar b, scalar c, scalar d):x(a),y(b),z(c),w(d)
	{
	}

	const Vector4& operator+=(const Vector4& rhs)
	{
		x += rhs.x; y+= rhs.y; z += rhs.z; w += rhs.w; return *this;
	}

	const Vector4& operator-=(const Vector4& rhs)
	{
		x -= rhs.x; y-= rhs.y; z -= rhs.z; w -= rhs.w; return *this;
	}

	inline scalar &operator [](int i)
	{
		return (&x)[i];
	}

	inline scalar operator [](int i) const
	{
		return (&x)[i];
	}

	scalar x, y, z, w; 
};


#if _ATVI_HIGH_PRECISION_
class Vector3Double 
{ 
public : 
	Vector3Double(){};
	Vector3Double(scalar_double a, scalar_double b, scalar_double c):x(a),y(b),z(c){}
	explicit Vector3Double(const Vector3 &rhs) :x(rhs.x), y(rhs.y), z(rhs.z){}
	
	const Vector3Double& operator+=(const Vector3Double& rhs){x += rhs.x; y+= rhs.y; z += rhs.z; return *this;}
	const Vector3Double& operator+=(const Vector3& rhs){x += rhs.x; y+= rhs.y; z += rhs.z; return *this;}
	const Vector3Double& operator-=(const Vector3Double& rhs){x -= rhs.x; y-= rhs.y; z -= rhs.z; return *this;}
	const Vector3Double& operator-=(const Vector3& rhs){x -= rhs.x; y-= rhs.y; z -= rhs.z; return *this;}
	double x, y, z; 
};
inline Vector3 ToVector3(const Vector3Double &vector_double)
{
	return Vector3(vector_double.x, vector_double.y, vector_double.z);
}

inline Vector3Double ToVector3Double(const Vector3 &vector_float)
{
	Vector3Double v(double(vector_float.x), double(vector_float.y), double(vector_float.z));
	return v;
}

inline Vector3::Vector3(const Vector3Double &rhs) : x(rhs.x), y(rhs.y), z(rhs.z){}

#else
#define ToVector3Double(x) (x)
#define ToVector3(x) (x)
#define GetPositionDouble GetPosition
typedef Vector3 Vector3Double;
typedef float scalar_double;
#endif


class Vector2Double { public: double u, v; };
class Vector4Double { public: double x, y, z, w; };





class Matrix34
{
public:
   Matrix34(const Vector3 &r, const Vector3 &u, const Vector3 &f, const Vector3 &p) 
			{ right=r;up=u;front=f;position=p;}

   Matrix34(){}
   
   Vector3 right ; 
   Vector3 up ;
   Vector3 front ;
   Vector3 position ;  
}; 



class Mad3D_Matrix23
{
public:
    void ApplyToPoint(Vector2 *dest, Vector2 *src);
    scalar m[3][2];
};


class Matrix33
{
public:
	Vector3 m[3];
};



class Matrix44 
{
public:
    Matrix44() {}
	Matrix44(const Matrix34 &mat);

	void SetPerspectiveMatrix( bool direct3d_projection_matrix, scalar fFOV, scalar fAspect,
                               float fNearPlane, float fFarPlane );
	void SetOrthographicMatrix( bool direct3d_projection_matrix, float fWidth, float fHeight,
                                float fNearPlane, float fFarPlane );
	void SetIdentity();

	scalar m[4][4];
};




class Plane
{
public:
	Plane() {}
	Plane(const Vector3 &_normal, float _offset)
	{
		normal = _normal ; offset = _offset; 
	}
	Plane(float a, float b, float c, float d)
	{
		normal.x = a; 
		normal.y = b; 
		normal.z = c; 
		offset = d;
	}

	Plane(const Vector3 &inNormal, const Vector3 &position) 
	{
		normal = inNormal;
		offset = -normal.Dot(position);
	}

	bool operator==( const Plane& rhs )
	{
		return normal == rhs.normal && offset == rhs.offset;
	}

	bool RayIntersection(const Vec3f &p, const Vec3f &dir, Vec3f *pOut)
	{
		
		float denominator = normal.Dot(dir);

		
		if (denominator == 0)
			return false;

		
		float numerator = normal.Dot(p) + offset;

		
		float m = -numerator / denominator; 

		
		*pOut = p + dir*m;	
		return true;
	}

	Vector3 normal;
	float   offset;
};


class BBox
{
public:
	
	BBox() 
	{}

	
	BBox(const Vector3 &mn, const Vector3 &mx)
	{
		min = mn;
		max = mx;
	}

	
	bool InsideInclusive(const Vector3 &pos) const
	{
		return ((min.x <= pos.x) 
			&& (pos.x <= max.x) 
			&& (min.y <= pos.y) 
			&& (pos.y <= max.y)
			&& (min.z <= pos.z) 
			&& (pos.z <= max.z));
	}

	
	bool InsideExclusive(const Vector3 &pos) const
	{
		return ((min.x < pos.x) 
			&& (pos.x < max.x) 
			&& (min.y < pos.y) 
			&& (pos.y < max.y)
			&& (min.z < pos.z) 
			&& (pos.z < max.z));
	}

	Vector3 min;			
	Vector3 max;			
};				


class BBox2f
{
public:
	//default constructor.
	BBox2f()
	{}

	BBox2f(const Vector2 &mn, const Vector2 &mx)
	{
		min = mn;
		max = mx;
	}
	BBox2f(float minx,float miny,float maxx,float maxy)
	{
		min.u = minx;
		min.v = miny;
		max.u = maxx;
		max.v = maxy;
	}
	Vector2 min;
	Vector2 max;
};

// An int bounding rectangle.
// Ususally considered exclusive, 
// ie, without the right and bottom.
class BBox2i
{
	int left;
	int top;
	int right;
	int bottom;
};

		

class Sphere 
{
public:
	Sphere(){};
	Sphere(const Vector3 &_origin, float _radius) 
		: origin(_origin)
		, radius(_radius) 
	{}

	Vector3 origin;		
	scalar	radius;		
};


class Rect2f
{
public:
	Rect2f() 
	{
		x = y = width = height = 0.0f;
	}

	Rect2f(float _x, float _y, float _w, float _h) 
		: x(_x)
		, y(_y)
		, width(_w)
		, height(_h)
		
	{ 
	}

	Rect2f(const Vector2 &origin, const Vector2 &size) 
		: x(origin.u)
		, y(origin.v)
		, width(size.u)
		, height(size.v)
	{ 
	}

	// Nice to have, and I think it compiles down better than
	// assignment to a temp local.
	void Set(float _x, float _y, float _w, float _h) {
		x = _x;
		y = _y;
		width = _w;
		height = _h;
	}

	void Set(const Rect2f& _rect)
	{
		x = _rect.x;
		y = _rect.y;
		width = _rect.width;
		height = _rect.height;
	}

	bool operator==(const Rect2f &rhs) const
	{
		return x == rhs.x 
			&& y == rhs.y
			&& width == rhs.width
			&& height == rhs.height;
	}

	bool Intersects( const Rect2f& r ) const
	{	
		return x+width  >= r.x  &&  r.x+r.width  >= x
			&& y+height >= r.y  &&  r.y+r.height >= y;
	}

	
	bool Intersection(Rect2f& outRect, const Rect2f& r)
	{
		if (!Intersects(r))
			return false;

		outRect.x = MAX(x, r.x);
		outRect.width = MIN(x + width, r.x + r.width) - outRect.x;
		outRect.y = MAX(y, r.y);
		outRect.height = MIN(y + height, r.y + r.height) - outRect.y;

		return true;
	}

	
	bool Inside(const Vector2 &pos) const
	{
		return pos.u >= x 
			&& pos.u <= (x + width) 
			&& pos.v >= y
			&& pos.v <= (y+ height);
	}

	
	bool Clip(const Rect2f &clip_rect);

	
	void Expand(const Rect2f &expandRect)
	{
		float right = MAX(x + width, expandRect.x + expandRect.width);
		float bottom = MAX(y + height, expandRect.y + expandRect.height);

		x = MIN(x, expandRect.x);
		y = MIN(y, expandRect.y);

		width = right - x;
		height = bottom - y;
	}
	
	
	virtual Vec2f GetCenter(void) const
	{
		return Vec2f(x + width / 2.0f, y + height / 2.0f);
	}

	
	Vector2 & XYRef() {
		return *((Vector2*)&x);
	}

	
	Vector2 & WHRef() {
		return *((Vector2*)&width);
	}

	
	const Vector2 & XY() const {
		return *((Vector2*)&x);
	}

	const Vector2 & WH() const {
		return *((Vector2*)&width);
	}

    float x; 
	float y;
	float width;
	float height;
};



 



#define inline_ASM_STANDARDS__

#define Round_Nearest  0x0000	
#define Round_Down     0x0400	
#define Round_Up       0x0800	
#define Round_Chop     0x0c00	

#define Precision_24   0x0000	
#define Precision_xx   0x0100	
#define Precision_53   0x0200	
#define Precision_64   0x0300	




 typedef struct {
   float Sin;
   float Cos;
} SINCOS;

typedef struct {
   Vector3 n;
   float w;
} QUAT;



#define Fabs(Val) ((float) fabs ((float)(Val)))
#define Sqrt(Val) sqrt(Val)
SINCOS SinCos (float Angle);

double rsqrt (double x);


long Set_Float_Exceptions_Mask  (long mask);


inline float sign (float a)
{
   if (a == 0.0f) return 0.0f;
   else if (a > 0.0f) return 1.0f;
   else return -1.0f;
}


int __fastcall Get_RoundingAndPrecision();
int __fastcall Set_Rounding(int Round);
int __fastcall Set_RoundingAndPrecision(int RoundPrecision);


class FPUGuardian
{
public:
	FPUGuardian() :
	  m_fpu_initial_state (Get_RoundingAndPrecision())
	{
	}

	~FPUGuardian()
	{
		Set_RoundingAndPrecision(m_fpu_initial_state);
	}
private:
	const int m_fpu_initial_state;
};

#endif 
