

#ifndef __Pointer_Pool_h__
#define __Pointer_Pool_h__

#include "list_array.h"

template <class DATA_TYPE> class list_array;



#define INITIAL_CHUNK_LIST_SIZE (30)
const int BYTE_ALIGNMENT = 4;


template <class DATA_TYPE>
class Pool_Element
{
	public:

	DATA_TYPE data;



#ifdef _DEBUG
	
	bool in_use;
#endif

};





template <class DATA_TYPE>
class Pointer_Pool
{
	typedef Pool_Element<DATA_TYPE> A_Pool_Element;

	public:

		
		Pointer_Pool
		(
			int i_chunk_size,			// How much to grow array by
			int i_max_chunks = -1		
		);


		
		~Pointer_Pool();


		///////////////////////////////////////////////////////////////////////
		//
		// Get_Next_Pointer
		//
		// Grab the ptr to the next available element
		//
		DATA_TYPE * Get_Next_Pointer();

		
		void Release_Pointer
		(
			DATA_TYPE * ptr_to_free
		);


		
		bool There_Are_No_Outstanding_Allocated_Pointers()
		{
			
			if (count == 0)
				return true;
			else
				return false;
		}


	protected:

		
		bool Prepare_New_Chunk();

		int chunk_size;					
		int max_chunks;					
		int count;						


		A_Pool_Element *next_element;	

		
		list_array<A_Pool_Element *> * chunks;

};





template <class DATA_TYPE>
bool Pointer_Pool<DATA_TYPE>::Prepare_New_Chunk()
{
	
	A_Pool_Element *new_chunk;				
	int bytes_to_allocate;				
	int size_of_data_type;				
	int i;								
	A_Pool_Element *next_ptr_in_list;		

	
	if (chunks->count == max_chunks) return false;

	
	size_of_data_type = sizeof(A_Pool_Element);

	
	if (BYTE_ALIGNMENT > 0)
	{
		
		size_of_data_type = ((size_of_data_type + (BYTE_ALIGNMENT-1))/
			BYTE_ALIGNMENT) * BYTE_ALIGNMENT;

	} 

	
	bytes_to_allocate = size_of_data_type * chunk_size;

	
	new_chunk = (A_Pool_Element *) new char[bytes_to_allocate];

	
	chunks->Append_Data(new_chunk);

	
	next_ptr_in_list = next_element;

	
	for (i = 0; i < chunk_size; i++)
	{
		
		next_element = &(new_chunk[i]);

		
		*((A_Pool_Element **) next_element) = next_ptr_in_list;


#ifdef _DEBUG
		// Initially, no pointer in chunk are in use
		next_element->in_use = false;
#endif

		
		next_ptr_in_list = next_element;

	} 

	// We made a good chunk
	return true;
}





template <class DATA_TYPE>
Pointer_Pool<DATA_TYPE>::Pointer_Pool
(
	int i_chunk_size,					
	int i_max_chunks					
)
{
	

	
	ASSERT(i_chunk_size > 0);



	// Assign the initialization values
	chunk_size = i_chunk_size;
	max_chunks = i_max_chunks;

	
	count = 0;

	// Allocate the list of chunks
	chunks = new list_array<A_Pool_Element *> (INITIAL_CHUNK_LIST_SIZE);
	
	
	next_element = NULL;

	
	Prepare_New_Chunk();

}





template <class DATA_TYPE>
Pointer_Pool<DATA_TYPE>::~Pointer_Pool() 
{
	
	A_Pool_Element *bad_chunk;				
	int i;								

	
	if (count != 0)
	{
		
		WARN(("Un-released objects still in pool (%d objects)", 
			count));

	} 

	
	if (chunks != NULL)
	{
		
		for (i = 0; i < chunks->count; i++)
		{
			
			bad_chunk = chunks->Return_Data_By_Number(i);

			
			delete[] reinterpret_cast <char *> (bad_chunk);
			bad_chunk = NULL;
		}

		delete chunks;
		chunks = NULL; 
	}

}





template <class DATA_TYPE>
DATA_TYPE * Pointer_Pool<DATA_TYPE>::Get_Next_Pointer() 
{
	
	A_Pool_Element *the_new_pointer;			// Which element are we grabbing?
	///////////////////////////////////////////////////////////////////////////

	
	count++;

	
	if (count > (chunks->count * chunk_size))
	{
		// Make a chunk, if we can.  If we can't return NULL.
		if (! Prepare_New_Chunk())
		{
			return NULL;
		}
	}

	
	the_new_pointer = next_element;

	
	next_element = *((A_Pool_Element **) next_element);

#ifdef _DEBUG
	
	ASSERT(!(the_new_pointer->in_use));

	
	the_new_pointer->in_use = true;
	
#endif

	
	return (&(the_new_pointer->data));
}





template <class DATA_TYPE>
void Pointer_Pool<DATA_TYPE>::Release_Pointer
(
	DATA_TYPE * ptr_to_free
) 
{
	
#ifdef _DEBUG

#endif
	A_Pool_Element *the_element;		

	
	ASSERT(ptr_to_free != NULL);

	
	the_element = (A_Pool_Element *) ptr_to_free;

#ifdef _DEBUG
	
	ASSERT(the_element->in_use);

	
	the_element->in_use = false;

	
#endif

	
	*((A_Pool_Element **) ptr_to_free) = next_element;

	
	next_element = the_element;

	
	count--;

}




#endif __Pointer_Pool_h__


