

#ifndef _ScripInterfaceUtils_
#define _ScripInterfaceUtils_
#pragma once

#include "TeamConsts.h"
#include "SpecialFlag.h"
#include "AiPath.h"

#include "Cineractive.h"
#include "hot_type.h"
#include "hot_int.h"
#include "dirty_int.h"
#include "timer_int.h"
#include "character.h"
#include "group.h"
#include "scripts.h"
#include "hot_float.h"


#include "AIFleet.h"
#include "SiegeFleet.h"
#include "MoveFleet.h"
#include "DefendFleet.h"
#include "AttackFleet.h"
#include "AlertStatus.h"


typedef Vector3	Position;
typedef char*	Name;
typedef int		Handle;
typedef int		TeamNum;
typedef float	Time;
typedef float	Distance;
typedef int		DilithiumValue;

enum AiCommand;
enum eUseType;

enum eShieldFacing{
	
};

enum eDamageType
{
	kDamageType_Photon,
	kDamageType_Phaser,
	kDamageType_Fragment,
	kDamageType_Temp1,
	kDamageType_Temp2,
	kNumDamageType
};


class group;
class script;
class rule;

typedef rule*	rule_ptr;
typedef script*	script_ptr;





#define SI_CURRENT_VERSION  84

class FileReader;
class FileWriter;

// The interface for talking to the game.
class ScriptInterface
{
public:

	// These functions create && destroy the classes that
	// are defined as context for the scripts.
	// These classes look like globals in the script source
	// but are actually pointers that get instantiated by these
	// calls
	virtual hot_int* ScriptCreate_hot_int(char *name) = 0;
	virtual void	 ScriptDestroy_hot_int(hot_int *) = 0;

	virtual dirty_int* ScriptCreate_dirty_int(char *name) = 0;
	virtual void	   ScriptDestroy_dirty_int(dirty_int *) = 0;

	virtual timer_int* ScriptCreate_timer_int(char *name) = 0;
	virtual void ScriptDestroy_timer_int(timer_int *) = 0;

	virtual hot_float* ScriptCreate_hot_float(char *name) = 0;
	virtual void ScriptDestroy_hot_float(hot_float *) = 0;


	

	virtual character *ScriptGet_WORLD() = 0;
	virtual dirty_int *ScriptGet_ALWAYS_EVALUATE() = 0;
	virtual hot_int   *ScriptGet_isCineractiveCancel() = 0;
	virtual hot_int   *ScriptGet_isCineractiveDone() = 0;

	
	virtual hot_int *ScriptGet_moons_team_1() = 0;
	virtual hot_int *ScriptGet_moons_team_2() = 0;
	virtual hot_int *ScriptGet_moons_team_3() = 0;
	virtual hot_int *ScriptGet_moons_team_4() = 0;
	virtual hot_int *ScriptGet_moons_team_5() = 0;
	virtual hot_int *ScriptGet_moons_team_6() = 0;
	virtual hot_int *ScriptGet_moons_team_7() = 0;
	virtual hot_int *ScriptGet_moons_team_8() = 0;

	
	virtual hot_float *ScriptGet_strength_team_1() = 0;
	virtual hot_float *ScriptGet_strength_team_2() = 0;
	virtual hot_float *ScriptGet_strength_team_3() = 0;
	virtual hot_float *ScriptGet_strength_team_4() = 0;
	virtual hot_float *ScriptGet_strength_team_5() = 0;
	virtual hot_float *ScriptGet_strength_team_6() = 0;
	virtual hot_float *ScriptGet_strength_team_7() = 0;
	virtual hot_float *ScriptGet_strength_team_8() = 0;

	
	virtual hot_int *ScriptGet_tried_warp_team_1() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_2() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_3() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_4() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_5() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_6() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_7() = 0;
	virtual hot_int *ScriptGet_tried_warp_team_8() = 0;

	// some hot ints for the tutorials, to check various && sundry
	// different things the user has done/can do
	virtual hot_int *ScriptGet_band_selected() = 0;

	


	

	
	virtual Time GetTime() = 0;

	

	

	
	virtual bool IsAITeam(int team_num) = 0;

	
	virtual bool IsEnabled(int team_num) = 0;

	// Set the strategy cycle ticking at the begining
	virtual void ResetStrategyCycle() = 0;

	

	
	virtual float GetCurrentCPTotal( int teamNum ) = 0;
	virtual float GetCurrentPPTotal( int teamNum ) = 0;

	
	virtual void AddCommandPoints( int teamNum, float points ) = 0;
	virtual void AddProductionPoints( int teamNum, float points ) = 0;

	
	virtual float GetCurrentFleetTotal( int teamNum ) = 0;

	

	
	virtual void HideObject(Handle h, bool setting) = 0;

	
	virtual bool IsOdf(Handle h, Name odf) = 0;

	
	virtual bool IsVisibleToTeam(Handle h, TeamNum t) = 0;

	
	virtual bool HasBeenSeen(Handle h, TeamNum t) = 0;

	
	virtual TeamNum GetRealTeam(Handle h) = 0;
	virtual TeamNum GetPerceivedTeam(Handle h) = 0;

	
	virtual void SetWormholeTeam(Handle wormhole, TeamNum team) = 0;

	
	virtual void StuffIntoWormhole(Handle wormhole, Handle craft) = 0;

	
	virtual void EnableWormholeTravel(TeamNum team, bool enabled) = 0;

	
	virtual void SetWormholeDeleteOnTransport( Handle wormhole, bool deleteOnTransport ) = 0;

	
	virtual void SetWormholeExpireOnClose( Handle wormhole, bool expire ) = 0;

	
	virtual int GetAllyShipAttacks(TeamNum myTeam, TeamNum otherTeam) = 0;
	virtual int GetAllyStationAttacks(TeamNum myTeam, TeamNum otherTeam) = 0;

	
	virtual bool IsRace(Handle h, const Name race_name) = 0;

	
	virtual int CountUnits(TeamNum t = -1, char *odf = NULL) = 0;

	
	virtual bool AllStationsDestroyed( TeamNum t = -1 ) = 0;

	
	virtual int CountStations( TeamNum t = -1 ) = 0;

	
	virtual bool AllCapitalShipsDestroyed( TeamNum t = -1 ) = 0;

	
	virtual int CountCapitalShips( TeamNum t = -1 ) = 0;

	
	virtual int CountVisibleUnits(TeamNum visible_to, TeamNum t = -1, bool perceived_team = false, char *odf = NULL) = 0;

	
	virtual int CountUnitsNearObject(Handle h, Distance d, TeamNum t = -1, char *odf = NULL) = 0;

	
	virtual int CountUnitsNearPoint(Name path, Distance d, TeamNum t = -1,
		char *odf = NULL, int point = 0) = 0;

	
	virtual int CountVisibleUnitsNearPoint(TeamNum visible_to, Name path, Distance d, TeamNum t = -1,
		bool perceived_team = false, char *odf = NULL, int point = 0 ) = 0;

	
	virtual int CountVisibleUnitsNearObject(Handle me, Distance d, TeamNum t = -1,
		bool perceived_team = false, char *odf = NULL) = 0;

	
	virtual Handle GetNearestObjectNearPosition(const Position& pos, Distance d = -1.0, TeamNum t = -1,
		char *odf = NULL, TeamNum visible_to = -1,
		bool perceived_team = false, Handle skipObj = 0, AiCommand cmd = AiCommand(0) ) = 0;

	
	virtual Handle GetNearestObjectNearMe(Handle me, Distance d = -1.0, TeamNum t = -1,
		char *odf = NULL, TeamNum visible_to = -1,
		bool perceived_team = false) = 0;

	
	virtual Handle GetNearestObjectNearWithCmd(Handle me, AiCommand cmd, Distance d = -1.0, TeamNum t = -1,
		char *odf = NULL, TeamNum visible_to = -1,
		bool perceived_team = false) = 0;

	
	virtual Handle GetNearestObjectNearPath(Name path, Distance d = -1.0, TeamNum t = -1,
		char *odf = NULL, TeamNum visible_to = -1,
		bool perceived_team = false) = 0;

	
	virtual int CountUnitsInRectangle(Name path, TeamNum t = -1, char *odf = NULL,
		int corner1 = 0, int corner2 = 1) = 0;

	
	virtual int CountUnitsInRectangle2(Name path, TeamNum t = -1, char *odf = NULL,
		int corner1 = 0, int corner2 = 1) = 0;

	
	virtual Distance GetDistance(Handle &h1, Handle &h2) = 0;

	
	virtual Distance GetDistance(Handle &h1, Name path, int point = 0) = 0;

	
	virtual Distance GetDistance(Name path1, int point1, Name path2, int point2) = 0;

	
	virtual bool Exists(Handle h) = 0;

	
	virtual bool DoesNotExist(Handle h) = 0;

	// Does the object exist && is it still alive && well.
	virtual bool IsAlive(Handle h) = 0;

	
	virtual bool IsExploding(Handle h) = 0;

	
	virtual bool IsDerelict(Handle h) = 0;

	
	virtual bool IsCompromised(Handle h, TeamNum team) = 0;

	
	virtual float GetCrew(Handle h) = 0;

	
	virtual int GetPlanetCivPop(Handle h ) = 0;

	
	virtual void SetPlanetCivPop(Handle h, int population ) = 0;

	
	virtual bool IsThereARomulanSpyOnShip(Handle h) = 0;

	
	virtual Handle GetTranswarpExitWormhole(Handle transwarpGate) = 0;

	
	virtual float GetWeaponsHealth( Handle h ) = 0;
	virtual float GetSensorHealth( Handle h ) = 0;
	virtual float GetEnginesHealth( Handle h ) = 0;
	virtual float GetShieldSystemHealth( Handle h ) = 0;
	virtual float GetHullHealth( Handle h ) = 0;	

	
	virtual float GetShieldHealth( Handle h ) = 0;
	virtual float GetShieldHealth( Handle h, eShieldFacing eFacing ) = 0;

	
	virtual float GetMaxWeaponsHealth( Handle h ) = 0;
	virtual float GetMaxSensorHealth( Handle h ) = 0;
	virtual float GetMaxEnginesHealth( Handle h ) = 0;
	virtual float GetMaxShieldSystemHealth( Handle h ) = 0;
	virtual float GetMaxHullHealth( Handle h ) = 0;	

	
	virtual float GetMaxShieldHealth( Handle h ) = 0;
	virtual float GetMaxShieldHealth( Handle h, eShieldFacing eFacing ) = 0;

	
	virtual void SetEngineHealth(Handle h, float health) = 0;
	virtual void SetSensorHealth(Handle h, float health) = 0;
	virtual void SetWeaponHealth(Handle h, float health) = 0;
	virtual void SetShieldGeneratorHealth(Handle h, float health) = 0;
	virtual void SetHullHealth( Handle h, float health) = 0;

	
	virtual void SetShieldHealth(Handle h, float health ) = 0;
	virtual void SetShieldHealth(Handle h, float health, eShieldFacing eFacing ) = 0;

	
	virtual void DisableEngines(Handle h, bool status = false) = 0;
	virtual void DisableSensors(Handle h, bool status = false) = 0;
	virtual void DisableWeapons(Handle h, bool status = false) = 0;
	virtual void DisableShieldSystem(Handle h, bool status = false) = 0;

	// kill the bastard!
	virtual void DestroyShip(Handle h) = 0;

	
	virtual Handle GetFirstCraft() = 0;
	virtual Handle GetNextCraft() = 0;

	
	virtual Handle GetCraftFromTeamList(int teamNumber, int slotNumber) = 0;

	

	
	virtual Handle GetLocationOfFlag(Handle flag) = 0;

	// Get team information for a flag.
	virtual TeamNum GetRealTeamOfFlag(Handle flag) = 0;
	virtual TeamNum GetPercievedTeamOfFlag(Handle flag) = 0;

	
	virtual bool IsRaceOfFlag(Handle flag, Name raceName) = 0;

	////////////////////////////////
	// Mission
	////////////////////////////////

	
	virtual bool IsUnconfirmedAlly(int team1, int team2) = 0;
	virtual bool IsUnconfirmedNeutral(int team1, int team2) = 0;
	virtual bool IsUnconfirmedEnemy(int team1, int team2) = 0;
	virtual bool IsConfirmedAlly(int team1, int team2) = 0;
	virtual bool IsConfirmedNeutral(int team1, int team2) = 0;
	virtual bool IsConfirmedEnemy(int team1, int team2) = 0;

	


	

	

	
	virtual void AI_Team_Load_AIP
		(
		int which_team_number,
		char *which_aip
		) = 0;


	
	virtual void SetIsAITeam(const int &team_num, const bool &is_ai_team) = 0;

	
	virtual void EnableTeam(int team_num) = 0;

	
	virtual void DisableTeam(int team_num) = 0;

	
	virtual void EnableAllAITeams() = 0;

	
	virtual void DisableAllAITeams() = 0;

	
	virtual Handle SetSpecialForcesFlag(Handle me, bool am_I_special) = 0;

	
	virtual bool RecruitSpecialForces
		(
		char *odf,							
		int team_for,						
		int &object_handle					
		) = 0;

	
	virtual bool RecruitSpecialForces
		(
		int team_for,						
		int &object_handle					
		) = 0;
	

	
	virtual void SetPathType(Name path, PathType pathType) = 0;

	
	virtual void GetRepair(Handle me) = 0;

	
	virtual void Scout(Handle me) = 0;

	
	virtual void SearchAndDestroy(Handle me) = 0;

	
	virtual bool InQueue(Handle me) = 0;

	
	virtual bool UnderAttack(Handle me, float seconds = 1.0f) = 0;

	
	virtual bool InAttackMode(Handle me) = 0;

	
	virtual bool OutOfSpecialAttackMode(Handle me) = 0;

	
	virtual bool InStopMode(Handle me) = 0;

	
	virtual void CommandAllShips(AiCommand command, int teamNumber = -1,
		const char *odf = NULL, 
		bool onlySpecialForces = true,
		bool ignoreStations = false) = 0;

	
	virtual void SetCommandWhatWhoParamVO
		(
		Handle me,
		AiCommand what,
		Handle who,
		long param, 
		bool voice_overs = false
		) = 0;

	
	virtual void SetCommandWhatWhoVO
		(
		Handle me,
		AiCommand what,
		Handle who,
		bool voice_overs = false
		) = 0;

	
	virtual AiCommand GetCurrentCommand
		(
		Handle me
		) = 0;

	
	virtual bool IsCargoShipReadyToLoad
		(
		Handle me
		) = 0;


	
	virtual bool IsCargoShipFullAndReadyToSend
		(
		Handle me
		) = 0;

	
	virtual void StopAllShips(int teamNumber = -1, const char *odf = NULL,
		bool onlySpecialForces = true, bool ignoreStations = true) = 0;

	
	virtual void Attack(Handle me, Handle him) = 0;

	
	virtual void Attack(Handle me, Position location) = 0;

	
	virtual void Special_Attack(Handle me, Handle him, const char *odfName) = 0;

	
	virtual void Special_Attack(Handle me, const char *odfName) = 0;

	// Have me use a special weapon on a location
	virtual void Special_Attack(Handle me, Position location, const char *odfName) = 0;

	
	virtual void SetSpecialWeaponAttackInterval(int teamNumber, float seconds) = 0;

	
	virtual bool IsSpecialWeaponActive(Handle me, const char* pName) = 0;

	
	virtual void TurnOffSpecialWeapon(Handle me, const char* pName) = 0;

	// Set the tactics a particular craft is to use - sls
	virtual void SetTactics(Handle me, eTactics tactics) = 0; 

	
	virtual void AllowMultipleSpecialWeaponFire(int teamNumber, bool setting) = 0;

	// Have me attack him, using regular && AI-chosen special weapons.
	virtual void AI_Attack(Handle me, Handle him) = 0;

	
	virtual void AI_Attack(Handle me) = 0;

	
	virtual void AI_Attack(Handle me, Position location) = 0;

	
	virtual Vector3 GetMainEnemyBaseLocation(int teamNum) = 0;

	
	virtual void AttackMainEnemyBase(Handle me) = 0;

	
	virtual void Cloak(Handle me) = 0;

	
	virtual void ImmediateCloak(Handle me) = 0;

	
	virtual void Decloak(Handle me) = 0;

	
	virtual bool IsCloaked(Handle me) = 0;

	
	virtual void Goto(Handle me, Name path, bool warp=false) = 0;

	
	virtual void Goto(Handle me, Handle him, bool warp=false) = 0;

	
	virtual void Stop(Handle me) = 0;

	
	virtual void Patrol(Handle me, Name path) = 0;

	// Have me build the unit from the given ODF file.
	virtual void Build(Handle me, Name odf) = 0;

	

	virtual void AddBillboardEffect(Handle me, char *spriteName) = 0;
	virtual void AddTransporterEffect(Handle me) = 0; 
	virtual void RemoveBillboardEffect(Handle me) = 0;

	
	virtual void PauseSimulation() = 0;

	
	virtual void UnpauseSimulation() = 0;

	

	
	virtual void Starfield_Load_Background_Geometry(char *filename) = 0;

	
	virtual void SetTacticalCamera(Position cameraPosition, Handle interestObject) = 0;

	
	virtual void SetTacticalCamera(Position cameraPosition, Position interestPosition) = 0;

	
	virtual void CenterCamera(Name path, int index = 0) = 0;

	
	virtual void CenterCamera(Position position ) = 0;

	// Move the camera to look at the given object. Put the
	// in game camera there, NOT the cineractive camera.
	virtual void CenterCamera(Handle me )= 0;

	
	virtual Vector3 GetCameraPosition () = 0;

	
	virtual Handle GetCameraInterest() = 0;

	
	virtual const int* GetSelectList() = 0;

	
	virtual Handle GetFirstSelectedShip() = 0;

	
	virtual void SetSelectedUnit(Handle in_unitHandle, bool in_bDeselect = true ) = 0;

	
	virtual void PlayCinematicMovie
		(
		script_ptr the_script,
		Name filename = NULL, 
		bool stepOnFinish = true
		) = 0;

	
	virtual void StopMovie(void) = 0;

	
	virtual bool IsMoviePlaying(void) = 0;

	
	virtual Handle FindGameEvent(const char *eventName) = 0;

	
	virtual void TriggerGameEvent(Handle event, Handle me) = 0;

	
	virtual void MinimapMessage(Name path, int radius = 5, float growRate = 0.2f,
		int index = 0) = 0;

	// Add a new message mark to the minimap at the world location given.  The
	// grow rate is the time between increments in size, && the maximum radius
	// is the size beyond which it disappears.  The location is given as a current
	// location of an object.
	virtual void MinimapMessage(Handle h, int radius = 5, float growRate = 0.2f) = 0;

	
	virtual Handle ShowTitle(int x, int y, const char *text, bool isFade = true) = 0;

	
	virtual Handle ShowTitle(int x, int y, const char *text, float red, float green, float blue, bool isFade = true) = 0;
	
	
	virtual Handle ShowTitle(const char* text, bool isFade) = 0;

	
	virtual void RemoveTitle(Handle handle, bool isFade = true) = 0;

	
	virtual void DebugChatMessage(bool debug, const char *fmt,...) = 0;

	
	virtual void ShowChatMessage(float r,float g,float b,const char *key,...) = 0;

	
	virtual Handle ShowSubtitle(int x, int y, const char *text, bool isFade = true) = 0;

	
	virtual void RemoveSubtitle(Handle handle, bool isFade = true) = 0;

	

	
	virtual Handle GetHandle(Name n) = 0;

	
	virtual bool IsValid(Handle me) = 0;

	
	virtual const Position& GetLocation(Handle me) = 0;

	
	virtual Position GetAttackLocation(Handle target, Handle me) = 0;

	
	virtual const Position& GetLocation(Name path, int point = 0) = 0;

	
	virtual Handle BuildObject(Name odf, int team, Handle h, float right, float above, float in_front) = 0;

	
	virtual Handle BuildObject(Name odf, int team, Name path, int point = 0, float right = 0.0f, float above = 0.0f, float in_front = 0.0f) = 0;
	virtual Handle BuildObjectWithScale(Name odf, int team, Name path, float in_transition_time_in_secs) = 0;
	virtual Handle BuildObjectWithFade(Name odf, int team, Name path, float in_transition_time_in_secs) = 0;
	virtual Handle BuildObjectWithScaleAndFade(Name odf, int team, Name path, float in_transition_time_in_secs) = 0;

	
	virtual Handle ReplaceObject(Handle h, Name odf) = 0;

	
	virtual void RemoveObject(Handle h) = 0;
	virtual void RemoveObjectWithScale(Handle h, float transition_time_in_secs = 0.0f) = 0;
	virtual void RemoveObjectWithFade(Handle h, float transition_time_in_secs = 0.0f) = 0;
	virtual void RemoveObjectWithScaleAndFade(Handle h, float transition_time_in_secs = 0.0f) = 0;

	
	virtual void RemoveAllObjects() = 0;

	
	virtual void RemoveAllCraft(bool stationsToo = true, TeamNum t = -1) = 0;

	
	virtual void SetRealTeam(Handle h, TeamNum t) = 0;
	virtual void SetPerceivedTeam(Handle h, TeamNum t) = 0;

	
	virtual void SetRace(Handle h, const char *race_name) = 0;

	
	virtual const char* GetRace(Handle h ) = 0;

	// What race was this ship when it was built?
	virtual const char* GetOriginalRace(Handle h ) = 0;

	
	virtual void LoadCargoShip( Handle h, int cargoType) = 0;

	
	virtual void SetPosition(Handle h, Name path) = 0;

	
	virtual void SetPosition(Handle h, Name path, int index) = 0;

	
	virtual void SetOrientation(Handle h, float yaw, float pitch, float roll) = 0;

	
	virtual void SetVelocity(Handle h, const Vector3 &newVelocity) = 0;

	
	virtual Vector3 GetRandomVector3( float max = 2*PI ) = 0;

	
	virtual void SetAngularVelocity(Handle h, const Vector3	&omega) = 0;

	
	virtual void PointAt(Handle me, Name path) = 0;

	
	virtual void PointAt(Handle me, Handle objectOfInterest) = 0;

	
	virtual void PathPlanning(bool planningOn) = 0;

	
	virtual void CollisionPrevention(bool preventionOn) = 0;

	
	virtual float GetImpulseSpeed(Handle h) = 0;
	
	virtual void SetImpulseSpeed(Handle h, float newImpulseSpeed) = 0;
	
	virtual void ResetImpulseSpeed(Handle h) = 0;

	
	virtual void SetCraftCinematicSpeed(Handle h, float in_speed) = 0;

	
	virtual void SetAutoLevel(Handle h, bool set) = 0;

	// Allow warping || not
	// set false to disallow warping
	// Returns previous state
	// Assumes is called when no ship is currently
	// trying to warp, ie at the start of a mission || something
	virtual bool AllowWarp(bool allow) = 0;

	
	virtual void SetCrew(Handle h, float crew_size) = 0;

	
	virtual void AllowAsTarget(Handle h, int team, bool validTarget) = 0;

	
	virtual void SetForceOntoMap(Handle h, bool forceItToSayOnMap) = 0;

	
	virtual void ExplodeNow(Handle h) = 0;

	
	virtual void CraftCanRepair(Handle h, bool canRepair) = 0;

	
	virtual void CraftCannotDie(Handle h, bool status = false) = 0;

	
	virtual void ButItCanBeDerelicted(Handle h, bool status = false) = 0;

	
	virtual Handle GetNearestStarbase(Handle me, int team) = 0;

	
	virtual Handle GetNearestShipyard(Handle me, int team, bool derelictOK = false) = 0;

	
	virtual Handle GetNearestRepairFacility(Handle me, int team) = 0;

	
	virtual Handle GetNearestObject(Handle h) = 0;

	
	virtual Handle GetNearestTranswarpGate(Handle h) = 0;

	
	virtual Handle GetNearestVehicle(Handle h) = 0;

	
	virtual Handle GetNearestVehicle(Name path,int point) = 0;

	
	virtual Handle GetNearestEnemy(Handle h, float cutoff_range = 450.0f) = 0;

	
	virtual void LinkWormholes(Handle first, Handle second) = 0;

	

	
	virtual Handle CreateSpecialFlag(Handle h, Name flag) = 0;

	
	virtual void RestrictSpecialFlag(Handle flag,
		SpecialFlag::Restriction restrict) = 0;
	virtual void RestrictSpecialFlag(Handle flag,
		SpecialFlag::Restriction restrict, Name odfName) = 0;
	virtual void RestrictSpecialFlag(Handle flag,
		SpecialFlag::Restriction restrict, Handle h) = 0;

	
	virtual void ClearRestrictions(Handle flag) = 0;

	
	virtual void TransferFlag(Handle flag, Handle h) = 0;

	
	virtual bool ObjectHasFlag( Handle h ) = 0;

	

	
	virtual void FailMission(Time t, Name fileName = NULL) = 0;

	
	virtual void SucceedMission(Time t, Name fileName = NULL) = 0;

	

	
	virtual void MonitorInputControls(bool (*pCallback)(void)) = 0;

	
	virtual const int* GetInputControlPointer(const char *pControlName) = 0;

	
	virtual void SetRelation(int team1, int team2, eTeamRelation relation) = 0;

	
	virtual void FerengiCanTrade( bool canTrade ) = 0;

	
	virtual void FerengiCanSteal( bool canSteal ) = 0;


	

	
	virtual void ObjectivesDisplay_Set_Visibility(bool on) = 0;

	


	
	virtual void ObjectivesDisplay_Set_Text_From_File(char *file_name, bool reset_objectives = true) = 0;

	
	virtual void ObjectivesDisplaySetObjectiveComplete(int whichObjective, bool complete) = 0;
	
	

	// Load an objectives file in XML format - crp
	virtual void ObjectivesDisplay_LoadXmlFile(char *pFileName) = 0;

	// Set the current description text - crp
	virtual void ObjectivesDisplay_SetDescription(int whichDescription) = 0;

	
	virtual void ObjectivesDisplay_SetPrimaryComplete(int whichObjective, bool complete) = 0;

	
	virtual void ObjectivesDisplay_SetPrimaryVisible(int whichObjective, bool visible) = 0;

	
	virtual void ObjectivesDisplay_SetSecondaryComplete(int whichObjective, bool complete) = 0;
	
	
	virtual void ObjectivesDisplay_SetSecondaryVisible(int whichObjective, bool visible) = 0;
	
	
	virtual void ObjectivesDisplay_SetHintVisible(int whichHint, bool visible) = 0;

	
	virtual void MessageDisplay_Set_Text_From_File(char *filename) = 0;

	
	virtual void FadeOut
		(
		script_ptr the_script,
		float time, 
		float delay = 0, 
		bool white = false,	
		bool stepOnFinish = false
		) = 0;

	virtual void FadeIn(
		script_ptr the_script,
		float time, 
		float delay = 0, 
		bool white = false,	
		bool stepOnFinish = false
		) = 0;


	virtual void FadeRemove() = 0;

	
	virtual void EnableLight(Handle object, bool setting = true) = 0;

	
	virtual void StartCredits(void) = 0;
	virtual void EndCredits(void) = 0;

	
	virtual void SetClippingDistance
		(
		float clipDistance
		) = 0;

	
	virtual void SetObjectCullingDistance
		(
		float cullingDistance
		) = 0;

	//
	// RestoreObjectCullingDistance
	//
	// Make sure objects "pop" at the default distance
	//
	virtual void RestoreObjectCullingDistance() = 0;

	
	virtual int AudioMessage
		(
		script_ptr the_script,
		const char *msg,
		bool stepOnFinish = true
		) = 0;

	

	
	virtual void JukeBoxPlaySpecific(const char *pFile) = 0;

	
	virtual void JukeBoxPlay() = 0;

	
	virtual void JukeBoxStop() = 0;

	
	virtual void JukeBoxAddCustomMusic(const char* in_TrackName, int in_ActionLevel, int in_EventType, 
		float in_Volume = 1.0f, bool in_IsLooping = true) = 0;

	
	virtual void JukeBoxDisableDynamicMusic(void) = 0;

	
	virtual void JukeBoxEnableDynamicMusic(void) = 0;

	
	virtual void JukeBoxPlaySpecificEventMusic(const char* pTrackName) = 0;

	
	virtual void JukeBoxSetUseCustomMusic(bool in_UseCustomMusic) = 0;

	
	virtual void Create3DAudioSound(const Position soundLocation, const char* soundName) = 0;

	// Wait to move on until user has pressed enter key
	virtual void WaitForEnterKey
		(
		script_ptr the_script
		) = 0;

	
	virtual bool IsAudioMessageDone(int msg) = 0;

	virtual void StopAudioMessage(int Msg) = 0;

	

	
	virtual void CineractiveStart
		(
		script_ptr the_script,
		bool stepOnFinish = true
		) = 0;

	
	virtual void CineractiveCameraOffset(float x, float y, float z, bool isOrientationRelative = true) = 0;
	virtual void CineractiveCameraOffset(const char* path, float offsetSpeed, bool isOrientationRelative = true) = 0;
	virtual void CineractiveTargetOffset(float x, float y, float z, bool isOrientationRelative = true) = 0;
	virtual void CineractiveTargetOffset(const char* path, float offsetSpeed, bool isOrientationRelative = true) = 0;

	
	virtual void CineractiveShot(script_ptr script, const char* path, int index, bool useOffsetDirection = false) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, int index, const char* targetPath, int targetIndex) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, int index, CinHandle targetHandle) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, int index, const char* targetPath, float targetSpeed) = 0;
	virtual void CineractiveShot(script_ptr script, CinHandle handle, bool useOffsetDirection = false) = 0;
	virtual void CineractiveShot(script_ptr script, CinHandle handle, const char* targetPath, int targetIndex) = 0;
	virtual void CineractiveShot(script_ptr script, CinHandle handle, CinHandle targetHandle) = 0;
	virtual void CineractiveShot(script_ptr script, CinHandle handle, const char* targetPath, float targetSpeed) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, float speed, bool useOffsetDirection = false) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, float speed, const char* targetPath, int targetIndex) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, float speed, CinHandle targetHandle) = 0;
	virtual void CineractiveShot(script_ptr script, const char* path, float speed, const char* targetPath, float targetSpeed) = 0;
	virtual void CineractiveShotTrackLeft(script_ptr script, const char* path, int index, const char* targetPath, int targetIndex, CinHandle targetHandle) = 0;

	// Make it so that the camera moves smoothly along the length of the path, instead of taking the
	// same amount of time in each segment
	virtual void CineractiveConstantSpeed(bool constantSpeed) = 0;

	
	virtual void SetPathSegmentSpeedModifier(int segment, float segmentSpeed) = 0;

	
	virtual void CineractiveEnd() = 0;

	
	virtual void AddToLoad(void *pPtr) = 0;
	virtual void RemoveFromLoad(void *pPtr) = 0;

	virtual void AddToSave(void *pPtr) = 0;
	virtual void RemoveFromSave(void *pPtr) = 0;

	
	virtual bool out(FileWriter &fp, int *buf, size_t size, char *text = "undefint") = 0;
	virtual bool out(FileWriter &fp, bool *buf, size_t size, char *text = "undefbool") = 0;	
	virtual bool out(FileWriter &fp, hot_int *buf, char *text = "undefhot_int") = 0;
	virtual bool out(FileWriter &fp, timer_int *buf, char *text = "undeftimer_int") = 0;
	virtual bool out(FileWriter &fp, Vector3 *buf, size_t size, char *text = "undefVector3") = 0;
	virtual bool out(FileWriter &fp, float *buf, size_t size, char *text = "undefFloat") = 0;

	virtual bool in(FileReader &fp, int *buf, size_t size) = 0;
	virtual bool in(FileReader &fp, bool *buf, size_t size) = 0;
	virtual bool in(FileReader &fp, hot_int *buf) = 0;
	virtual bool in(FileReader &fp, timer_int *buf) = 0;
	virtual bool in(FileReader &fp, Vector3 *buf, size_t size) = 0;
	virtual bool in(FileReader &fp, float *buf, size_t size) = 0;

	
	virtual void SetTeamText(char *newMessage) = 0;

	
	virtual SiegeFleet * CreateNewSiegeFleet() = 0;

	
	virtual MoveFleet * CreateNewMoveFleet() = 0;

	
	virtual AttackFleet * CreateNewAttackFleet() = 0;

	
	virtual DefendFleet * CreateNewDefendFleet() = 0;

	
	virtual unsigned int RandomGetUnsigned (unsigned modulo) = 0;

	
	virtual float RandomGetFloat ( float max ) = 0;

	
	virtual bool IsShipAliveAndOffMap(Handle h) = 0;

	// load a new background && stars
	virtual void Starfield_Load_New_Background_Geometry
		(
		char *background, 
		char *stars
		) = 0;

	
	virtual bool IsStation(Handle h) = 0;

	
	virtual void RemoveFromAdLog(Handle h) = 0;

	// If this craft was removed from the Admiral's Log && you want to add
	// it back in, this will put it back in the log, with information starting
	// with when you added it
	virtual void AddToAdLog(Handle h) = 0;

	// This can be used to delay sound until the first frame is rendered.
	virtual bool HasTimeElapsed(float time) = 0;

	
	virtual void SetCanBeDecommissioned(Handle h,bool val) = 0;

	
	virtual void DisableChat(bool disable) = 0;

	
	virtual void MuteSounds(float muting) = 0;

	
	virtual void SetName(Handle craft, const char *pName) = 0;
	virtual const char* GetName(Handle craft) = 0;

	virtual void DisableSave() = 0;
	virtual void EnableSave() = 0;

	
	virtual void TeamKamikaze(int team) = 0;

	
	virtual void TeamAttackAll(int team) = 0;

	// Set the distress call range for a team
	virtual void SetTeamDistressCallRange(int teamNum, float range) = 0;

	
	virtual void SetTimelineEraAndTick(int era, int tick, int team) = 0;
	// advance to the next sequential era for team 'team', starting at tick 0
	virtual void AdvanceEra(int team) = 0;
	
	virtual void AdvanceTick(int team) = 0;

	
	virtual Handle GetDockedShip(Handle shipyard) = 0;


	
	virtual void ShowBriefing() = 0;

	
	virtual bool ApplyDamageEffect(Handle object2damage, eDamageType damagetype, float in_data) = 0;


	
	virtual Handle GetTarget(Handle in_craft) = 0;

	//
	//  Get the radius of the given craft.
	//
	virtual float GetRadius(Handle in_craft) = 0;

	
	virtual bool Place3DMarkerAtLocation(const char* in_MarkerObjName, const Position &in_position, const Position &in_orientation) = 0;
	virtual bool Hide3DMarker(const char *in_Marker) = 0;

	
	virtual bool AttachScreenMarkerToObject(const char* in_ScreenMarkerName, Handle in_object) = 0;
	virtual bool SetVisibleScreenMarker(const char* in_ScreenMaker, bool visible) = 0;

	
	virtual void AllowDamage(int in_team, bool in_allowDamage) = 0;

	
	virtual void UITrigger(const char *in_event) = 0;

	
	virtual bool IsUsed(Handle me, eUseType useType) = 0;

	
	virtual Handle WhoUsed(Handle me, eUseType useType) = 0;

	
	virtual void ResetUse(Handle me, eUseType useType) = 0;

	
	virtual void SetUseAutoReset(Handle me, eUseType useType, bool in_autoReset) = 0;

	
	virtual void MarkAsUsed(Handle me, eUseType useType, Handle theUser) = 0;

	
	virtual bool InNebula(Handle me, int NebulaType) = 0;

	
	virtual Handle FireParticleEffect(const char* partFXName, Position in_pos, float size = 1.0f) = 0;

	
	virtual bool ShowSpaceDust(bool in_show) = 0;

	

	
	virtual int GetNumberOfHumanPlayers() = 0;

	
	virtual bool IsHumanControlledTeam(int in_team) = 0;

	
	virtual void Victorious(int in_team) = 0;
	virtual void Defeated(int in_team) = 0;

	
	virtual bool AddUpgrade(Handle me, const char* in_upgradeToAdd) = 0;
	virtual bool RemoveUpgrade(Handle me, const char* in_upgradeToRemove) = 0;
	virtual bool IsUpgradeAvailable(Handle me, const char* in_upgradeName) = 0;
	virtual bool IsUpgradePurchased(Handle me, const char* in_upgradeName) = 0;

	
	virtual bool AwardAchievement(const char* in_AchievementName, int playerSlot) = 0;

	
	virtual void SetSelfDestructRadius(Handle me, float in_radius) = 0;

	
	virtual void SetWarpSuppresionRadius(Handle me, float inRadius) = 0;

	
	virtual bool IsFocusEntity(Handle me) = 0;

	
	virtual Handle GetTeamFocusEntity(int in_team) = 0;

	
	virtual void SetCraftVisibleToTeam(Handle me, int in_team, bool override = true) = 0;
	
	
	
	virtual void ResetSpecialTimers(Handle me) = 0;
	
	virtual bool GetUserShowTutorialBoxes() = 0;
	
	virtual bool IsUserInStratMap() = 0;
	
	virtual Handle GetCurrentHighlight() = 0;
	
	virtual int GetNumberOfRespawns(int in_team) = 0;
	
	virtual void EnableDirectControl() = 0;
	
	virtual void DisableDirectControl() = 0;

	

	
	virtual void CreatePath(const char* in_Name)  = 0;

	
	virtual void ClearPath(const char* in_Name) = 0;

	
	virtual void AppendPointToPath(const char *in_Name, Position in_point)  = 0;

	
	virtual void InsertPointInPath(const char *in_Name, int atIndex, Position in_point) = 0;

	
	virtual void AppendPathToPath(const char *basePath, const char *appendedPath) = 0;

	
	virtual int GetNumberOfPointsOnPath(const char *in_Name) = 0;
};


extern ScriptInterface *g_pScriptInterface;


extern float g_scriptInterfaceElapsedTime;

#endif 
