

#ifndef __semi_dynamic_array_h__
#define __semi_dynamic_array_h__
#pragma once




template <class T>
class semi_dynamic_array
{
	public:

		
		semi_dynamic_array
		(
			int i_size					
		);

		///////////////////////////////////////////////////////////////////////
		//
		// ~semi_dynamic_array
		//
		// Release array of objects
		~semi_dynamic_array();

		
		semi_dynamic_array
		(
			semi_dynamic_array<T>& sda
		);

		
		semi_dynamic_array
		(
             int r_size,
             char *r_data
		);

		
		void Get_Raw
        (
            int *r_size,                
            char **r_data               
        );

		
		semi_dynamic_array& operator=
		(
			semi_dynamic_array<T>& sda
		);

		
		T& operator[](const int nIndex);
		T operator[](const int nIndex) const;

		
		void Resize
		(
			int new_size
		);

		
		int Get_Size();

		T* GetData(){return data;}
	protected:

		T* data;						
		int size;						// How big is array currently
		int m_allocated_size;			// what is the arrays currently allocated underlying memory size?  (only valid if allocated)

		
		void Allocate
		(
			int size
		);


		
		void Assign
		(
			semi_dynamic_array<T>& sda
		);

		
		void Clean_Up();



};





template <class T>
void semi_dynamic_array<T>::Allocate
(
	int new_size
)
{
	
	if (new_size <= 0) return;

	
	size = new_size;

	
	m_allocated_size = sizeof(T) * new_size;
	data = (T*) new char [m_allocated_size];
	memset(data, 0, m_allocated_size);

}





template <class T>
void semi_dynamic_array<T>::Assign
(
	semi_dynamic_array<T>& sda
)
{
	
	int i;
	

	
	Clean_Up();

	
	Allocate(sda.size);

	
	for (i = 0; i < size; i++)
	{
		
		data[i] = sda.data[i];
	}

}





template <class T>
void semi_dynamic_array<T>::Resize
(
	int new_size
)
{
	
	if (new_size <= 0) return;

	int old_size = size;

	
	size = new_size;

	// Allocate the memory
	if (data == NULL){
		Allocate(new_size);
	}
	else{
		
		const int required_allocated_size = sizeof(T)*new_size;
		if ((required_allocated_size < (m_allocated_size / 2)) || (required_allocated_size > m_allocated_size))
		{
			
			void *old_data = data;

			
			int m_allocated_size = required_allocated_size * 3 / 2;
			data = reinterpret_cast <T*> (new char[m_allocated_size]);

			
			memcpy (data, old_data, (old_size < new_size ? old_size : new_size) * sizeof (T));

			
			delete[] old_data;
		}

		
		if (new_size > old_size) {
			memset(data + old_size, 0, (new_size - old_size) * sizeof(T));
		}
	}

}





template <class T>
int semi_dynamic_array<T>::Get_Size()
{
	return size;
}





template <class T>
void semi_dynamic_array<T>::Clean_Up()
{
	
	delete[] data;

	
	data = NULL;
	m_allocated_size = 0;
}





template <class DATA_TYPE>
void semi_dynamic_array<DATA_TYPE>::Get_Raw
(
    int *r_size,
    char **r_data
) 
{
	*r_size = size * sizeof(T);
	*r_data = (char *) data;

    return;
}





template <class T>
semi_dynamic_array<T>::semi_dynamic_array
(
	int i_size					
)
{

	
	data = NULL;
    size = 0; 
	m_allocated_size = 0;

	
	Allocate(i_size);

}





template <class T>
semi_dynamic_array<T>::~semi_dynamic_array()
{
	
	Clean_Up();
}




///////////////////////////////////////////////////////////////////////////////
//
// semi_dynamic_array<T>::semi_dynamic_array
//
// Creates any object with a copy constructor.
template <class T>
semi_dynamic_array<T>::semi_dynamic_array
(
	semi_dynamic_array<T>& sda
)
{
	
	data = NULL;
	m_allocated_size = 0;

	
	chunk_size = sda.chunk_size;
	max_chunks = sda.max_chunks;
	chunks = sda.chunks;

	
	Assign(sda);
}





template <class T>
semi_dynamic_array<T>::semi_dynamic_array
(
    int r_size,
    char *r_data
)
{

	// Until we successfully allocate it, there is no data
	data = NULL;
	m_allocated_size = 0;

	
	Allocate((r_size / sizeof(T)) + 1);

    Assert(r_data); 
    memcpy((char *)data, r_data, r_size);
}





template <class T>
semi_dynamic_array<T>& semi_dynamic_array<T>::operator=
(
	semi_dynamic_array<T>& sda
)
{

	
	Assign(sda);

	return this;
}





template <class T>
T& semi_dynamic_array<T>::operator[](const int nIndex)
{
	
	_ASSERTE(nIndex >= 0 && nIndex < size);

	
	return data[nIndex];
}





template <class T>
T semi_dynamic_array<T>::operator[](const int nIndex) const
{
	
	semi_dynamic_array<T>& curr = *(semi_dynamic_array<T>*)this;

	return curr[nIndex];
}
//
// operator[]
//
///////////////////////////////////////////////////////////////////////////////


#endif // semi_dynamic_array
