using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace LegacyMissionEditor
{
    class Missions
    {
        public static DataTable table;
        public const int DEFAULT_COUNT = 4;

        public static string GetBaseName(DataRow mission)
        {
            string missionName = mission["Mission Name"].ToString();
            if (missionName.ToUpper() == "VANGUARD")
                return "Vanguard";
            if (missionName.ToUpper() == "ESCORT")
                return "Escort";
            if (missionName.ToUpper() == "RAIDER")
                return "Raider";
            if (missionName.ToUpper() == "COOPWAVE")
                return "CoopWave";
            string baseName = mission["SOL File"].ToString();
            return baseName.Substring(0, baseName.Length - 4);
        }

        public static void Load()
        {
            table = new DataTable();
            table.TableName = "table";
            DataColumn missionIndex = new DataColumn("MissionIndex", typeof(int));
            missionIndex.AutoIncrement = true;
            missionIndex.AutoIncrementSeed = 1;
            missionIndex.AutoIncrementStep = 1;
            table.Columns.Add("Mission Name");
            table.Columns.Add("SOL File");
            table.Columns.Add("Rule File");
            table.Columns.Add("Script File");
            table.Columns.Add("Date");
            table.Columns.Add("Image");
            table.Columns.Add("Num Ships");
            table.Columns.Add("Hero");
            table.Columns.Add("Skip Points");
            table.Columns.Add("Free CPs");

            // Don't add the default missions
            DataRow row;
            row = table.NewRow();
            row["Mission Name"] = "CoopWave";
            row["SOL File"] = "";
            row["Rule File"] = "CoopWaveR.rul";
            row["Script File"] = "CoopWaveS.scp";
            row["Date"] = "";
            row["Image"] = "";
            row["Num Ships"] = "";
            row["Hero"] = "";
            row["Skip Points"] = "";
            row["Free CPs"] = "";
            table.Rows.Add(row);
            row = table.NewRow();
            row["Mission Name"] = "Escort";
            row["SOL File"] = "";
            row["Rule File"] = "EscortR.rul";
            row["Script File"] = "EscortS.scp";
            row["Date"] = "";
            row["Image"] = "";
            row["Num Ships"] = "";
            row["Hero"] = "";
            row["Skip Points"] = "";
            row["Free CPs"] = "";
            table.Rows.Add(row);
            row = table.NewRow();
            row["Mission Name"] = "Raider";
            row["SOL File"] = "";
            row["Rule File"] = "RaiderR.rul";
            row["Script File"] = "RaiderS.scp";
            row["Date"] = "";
            row["Image"] = "";
            row["Num Ships"] = "";
            row["Hero"] = "";
            row["Skip Points"] = "";
            row["Free CPs"] = "";
            table.Rows.Add(row);
            row = table.NewRow();
            row["Mission Name"] = "Vanguard";
            row["SOL File"] = "";
            row["Rule File"] = "VanguardR.rul";
            row["Script File"] = "VanguardS.scp";
            row["Date"] = "";
            row["Image"] = "";
            row["Num Ships"] = "";
            row["Hero"] = "";
            row["Skip Points"] = "";
            row["Free CPs"] = "";
            table.Rows.Add(row);

            // Populate the table list from LegacyMissions.xml
            string filename = "campaign\\Legacy\\LegacyMissions.xml";
            XmlTextReader reader = new XmlTextReader(filename);
            while (reader.Read())
            {
                if (reader.Name.Length > 0 && reader.Name != "ShellSettings")
                {
                    row = table.NewRow();
                    row["Mission Name"] = reader.Name;
                    table.Rows.Add(row);
                    row["SOL File"] = reader.GetAttribute("file");

                    // Correct the name; grab the label
                    string stest = Globals.LabelsEN.Get("mission_select" + row["SOL File"]);
                    if (stest != null)
                    {
                        if (stest[stest.Length - 1] == ' ') stest = stest.Substring(0, stest.Length - 1);
                        row["Mission Name"] = stest;
                    }
                    row["Rule File"] =
                        reader.GetAttribute("file").Substring(0,
                            reader.GetAttribute("file").Length - 4) + "R.rul";
                    row["Script File"] =
                        reader.GetAttribute("file").Substring(0,
                            reader.GetAttribute("file").Length - 4) + "S.scp";
                    row["Date"] = reader.GetAttribute("date");
                    row["Image"] = reader.GetAttribute("image");
                    row["Num Ships"] = reader.GetAttribute("numShips");
                    row["Hero"] = reader.GetAttribute("hero");
                    row["Skip Points"] = reader.GetAttribute("skipPoints");
                    row["Free CPs"] = reader.GetAttribute("freeCPs");
                }
            }
            reader.Close();
        }

        public static int FindMission(string name)
        {
            for ( int i = 0; i < table.Rows.Count; i++ )
            {
                string missionName = table.Rows[i]["Mission Name"].ToString();
                missionName = missionName.ToLower();
                string solName = table.Rows[i]["SOL File"].ToString();
                solName = solName.ToLower();

                if (missionName == name.ToLower()) return i;
                if (solName == (name+".sol").ToLower()) return i;
            }
            return -1;
        }

        public static void ChangeMissionName( string origName, string newName )
        {
            int i = FindMission(origName);
            if ( i == -1 ) return;
            string baseName = GetBaseName(table.Rows[i]);
            if (baseName.Length != 5) return;
            if (baseName.Substring(0, 3).ToUpper() != "STL") return;
            if (CreateNew(newName, baseName, origName))
            {
                Delete(origName);
            }
        }

        public static bool CreateNew(string name, string basename, string source )
        {
            // Copy the template mission to a new directory. Assume for now that the name is valid.
            string newpath = "missions\\" + basename + "\\";
            if (Directory.Exists(newpath)) Directory.Delete(newpath, true);
            Directory.CreateDirectory(newpath);

            File.Copy("missions\\" + source + "\\" + source + ".sln", newpath + basename + ".sln");
            File.Copy("missions\\" + source + "\\" + source + "R.rul", newpath + basename + "R.rul");
            //File.Copy("missions\\" + source + "\\" + source + "R.def", newpath + basename + "R.def");
            File.Copy("missions\\" + source + "\\" + source + "R.vcproj", newpath + basename + "R.vcproj");
            File.Copy("missions\\" + source + "\\" + source + "S.scp", newpath + basename + "S.scp");
//            File.Copy("missions\\" + source + "\\" + source + "S.def", newpath + basename + "S.def");
            File.Copy("missions\\" + source + "\\" + source + "S.vcproj", newpath + basename + "S.vcproj");

            if (File.Exists("sol\\" + basename + ".sol")) File.Delete("sol\\" + basename + ".sol");
            File.Copy("sol\\"+frmAddMission.sol, "sol\\" + basename + ".sol");

            if (File.Exists("objectives\\" + basename + "_Objectives.xml")) File.Delete("objectives\\" + basename + "_Objectives.xml");

            if (File.Exists("objectives\\" + source + "_Objectives.xml"))
                File.Copy("objectives\\" + source + "_Objectives.xml", "objectives\\" + basename + "_Objectives.xml\n");
            else
                File.Copy("objectives\\" + "ObjectivesBlank.xml", "objectives\\" + basename + "_Objectives.xml\n");


            // Open the project files and edit their contents
            Globals.SearchReplace(newpath + basename + ".sln",  source, basename);
            Globals.SearchReplace(newpath + basename + "R.rul",  source, basename);
            Globals.SearchReplace(newpath + basename + "R.rul", source+"S", basename+"S");
//            Globals.SearchReplace(newpath + basename + "R.def", source, basename);
            Globals.SearchReplace(newpath + basename + "R.vcproj",  source, basename);
//            Globals.SearchReplace(newpath + basename + "S.scp",  source, basename);
//            Globals.SearchReplace(newpath + basename + "S.def",  source, basename);
            Globals.SearchReplace(newpath + basename + "S.vcproj",  source, basename);
            Globals.SearchReplace(newpath + basename + "S.vcproj", source, basename);

            // Find the script and rule in the sol file, if they exist
            string newsolfile = "sol\\" + basename + ".sol";
            int at = Globals.BinarySearch(newsolfile, ".dsl");
            if (at > -1)
            {
                string srep = Globals.BinaryToSubstring(newsolfile, at - 32, 32);
                int i;
                for ( i = 0; i < srep.Length-3; i++ )
                    if ( srep.Substring(i,3).ToUpper() == "STL" ) break;
                if (i < srep.Length - 3)
                {
                    srep = srep.Substring(i, srep.Length - i - 1);
                    Globals.BinarySearchReplace(newsolfile, srep, basename);
                }
            }

            at = Globals.BinarySearch("sol\\" + basename + ".sol", ".drl");
            if (at > -1)
            {
                string srep = Globals.BinaryToSubstring(newsolfile, at - 32, 32);
                int i;
                for (i = 0; i < srep.Length - 3; i++)
                    if (srep.Substring(i, 3).ToUpper() == "STL") break;
                if (i < srep.Length - 3)
                {
                    srep = srep.Substring(i, srep.Length - i - 1);
                    Globals.BinarySearchReplace(newsolfile, srep, basename);
                }
            }

            Globals.SearchReplace(newpath + basename + ".sln", source.ToLower(), basename.ToLower());
            Globals.SearchReplace(newpath + basename + "R.rul", source.ToLower(), basename.ToLower());
            Globals.SearchReplace(newpath + basename + "R.rul", source.ToLower() + "S", basename.ToLower() + "S");
//            Globals.SearchReplace(newpath + basename + "R.def", source.ToLower(), basename.ToLower());
            Globals.SearchReplace(newpath + basename + "R.vcproj", source.ToLower(), basename.ToLower());
//            Globals.SearchReplace(newpath + basename + "S.scp", source.ToLower(), basename.ToLower());
//            Globals.SearchReplace(newpath + basename + "S.def", source.ToLower(), basename.ToLower());
            Globals.SearchReplace(newpath + basename + "S.vcproj", source.ToLower(), basename.ToLower());
            Globals.SearchReplace(newpath + basename + "S.vcproj", source.ToLower(), basename.ToLower());

            Globals.SearchReplace(newpath + basename + ".sln", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "R.rul", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "R.rul", source.ToUpper() + "S", basename.ToUpper() + "S");
//            Globals.SearchReplace(newpath + basename + "R.def", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "R.vcproj", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "S.scp", source + "_TITLE", basename.ToUpper() + "_TITLE");
            Globals.SearchReplace(newpath + basename + "S.scp", source.ToUpper() + "_Objectives.xml", basename.ToUpper() + "_Objectives.xml");
//            Globals.SearchReplace(newpath + basename + "S.def", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "S.vcproj", source.ToUpper(), basename.ToUpper());
            Globals.SearchReplace(newpath + basename + "S.vcproj", source.ToUpper(), basename.ToUpper());

            Globals.SearchReplace("objectives\\" + basename + "_Objectives.xml", "mission name", name);
            Globals.SearchReplace("objectives\\" + basename + "_Objectives.xml", source.ToUpper(), basename.ToUpper());

            // Make sure the "new" objectives are in the label map! Sort of a hack.
            Globals.StringsEN.Load("Objectives_Localized_Strings_EN.xml");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_DESCRIPTIONS_DESCRIPTION0", "New Description");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_DESCRIPTIONS_DESCRIPTION1", "New Description");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY0", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY1", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY2", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY3", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY4", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY5", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_PRIMARY_PRIMARY6", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_OBJECTIVES_SECONDARY_SECONDARY0", "New Objective");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_HINTS_HINT0", "New Hint");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_HINTS_HINT1", "New Hint");
            Globals.StringsEN.Set(basename.ToUpper() + "_OBJECTIVES_HINTS_HINT2", "New Hint");
            Globals.StringsEN.Save();



            DataRow row = table.NewRow();
            int m = FindMission( source );

            if ( m == -1 )
            {
                row["Mission Name"] = name;
                row["Date"] = 0;
                row["Image"] = "snapshot013";
                row["Num Ships"] = 4;
                row["Hero"] = "";
                row["Skip Points"] = 0;
                row["Free CPs"] = 0;
            }
            else
            {
                row["Mission Name"] = name;
                row["Date"] = table.Rows[m]["Date"];
                row["Image"] = table.Rows[m]["Image"];
                row["Num Ships"] = table.Rows[m]["Num Ships"];
                row["Hero"] = table.Rows[m]["Hero"];
                row["Skip Points"] = table.Rows[m]["Skip Points"];
                row["Free CPs"] = 0;
            }
            row["SOL File"] = basename + ".sol";
            row["Rule File"] = basename + "r.rul";
            row["Script File"] = basename + "s.scp";
            table.Rows.Add(row);
            string mid = row["SOL File"].ToString();
            mid = mid.Substring(0, mid.Length - 4);
            Globals.StringsEN.Set(mid.ToUpper() + "_TITLE", name);
            Globals.StringsEN.Save();
            Globals.LabelsEN.Load("LabelMap_EN.xml");
            Globals.LabelsEN.Set("mission_select" + row["SOL File"].ToString(), name + " ");
            Globals.LabelsEN.Save();
            return true;
        }

        public static void Delete(string name)
        {
            int removeat = -1;
            for (int i = 0; i < table.Rows.Count; i++)
            {
                if ((string)(table.Rows[i]["Mission Name"]) == name)
                {
                    removeat = i;
                }
            }
            if ( removeat == -1 ) return;

            // Get the base name and try to remove the directory
            if (MessageBox.Show("Remove mission permanently from disk?", "Confirm", MessageBoxButtons.YesNo) == DialogResult.Yes)
            {
                string basename = table.Rows[removeat]["SOL File"].ToString();
                basename = basename.Substring(0, basename.Length - 4);
                // Ensure the validity of the base name
                if (basename.Length > 0 )
                {
                    try
                    {
                        Directory.Delete("missions\\" + basename, true);
                    }
                    catch (Exception e)
                    {
                        MessageBox.Show(e.ToString());
                    }
                    try
                    {
                        File.Delete("sol\\" + basename + ".sol" );
                    }
                    catch (Exception e)
                    {
                    }
                    try
                    {
                        File.Delete("missions\\" + basename + "r.drl" );
                    }
                    catch (Exception e)
                    {
                    }
                    try
                    {
                        File.Delete("missions\\" + basename + "s.dsl" );
                    }
                    catch (Exception except )
                    {
                        MessageBox.Show(except.ToString());                    
                    }
                }
            }

            table.Rows.RemoveAt(removeat);
        }

        public static void Save()
        {
            // Save to a temporary file
            string tempfile = "campaign\\Legacy\\LegacyMissions.xmlx";
            string realfile = "campaign\\Legacy\\LegacyMissions.xml";
            TextWriter tw = new StreamWriter(tempfile);

            tw.WriteLine("<ShellSettings sequence=\"false\" name=\"LegacyMissions\">");
            for (int i = 4; i < table.Rows.Count; i++)
            {
                bool bhero = (table.Rows[i]["Hero"].ToString() != "");
                bool bskip = (table.Rows[i]["Skip Points"].ToString() != "");
                bool bfreeCPs = (table.Rows[i]["Free CPs"].ToString() != "");

                string therest = "\"";
                if (bhero) therest += " hero=\"" + table.Rows[i]["Hero"] + "\"";
                if (bskip) therest += " skipPoints=\"" + table.Rows[i]["Skip Points"] + "\"";
                if (bfreeCPs) therest += " freeCPs=\"" + table.Rows[i]["Free CPs"] + "\"";
                string missionName = table.Rows[i]["Mission Name"].ToString();
                missionName = Globals.RemoveSpaces(missionName);

                tw.WriteLine("\t<" + missionName +
                    " file=\"" + table.Rows[i]["SOL File"] +
                    "\" date=\"" + table.Rows[i]["Date"] +
                    "\" image=\"" + table.Rows[i]["Image"] +
                    "\" numShips=\"" + table.Rows[i]["Num Ships"] +
                    therest + "/>");
            }
            tw.WriteLine("</ShellSettings>");
            tw.Close();
            
            // This was successful. Save it over the main file
            try
            {
                if ( File.Exists(realfile) ) File.Delete(realfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            try
            {
                File.Copy(tempfile, realfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            try
            {
                File.Delete(tempfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
        }

        public static void Build(string missionName)
        {
            try
            {
                DataRow mission = table.Rows[FindMission(missionName)];
                String soutput = "";
                Process process = new Process();
                string baseName = GetBaseName(mission);
                string rawScriptFile = "missions\\" + baseName + "\\" + baseName + "S.scp";
                string rawRuleFile = "missions\\" + baseName + "\\" + baseName + "R.rul";
                string compiledScriptFile = baseName + "\\" + baseName + "S.cpp";
                string compiledRuleFile = baseName + "\\" + baseName + "R.cpp";
                string pathToCompiler = Globals.VSBinPath + "\\cl.exe";
                string pathToScriptMaker = "Parsescr.exe";
                string pathToRuleMaker = "Parserul.exe";

                // First step is to compile the raw script and rule files
                process.StartInfo.UseShellExecute = false;
                process.StartInfo.RedirectStandardOutput = true;
                process.StartInfo.RedirectStandardError = true;
                process.StartInfo.CreateNoWindow = true;
                process.StartInfo.FileName = pathToScriptMaker;
                process.StartInfo.Arguments = rawScriptFile;
                process.Start();
                soutput = process.StandardOutput.ReadToEnd();
                if ((soutput == null) || (soutput == ""))
                {
                    MessageBox.Show("Problem parsing script file. See log for details.", "Error");
                    return;
                }
                else
                {
                    frmResultOutput.text = soutput;
                    new frmResultOutput().ShowDialog();
                }


                process.StartInfo.FileName = pathToRuleMaker;
                process.StartInfo.Arguments = rawRuleFile;
                process.Start();
                soutput = process.StandardOutput.ReadToEnd();
                if ((soutput == null) || (soutput == ""))
                {
                    MessageBox.Show("Problem parsing rule file. See log for details.", "Error");
                }
                else
                {
                    frmResultOutput.text = soutput;
                    new frmResultOutput().ShowDialog();
                }

                // Second step is to compile the generated cpp files
                string cwd = Directory.GetCurrentDirectory();
                Directory.SetCurrentDirectory("missions\\" + baseName);
                string filename = baseName + "s.cpp";
                string outname;
                string inname;
                string libname;

                TextWriter batchFileWriter = new StreamWriter("build.bat");
                batchFileWriter.WriteLine("call \"" + Globals.VSBinPath + "\\vcvars32.bat\"\n");
                filename = baseName + "s.cpp";
                batchFileWriter.WriteLine("cl.exe " + filename + " /O2 /Ob1 /I \"..\\..\\Include\" /D \"WIN32\" /D \"NDEBUG\" /D \"_WINDOWS\" /D \"_WINDLL\" /GF /FD /EHsc /MT /Gy /W3 /nologo /c /errorReport:prompt\n");
                filename = baseName + "s.obj";
                outname = baseName + "s.dsl";
                libname = baseName + "s.lib";
                try { File.Delete(filename); }
                catch (Exception except) { }
                try { File.Delete(libname); }
                catch (Exception except) { }
                batchFileWriter.WriteLine("link.exe " + filename + " /OUT:\"" + outname +
                    "\" /INCREMENTAL:NO /NOLOGO /DLL /SUBSYSTEM:WINDOWS /IMPLIB:\"" + libname +
                    "\" /MACHINE:X86 /ERRORREPORT:PROMPT kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib odbc32.lib odbccp32.lib\n");
                filename = baseName + "r.cpp";
                batchFileWriter.WriteLine("cl.exe " + filename + " /O2 /Ob1 /I \"..\\..\\Include\" /D \"WIN32\" /D \"NDEBUG\" /D \"_WINDOWS\" /D \"_WINDLL\" /GF /FD /EHsc /MT /Gy /W3 /nologo /c /errorReport:prompt\n");
                filename = baseName + "r.obj";
                outname = baseName + "r.drl";
                inname = baseName + "s.lib";
                libname = baseName + "r.lib";
                try { File.Delete(filename); }
                catch (Exception except) { }
                try { File.Delete(libname); }
                catch (Exception except) { }
                batchFileWriter.WriteLine("link.exe " + filename + " /OUT:\"" + outname +
                    "\" /INCREMENTAL:NO /NOLOGO /DLL /SUBSYSTEM:WINDOWS /IMPLIB:\"" + libname + 
                    "\" /MACHINE:X86 /ERRORREPORT:PROMPT kernel32.lib user32.lib gdi32.lib winspool.lib comdlg32.lib advapi32.lib shell32.lib ole32.lib oleaut32.lib uuid.lib odbc32.lib odbccp32.lib \""+inname+"\"");
                batchFileWriter.Close();
                process.StartInfo.FileName = "build.bat";
                process.Start();

                soutput = process.StandardOutput.ReadToEnd();
                frmResultOutput.text = soutput;
                new frmResultOutput().ShowDialog();
                File.Delete("build.bat");

                // Rename the dll files to drl and dsl and move them into the appropriate directory.
                string dsl = "..\\" + baseName + "S.dsl";
                string drl = "..\\" + baseName + "R.drl";
                if (File.Exists(dsl)) File.Delete(dsl);
                if (File.Exists(drl)) File.Delete(drl);
                if (File.Exists(baseName+"s.dsl"))
                    File.Move(baseName + "s.dsl", dsl);
                if (File.Exists(baseName+"r.drl"))
                    File.Move(baseName + "r.drl", drl);

                // Restore the directory
                Directory.SetCurrentDirectory(cwd);

                // Done!
                MessageBox.Show("Finished building mission " + missionName + ".", "Done");
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString(), "An error occured.");
            }
        }

        public static bool ValidateName(string name)
        {
            // Make sure there's no leading or trailing whitespace
            if (name.Trim() != name) return false;

            // Other whitespace is okay, but all other characters must be alphanumeric.
            for (int i = 0; i < name.Length; i++)
            {
                bool bokay = false;
                char c = name[i];
                if (c >= 'A' && c <= 'Z') bokay = true;
                if (c >= 'a' && c <= 'z') bokay = true;
                if (c >= '0' && c <= '9') bokay = true;
                if (c == ' ') bokay = true;
                if (!bokay) return bokay;
            }

            return true;
        }

        public static void Export(int m)
        {
            DataRow row = table.Rows[m];
            string solName = row["SOL File"].ToString();
            string baseName = GetBaseName(row);


            string inFile;
            inFile = "missions\\" + baseName + "R.drl";
            if (!File.Exists(inFile))
            {
                MessageBox.Show("Compiled rule file not found. You must build before exporting.");
                return;
            }
            inFile = "missions\\" + baseName + "S.dsl";
            if (!File.Exists(inFile))
            {
                MessageBox.Show("Compiled script file not found. You must build before exporting.");
            }

            string fileName = row["Mission Name"]+".mission";

            SaveFileDialog ofd = new SaveFileDialog();
            ofd.InitialDirectory = "";
            ofd.Title = "Choose an mission file";
            ofd.Filter = "Mission files (*.mission)|*.mission";
            ofd.FileName = fileName;
            ofd.RestoreDirectory = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                fileName = ofd.FileName;
            }
            else return;

            BinaryWriter writer = null;
            try
            {
                writer = new BinaryWriter(new FileStream(fileName, FileMode.OpenOrCreate));
            }
            catch (Exception except)
            {
                MessageBox.Show(except.ToString());
                return;
            }

            // Write mission attributes
            string s;
            s = row["Mission Name"].ToString();
            Globals.ExportString(writer, row["Mission Name"].ToString());
            Globals.ExportString(writer, row["SOL File"].ToString());
            Globals.ExportString(writer, row["Date"].ToString());
            Globals.ExportString(writer, row["Image"].ToString());
            Globals.ExportString(writer, row["Num Ships"].ToString());
            Globals.ExportString(writer, row["Hero"].ToString());
            Globals.ExportString(writer, row["Skip Points"].ToString());
            Globals.ExportString(writer, row["Free CPs"].ToString());

            // Write the drl, dsl, and sol files
            inFile = "missions\\" + baseName + "R.drl";
            BinaryReader reader = new BinaryReader(new FileStream(inFile, FileMode.Open));
            Globals.ExportFile(writer, reader);
            reader.Close();
            inFile = "missions\\" + baseName + "S.dsl";
            reader = new BinaryReader(new FileStream(inFile, FileMode.Open));
            Globals.ExportFile(writer, reader);
            reader.Close();
            inFile = "sol\\" + baseName + ".sol";
            try
            {
                reader = new BinaryReader(new FileStream(inFile, FileMode.Open));
            }
            catch (Exception except) { reader = null; }
            Globals.ExportFile(writer, reader);
            if ( reader != null ) reader.Close();

            // Write the objectives xml file (as binary) to catch "visible" and "points", etc.
            inFile = "objectives\\" + baseName + "_objectives.xml";
            reader = new BinaryReader(new FileStream(inFile, FileMode.Open));
            Globals.ExportFile(writer, reader);
            reader.Close();

            // Export all mission-specific labels in "Objectives_localized_Strings..."
            Int32 exportCount = 0;
            int r;
            for (r = 0; r < Globals.StringsEN.strings.Rows.Count; r++)
            {
                DataRow exprow = Globals.StringsEN.strings.Rows[r];
                if (exprow["ID"].ToString().Substring(0, 5).ToUpper() == baseName.ToUpper())
                    exportCount++;
            }
            writer.Write(exportCount);
            for (r = 0; r < Globals.StringsEN.strings.Rows.Count; r++)
            {
                DataRow exprow = Globals.StringsEN.strings.Rows[r];
                if (exprow["ID"].ToString().Substring(0, 5).ToUpper() == baseName.ToUpper())
                {
                    Globals.ExportString(writer, exprow["ID"].ToString());
                    Globals.ExportString(writer,exprow["Text"].ToString());
                }
            }

/*            // Write the objective text
            fileName = "objectives\\" + baseName + "_Objectives.xml";
            if (!File.Exists(fileName))
            {
                if (baseName.Length > 5) baseName = baseName.Substring(0, 5);
                fileName = "objectives\\" + baseName + "_Objectives.xml";
                if (!File.Exists(fileName))
                {
                    MessageBox.Show("Could not find objectives file.");
                    return;
                }
            }

            
            frmObjectivesEdit oe = new frmObjectivesEdit();
            oe.SetEditFile(fileName);
            oe.LoadObjectives();
            oe.ExportObjectives(writer);*/

            writer.Close();
        }
    }
}
