using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace LegacyMissionEditor
{
    public partial class frmAddMission : Form
    {
        public static string name;
        public static string basename;
        public static string copyFrom;
        public static string sol;

        public frmAddMission()
        {
            InitializeComponent();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            name = tbMissionName.Text;
            // Validate the mission name
            if ((name == null) || (name.Length == 0) || !Missions.ValidateName( name ) )
            {
                MessageBox.Show("Invalid mission name.");
                name = null;
                return;
            }
            else
            {
                if ( Missions.FindMission( name ) > -1 )
                {
                    MessageBox.Show("Mission already exists.");
                    name = null;
                    return;
                }
            }

            // Find the next available mission
            int i;
            for (i = 12; i < 100; i++)
            {
                string testname = "stl" + i;
                if (!Directory.Exists("missions\\"+testname))
                    if (!File.Exists("missions\\"+testname+"S.dsl"))
                        if (!File.Exists("missions\\"+testname+"R.drl"))
                            break;
            }
            if ( i == 100 ) 
            {
                MessageBox.Show("No mission slots available. Delete a mission first.");
                name = null;
                return;
            }

            basename = "stl"+i;

            DataRow copyRow = Missions.table.Rows[Missions.FindMission(cbCopyFrom.Text)];
            if (copyRow != null)
            {
                // If no sol, ask what sol file to use
                sol = copyRow["SOL File"].ToString();
                if ((sol == null) || (sol.Length < 4))
                {
                    // Choose a sol file from the directory
                    sol = null;
                    while (true)
                    {
                        string file = null;
                        OpenFileDialog ofd = new OpenFileDialog();
                        ofd.InitialDirectory = "sol";
                        ofd.Title = "Choose an sol file";
                        ofd.Filter = "SOL files (*.sol)|*.sol";
                        ofd.RestoreDirectory = true;
                        if (ofd.ShowDialog() == DialogResult.OK)
                        {
                            file = ofd.FileName.Substring(
                                ofd.FileName.LastIndexOf('\\') + 1,
                                ofd.FileName.Length - ofd.FileName.LastIndexOf('\\') - 1);
                        }

                        // Can't use STL maps with default missions
                        if (file.Substring(0, 3).ToUpper() == "STL")
                        {
                            if (cbCopyFrom.SelectedIndex < Missions.DEFAULT_COUNT)
                            {
                                MessageBox.Show("Can't use STL map with non-campaign missions.");
                            }
                            else sol = file;
                        }
                        else sol = file;
                        if (sol != null) break;
                    }
                }
                if ((sol != null) && (sol.Length > 4))
                {
                    copyFrom = Missions.GetBaseName(copyRow);
                    Close();
                }
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            name = null;
            Close();
        }

        private void frmAddMission_Load(object sender, EventArgs e)
        {
            sol = null;
            name = null;
            basename = null;
            copyFrom = null;
            int i;
            for (i = 0; i < Missions.table.Rows.Count; i++)
            {
                DataRow row = Missions.table.Rows[i];
                cbCopyFrom.Items.Add(row["Mission Name"].ToString());
            }
            cbCopyFrom.SelectedIndex = 0;
        }
    }
}