using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.Diagnostics;

namespace LegacyMissionEditor
{
    public partial class frmMissions : Form
    {
        enum MISSION_COLS
        {
             MISSION_COL_NAME = 0,
             MISSION_COL_SOL = 1,
             MISSION_COL_RULE,
             MISSION_COL_SCRIPT,
             MISSION_COL_DATE,
             MISSION_COL_IMAGE,
             MISSION_COL_NUMSHIPS,
             MISSION_COL_HERO,
             MISSION_COL_SKIPPOINTS,
             MISSION_COL_FREECPS
        };


        public frmMissions()
        {
            InitializeComponent();
        }

        private void frmMissions_Load(object sender, EventArgs e)
        {
            dgvMissions.DataSource = Missions.table;
            dgvMissions.Update();
            for (int i = 0; i < dgvMissions.Columns.Count; i++)
            {
                 dgvMissions.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            }

            

        }

        private void dgvMissions_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            DataRow row;
            try
            {
                row = Missions.table.Rows[e.RowIndex];
            }
            catch (Exception exception)
            {
                return;
            }

            if (e.RowIndex < Missions.DEFAULT_COUNT)
            {
                switch ((MISSION_COLS)e.ColumnIndex)
                {
                    case MISSION_COLS.MISSION_COL_RULE:
                        break;
                    case MISSION_COLS.MISSION_COL_SCRIPT:
                        break;
                    default:
                        return;
                }
            }

            if (e.ColumnIndex == -1)
            {
                string solName =
                    row[(int)MISSION_COLS.MISSION_COL_SOL].ToString();
                EditObjectivesFromSol(solName);
                return;
            }

            String cellText =
                row.ItemArray.GetValue(e.ColumnIndex).ToString();

            switch ((MISSION_COLS)e.ColumnIndex)
            {
                case MISSION_COLS.MISSION_COL_NAME:
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter mission name below";
                        string name = dgvMissions.Rows[e.RowIndex].Cells[(int)MISSION_COLS.MISSION_COL_NAME].Value.ToString();
                        frmInputBox.text = name;
                        ib.alphanumeric = true;
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                        {
                            row["Mission Name"] = frmInputBox.text;
                            string mid = row["SOL File"].ToString();
                            // asdf
                            mid = mid.Substring(0, mid.Length - 4);
                            mid = mid.ToUpper();
                            Globals.StringsEN.Set(mid + "_TITLE", frmInputBox.text);
                            Globals.StringsEN.Save();
                            Globals.LabelsEN.Set("mission_select"+row["SOL File"].ToString(),frmInputBox.text+" ");
                            Globals.LabelsEN.Save();
                        }
                    }

                    break;
                case MISSION_COLS.MISSION_COL_SOL:
                    {
                        // Make sure the marker file exists
                        String fileName =
                            "unsupportedextension_mapedit";
                        TextWriter tw = new StreamWriter(fileName);
                        tw.Close();

                        Process process = new Process();
                        process.StartInfo.UseShellExecute = false;
                        process.StartInfo.RedirectStandardOutput = true;
                        process.StartInfo.RedirectStandardError = true;
                        process.StartInfo.CreateNoWindow = true;
                        process.StartInfo.FileName = "Legacy.exe";
                        string name = dgvMissions.Rows[e.RowIndex].Cells[(int)MISSION_COLS.MISSION_COL_SOL].Value.ToString();
                        process.StartInfo.Arguments = "/edit "+name;
                        process.Start();
                        string soutput = process.StandardOutput.ReadToEnd();
                    }
                    break;
                case MISSION_COLS.MISSION_COL_RULE:
                    {
                        // Open the rule editor form
                        string name = dgvMissions.Rows[e.RowIndex].Cells[(int)MISSION_COLS.MISSION_COL_RULE].Value.ToString();
                        frmScriptEdit se = new frmScriptEdit();
                        se.SetEditFile(name);
                        se.ShowDialog();
                    }
                    break;
                case MISSION_COLS.MISSION_COL_SCRIPT:
                    {
                        // Open the script editor form
                        string name = dgvMissions.Rows[e.RowIndex].Cells[(int)MISSION_COLS.MISSION_COL_SCRIPT].Value.ToString();
                        frmScriptEdit se = new frmScriptEdit();
                        se.SetEditFile(name);
                        se.ShowDialog();
                    }
                    break;
                case MISSION_COLS.MISSION_COL_IMAGE:
                    {
                        // Choose an image file from the directory
                        OpenFileDialog ofd = new OpenFileDialog();
                        ofd.InitialDirectory = "campaign\\Legacy\\";
                        ofd.Title = "Choose an image file";
                        ofd.Filter = "DDS files (*.dds)|*.dds";
                        ofd.RestoreDirectory = true;
                        if (ofd.ShowDialog() == DialogResult.OK)
                        {
                            string file = ofd.FileName.Substring(
                                ofd.FileName.LastIndexOf('\\')+1,
                                ofd.FileName.Length - ofd.FileName.LastIndexOf('\\')-1);
                                row["Image"] = file.Substring(0, file.Length - 4);
                        }
                    }
                    break;
                case MISSION_COLS.MISSION_COL_NUMSHIPS:
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter number of ships below";
                        frmInputBox.text = row["Num Ships"].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                        {
                            int i;
                            if (int.TryParse(frmInputBox.text, out i))
                            {
                                row["Num Ships"] = frmInputBox.text;
                            }
                        }
                    }
                    break;
                case MISSION_COLS.MISSION_COL_DATE:
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter date below";
                        frmInputBox.text = row["Date"].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                        {
                            int i;
                            if (int.TryParse(frmInputBox.text, out i))
                            {
                                row["Date"] = frmInputBox.text;
                            }
                        }
                    }
                    break;
                case MISSION_COLS.MISSION_COL_HERO:
                    {
                        // Choose an image file from the directory
                        OpenFileDialog ofd = new OpenFileDialog();
                        ofd.InitialDirectory = "odf\\ships";
                        ofd.Title = "Choose an odf file";
                        ofd.Filter = "ODF files (*.odf)|*.odf";
                        ofd.RestoreDirectory = true;
                        if (ofd.ShowDialog() == DialogResult.OK)
                        {
                            string file = ofd.FileName.Substring(
                                ofd.FileName.LastIndexOf('\\') + 1,
                                ofd.FileName.Length - ofd.FileName.LastIndexOf('\\') - 1);
                            row["Hero"] = file;
                        }
                    }
                    break;
                case MISSION_COLS.MISSION_COL_SKIPPOINTS:
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter skip points below";
                        frmInputBox.text = row["Skip Points"].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                        {
                            int i;
                            if (int.TryParse(frmInputBox.text, out i))
                            {
                                row["Skip Points"] = frmInputBox.text;
                            }
                        }
                    }
                    break;
                case MISSION_COLS.MISSION_COL_FREECPS:
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter free CPs below";
                        frmInputBox.text = row["Free CPs"].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                        {
                            int i;
                            if (int.TryParse(frmInputBox.text, out i))
                            {
                                row["Free CPs"] = frmInputBox.text;
                            }
                        }
                    }
                    break;
            }
            Missions.Save();
        }

        private void dgvMissions_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Right)
            {
            }
            dgvMissions.Update();
        }

        private void dgvMissions_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {

        }

        private void createNewMissionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAddMission form = new frmAddMission();
            form.ShowDialog();
            if ( frmAddMission.name != null )
                Missions.CreateNew(frmAddMission.name,frmAddMission.basename,frmAddMission.copyFrom);
            Missions.Save();
        }

        private void deleteMissionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ( (dgvMissions.SelectedRows.Count != 1) && ( dgvMissions.SelectedCells.Count != 1 ) )
            {
                MessageBox.Show("Select one mission from the table below.");
            }
            else
            {
                string name;
                DataGridViewRow row;
                int rowindex = -1;
                if (dgvMissions.SelectedRows.Count == 1)
                    rowindex = dgvMissions.SelectedRows[0].Index;
                else
                    rowindex = dgvMissions.SelectedCells[0].RowIndex;
                if (rowindex < Missions.DEFAULT_COUNT) return;

                row = dgvMissions.Rows[rowindex];
                name = row.Cells[0].Value.ToString();
                if (MessageBox.Show("Are you sure you want to delete " + name + "?", "Are you sure?", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    Missions.Delete(name);
                    Missions.Save();
                }
            }
        }

        private void editObjectivesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((dgvMissions.SelectedRows.Count != 1) && (dgvMissions.SelectedCells.Count != 1))
            {
                MessageBox.Show("Select one mission from the table below.");
            }
            else
            {
                DataGridViewRow row;
                if (dgvMissions.SelectedRows.Count == 1)
                    row = dgvMissions.SelectedRows[0];
                else
                    row = dgvMissions.Rows[dgvMissions.SelectedCells[0].RowIndex];
                string solName = 
                    row.Cells[(int)MISSION_COLS.MISSION_COL_SOL].Value.ToString();
                if ( solName != "" ) EditObjectivesFromSol(solName);
            }
        }

        void EditObjectivesFromSol(string solName)
        {
            string baseName = solName.Substring(0, solName.Length - 4);
            string fileName = "objectives\\" + baseName + "_Objectives.xml";
            if (!File.Exists(fileName))
            {
                if (baseName.Length > 5) baseName = baseName.Substring(0, 5);
                fileName = "objectives\\" + baseName + "_Objectives.xml";
                if (!File.Exists(fileName))
                {
                    MessageBox.Show("Could not find objectives file.");
                    return;
                }
            }
            frmObjectivesEdit oe = new frmObjectivesEdit();
            oe.SetEditFile(fileName);
            oe.ShowDialog();
            Missions.Save();
        }

        private void buildToolStripMenuItem_Click_1(object sender, EventArgs e)
        {
            if ((dgvMissions.SelectedRows.Count != 1) && (dgvMissions.SelectedCells.Count != 1))
            {
                MessageBox.Show("Select one mission from the table below.");
            }
            else
            {
                DataGridViewRow row;
                if (dgvMissions.SelectedRows.Count == 1)
                    row = dgvMissions.SelectedRows[0];
                else
                    row = dgvMissions.Rows[dgvMissions.SelectedCells[0].RowIndex];
                Missions.Build( row.Cells[0].Value.ToString() );
            }
        }

        private void createBackupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Back up all files used by this program
            if ( MessageBox.Show("This will overwrite any current backup. Continue?","Warning",MessageBoxButtons.YesNo) == DialogResult.Yes )
            {
                TextWriter batFileWriter = new StreamWriter("backup.bat");
                batFileWriter.WriteLine("mkdir missioneditorbackup");
                batFileWriter.WriteLine("copy missions missioneditorbackup");
                batFileWriter.WriteLine("copy objectives missioneditorbackup");
                batFileWriter.WriteLine("copy campaign missioneditorbackup");
                batFileWriter.WriteLine("copy Objectives_Localized_Strings_EN.xml missioneditorbackup");
                batFileWriter.WriteLine("del backup.bat");
                batFileWriter.Close();
                Process proc = new Process();
                proc = Process.Start("backup.bat");
            }
        }

        private void restoreFromBackupToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // Restore all files used by this program
            if ( MessageBox.Show("This will overwrite any current files. Continue?","Warning",MessageBoxButtons.YesNo) == DialogResult.Yes )
            {
                TextWriter batFileWriter = new StreamWriter("restore.bat");
                batFileWriter.WriteLine("copy missioneditorbackup\\missions .");
                batFileWriter.WriteLine("copy missioneditorbackup\\objectives .");
                batFileWriter.WriteLine("copy missioneditorbackup\\campaign .");
                batFileWriter.WriteLine("copy missioneditorbackup\\Objectives_Localized_Strings_EN.xml .");
                batFileWriter.WriteLine("del restore.bat");
                batFileWriter.Close();
                Process proc = new Process();
                proc = Process.Start("restore.bat");
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void configurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmSettings form = new frmSettings();
            form.ShowDialog();
        }

        private void frmMissions_ResizeEnd(object sender, EventArgs e)
        {
            dgvMissions.Width = Width - 5;
            dgvMissions.Height = Height - menuStrip1.Height - 10;
            dgvMissions.Update();
        }

        private void frmMissions_SizeChanged(object sender, EventArgs e)
        {
            dgvMissions.Width = Width - 5;
            dgvMissions.Height = Height - menuStrip1.Height - 10;
            dgvMissions.Update();
        }

        private void dgvMissions_KeyUp(object sender, KeyEventArgs e)
        {
            switch ( e.KeyCode )
            {
                // New mission
                case Keys.Insert:
                    createNewMissionToolStripMenuItem_Click(sender, (EventArgs)e);
                    break;
                case Keys.Delete:
                    deleteMissionToolStripMenuItem_Click(sender, (EventArgs)e);
                    break;
            }
        }

        private void documentationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process proc = Process.Start("http://stl.bethsoft.com");
        }

        private void testToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Globals.BinarySearchReplace("sol\\stl12.sol", "stl12", "stlll");
        }

        private void runLegacyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process proc = Process.Start("Legacy.exe");
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmAbout about = new frmAbout();
            about.ShowDialog();
        }

        private void bethesdaStarTrekWebsiteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Process proc = Process.Start("http://startrek.bethsoft.com/home/home.html");
        }

        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if ((dgvMissions.SelectedRows.Count != 1) && (dgvMissions.SelectedCells.Count != 1))
            {
                MessageBox.Show("Select one mission from the table below.");
            }
            else
            {
                DataGridViewRow row;
                if (dgvMissions.SelectedRows.Count == 1)
                    row = dgvMissions.SelectedRows[0];
                else
                    row = dgvMissions.Rows[dgvMissions.SelectedCells[0].RowIndex];
                if (row.Index >= Missions.DEFAULT_COUNT)
                    Missions.Export(row.Index);
                else
                    MessageBox.Show("This mission cannot be exported. In order to share it, copy its .drl and .dsl files or create a new mission using it as a template and export that one.");
            }
        }

        private void frmMissions_AutoSizeChanged(object sender, EventArgs e)
        {

        }
    }
}