using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace LegacyMissionEditor
{
    public partial class frmObjectivesEdit : Form
    {
        string editFile;
        ContextMenu popupMenu;
        MenuItem popupAdd;
        MenuItem popupDelete;
        string missionName;
        DataTable descriptions;
        DataTable objectives_primary;
        DataTable objectives_secondary;
        DataTable hints;

        enum MenuSource
        {
            MS_DESC = 0,
            MS_OBJ_PRI,
            MS_OBJ_SEC,
            MS_HINT
        };

        public void SetEditFile(string file)
        {
            editFile = file;
        }

        public frmObjectivesEdit()
        {
            InitializeComponent();
        }

        enum ReadOrder
        {
            Reading_Name = 0,
            Reading_Descriptions = 1,
            Reading_Primary_Objectives = 2,
            Reading_Secondary_Objectives = 3,
            Reading_Hints = 4
        };

        public void LoadObjectives()
        {
            ReadOrder readPos = ReadOrder.Reading_Name;

            descriptions = new DataTable();
            descriptions.Columns.Add("Name");
            descriptions.Columns.Add("Text");
            objectives_primary = new DataTable();
            objectives_primary.Columns.Add("Visible");
            objectives_primary.Columns.Add("Points");
            objectives_primary.Columns.Add("Name");
            objectives_primary.Columns.Add("Text");
            objectives_secondary = new DataTable();
            objectives_secondary.Columns.Add("Visible");
            objectives_secondary.Columns.Add("Points");
            objectives_secondary.Columns.Add("Name");
            objectives_secondary.Columns.Add("Text");
            hints = new DataTable();
            hints.Columns.Add("Visible");
            hints.Columns.Add("Name");
            hints.Columns.Add("Text");

            // Load the objectives if the file exists
            XmlTextReader reader = new XmlTextReader(editFile);
            if (reader != null)
            {
                DataRow row = null;
                while (reader != null && reader.Read())
                {
                    if (reader.Name == "Descriptions")
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                            readPos = ReadOrder.Reading_Descriptions;
                        continue;
                    }
                    if (readPos == ReadOrder.Reading_Name)
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                            missionName = reader.GetAttribute("MissionName");
                    }

                    if (reader.Name == "Primary")
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                            readPos = ReadOrder.Reading_Primary_Objectives;
                        continue;
                    }
                    if (reader.Name == "Secondary")
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                            readPos = ReadOrder.Reading_Secondary_Objectives;
                        continue;
                    }
                    if (reader.Name == "Hints")
                    {
                        if (reader.NodeType == XmlNodeType.Element)
                            readPos = ReadOrder.Reading_Hints;
                        continue;
                    }

                    switch (readPos)
                    {
                        case ReadOrder.Reading_Descriptions:
                            if (reader.NodeType == XmlNodeType.Text)
                            {
                                row = descriptions.NewRow();
                                descriptions.Rows.Add(row);
                                row["Name"] = reader.Value;
                                try
                                {
                                    row["Text"] = Globals.StringsEN.strings.Rows.Find(reader.Value)["Text"];
                                }
                                catch (Exception except) { }
                                if (row["Text"].ToString() == "")
                                    row["Text"] = "New Description)";
                            }
                            break;
                        case ReadOrder.Reading_Primary_Objectives:
                            {
                                if (reader.NodeType == XmlNodeType.Element)
                                {
                                    row = objectives_primary.NewRow();
                                    objectives_primary.Rows.Add(row);
                                    row["Visible"] = reader.GetAttribute("visible");
                                    row["Points"] = reader.GetAttribute("points");
                                }
                                if (reader.NodeType == XmlNodeType.Text)
                                {
                                    row["Name"] = reader.Value;
                                    try
                                    {
                                        row["Text"] = Globals.StringsEN.strings.Rows.Find(reader.Value)["Text"];
                                    }
                                    catch (Exception except) { }
                                    if (row["Text"].ToString() == "")
                                        row["Text"] = "New Objective";
                                }
                            }
                            break;
                        case ReadOrder.Reading_Secondary_Objectives:
                            {
                                if (reader.NodeType == XmlNodeType.Element)
                                {
                                    row = objectives_secondary.NewRow();
                                    objectives_secondary.Rows.Add(row);
                                    row["Visible"] = reader.GetAttribute("visible");
                                    row["Points"] = reader.GetAttribute("points");
                                }
                                if (reader.NodeType == XmlNodeType.Text)
                                {
                                    row["Name"] = reader.Value;
                                    try
                                    {
                                        row["Text"] = Globals.StringsEN.strings.Rows.Find(reader.Value)["Text"];
                                    }
                                    catch (Exception except) { }
                                    if (row["Text"].ToString() == "")
                                        row["Text"] = "New Objective";
                                }
                            }
                            break;
                        case ReadOrder.Reading_Hints:
                            if (reader.NodeType == XmlNodeType.Element)
                            {
                                row = hints.NewRow();
                                hints.Rows.Add(row);
                                row["Visible"] = reader.GetAttribute("visible");
                            }
                            if (reader.NodeType == XmlNodeType.Text)
                            {
                                row["Name"] = reader.Value;
                                try
                                {
                                    row["Text"] = Globals.StringsEN.strings.Rows.Find(reader.Value)["Text"];
                                }
                                catch (Exception except) { }
                                if (row["Text"].ToString() == "")
                                    row["Text"] = "New Hint";
                            }
                            break;
                    }
                }
                reader.Close();
            }
        }

        private void frmObjectivesEdit_Load(object sender, EventArgs e)
        {
            LoadObjectives();
            dgvDescriptions.DataSource = descriptions;
            dgvDescriptions.Columns["Text"].Width *= 4;
            dgvDescriptions.Update();
            dgvObjectivesPrimary.DataSource = objectives_primary;
            dgvObjectivesPrimary.Update();
            dgvObjectivesPrimary.Columns["Text"].Width *= 4;
            dgvObjectivesSecondary.DataSource = objectives_secondary;
            dgvObjectivesSecondary.Update();
            dgvObjectivesSecondary.Columns["Text"].Width *= 4;
            dgvHints.DataSource = hints;
            dgvHints.Update();
            dgvHints.Columns["Text"].Width *= 4;

            int i;
            for (i = 0; i < dgvDescriptions.Columns.Count; i++)
                dgvDescriptions.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            for (i = 0; i < dgvObjectivesPrimary.Columns.Count; i++)
                dgvObjectivesPrimary.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            for (i = 0; i < dgvObjectivesSecondary.Columns.Count; i++)
                dgvObjectivesSecondary.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
            for (i = 0; i < dgvHints.Columns.Count; i++)
                dgvHints.Columns[i].SortMode = DataGridViewColumnSortMode.NotSortable;
        }

        public void SaveObjectives()
        {
            int i, cur, count;
            TextWriter tw = new StreamWriter(editFile+"x");
            string[] splits = editFile.Split('\\');
            string baseName = splits[1].Substring(0, splits[1].Length-4);
            tw.WriteLine("<"+baseName+" MissionName=\""+missionName+"\">");
            tw.WriteLine("\t<Descriptions initialDescription=\"0\">");
            for ( i = 0; i < descriptions.Rows.Count; i++ )
            {
                string id = "Description" + i.ToString();
                tw.WriteLine("\t\t<"+id+">" + descriptions.Rows[i]["Name"] + "</"+id+">", i, i);
            }
            tw.WriteLine("\t</Descriptions>");
            tw.WriteLine("\t<Objectives>");

            if (objectives_primary.Rows.Count > 0)
            {
                tw.WriteLine("\t\t<Primary>");
                for (i = 0; i < objectives_primary.Rows.Count; i++)
                {
                    string id = "Primary" + i.ToString();
                    string visibility = "";
                    if (objectives_primary.Rows[i]["Visible"].ToString().ToLower() == "true")
                        visibility = " visible=\"true\"";
                    if (objectives_primary.Rows[i]["Visible"].ToString().ToLower() == "false")
                        visibility = " visible=\"false\"";

                    string points = "";
                    if (objectives_primary.Rows[i]["Points"].ToString().ToLower().Length > 0)
                        points = " points=\"" + objectives_primary.Rows[i]["Points"].ToString()+ "\"";

                    tw.WriteLine("\t\t\t<" + id + visibility + points + ">" +
                        objectives_primary.Rows[i]["Name"] + "</" + id + ">");
                }
                tw.WriteLine("\t\t</Primary>");
            }
            else
                tw.WriteLine("\t\t<Primary/>");

            if (objectives_secondary.Rows.Count > 0)
            {
                tw.WriteLine("\t\t<Secondary>");
                for (i = 0; i < objectives_secondary.Rows.Count; i++)
                {
                    string id = "Secondary" + i.ToString();

                    string visibility = "";
                    if  ( objectives_secondary.Rows[i]["Visible"].ToString().ToLower() == "true" )
                        visibility = " visible=\"true\"";
                    if (objectives_secondary.Rows[i]["Visible"].ToString().ToLower() == "false")
                        visibility = " visible=\"false\"";

                    string points = "";
                    if (objectives_secondary.Rows[i]["Points"].ToString().ToLower().Length > 0 )
                        points = " points=\""+objectives_secondary.Rows[i]["Points"].ToString()+"\"";

                    tw.WriteLine("\t\t\t<" + id + visibility + points + ">" +
                        objectives_secondary.Rows[i]["Name"] + "</" + id + ">");
                }
                tw.WriteLine("\t\t</Secondary>");
            }
            else
                tw.WriteLine("\t\t<Secondary/>");

            tw.WriteLine("\t</Objectives>");
            tw.WriteLine("\t<Hints>");
            for ( i = 0; i < hints.Rows.Count; i++ )
            {
                string id = "Hint" + i.ToString();
                tw.WriteLine("\t\t<"+id+" visible=\""+hints.Rows[i]["Visible"]+"\">" + hints.Rows[i]["Name"] + "</"+id+">", i, i);
            }
            tw.WriteLine("\t</Hints>");
            tw.WriteLine("</"+baseName+">");
            tw.Close();

            try
            {
                File.Delete(editFile);
            }
            catch (Exception exception)
            {
            }

            File.Move(editFile + "x", editFile);
            File.Delete(editFile + "x");
        }

        private void frmObjectivesEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
        }

/*        private void dgvDescriptions_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            // Make sure the format is correct
            if (e.ColumnIndex == 0)
            {
                string[] splits = editFile.Split('\\');
                string baseName = splits[1].Substring(0, splits[1].Length-4);
                string match = baseName.ToUpper() + "_DESCRIPTIONS_DESCRIPTION"+e.RowIndex.ToString();
                if (descriptions.Rows[e.RowIndex][0].ToString() != match) e.Cancel = true;
            }
        }

        private void dgvObjectives_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            // Make sure the format is correct
            if (e.ColumnIndex == 0)
            {
                if (descriptions.Rows[e.RowIndex][0].ToString().ToLower() != "primary" &&
                     descriptions.Rows[e.RowIndex][0].ToString().ToLower() != "secondary") e.Cancel = true;
            }

            if (e.ColumnIndex == 3)
            {
                string[] splits = editFile.Split('\\');
                string baseName = splits[1].Substring(0, splits[1].Length - 4);
                string match;
                string primary = objectives.Rows[e.RowIndex][0].ToString();
                if ( primary.ToLower() == "primary" )
                    match = baseName.ToUpper() + "_OBJECTIVES_PRIMARY_PRIMARY" + e.RowIndex.ToString();
                else
                    match = baseName.ToUpper() + "_OBJECTIVES_SECONDARY_SECONDARY" + e.RowIndex.ToString();
                if (descriptions.Rows[e.RowIndex][0].ToString() != match) e.Cancel = true;
            }
        }

        private void dgvHints_CellValidating(object sender, DataGridViewCellValidatingEventArgs e)
        {
            // Make sure the format is correct
            if (e.ColumnIndex == 0)
            {
                string[] splits = editFile.Split('\\');
                string baseName = splits[1].Substring(0, splits[1].Length - 4);
                string match = baseName.ToUpper() + "_HINTS_HINT" + e.RowIndex.ToString();
                if (descriptions.Rows[e.RowIndex][0].ToString() != match) e.Cancel = true;
            }
        }*/

        private void dgvHints_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            switch (e.ColumnIndex)
            {
                case 0: // visible
                    if ( hints.Rows[e.RowIndex][0].ToString() == "true" )
                        hints.Rows[e.RowIndex][0] = "false";
                    else
                        hints.Rows[e.RowIndex][0] = "true";
                    break;
                case 1: // name
                    // do nothing
                    break;
                case 2: // description
                    frmLargeInputBox ib = new frmLargeInputBox();
                    frmLargeInputBox.prompt = "Enter new text below";
                    frmLargeInputBox.defaultText = hints.Rows[e.RowIndex][2].ToString();
                    ib.ShowDialog();
                    if ( frmLargeInputBox.text != null )
                    {
                        Globals.StringsEN.Set(hints.Rows[e.RowIndex][1].ToString(), frmLargeInputBox.text);
                            hints.Rows[e.RowIndex][2] = frmLargeInputBox.text;
                    }
                    break;
            }
        }

        private void dgvObjectivesPrimary_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            switch (e.ColumnIndex)
            {
                case 0: // visible
                    if ( objectives_primary.Rows[e.RowIndex][0].ToString() == "true" )
                        objectives_primary.Rows[e.RowIndex][0] = "false";
                    else
                        objectives_primary.Rows[e.RowIndex][0] = "true";
                    break;
                case 1: // points
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter new points value below";
                        frmInputBox.text = objectives_primary.Rows[e.RowIndex][1].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                            objectives_primary.Rows[e.RowIndex][1] = frmInputBox.text;
                    }
                    break;
                case 2: // name
                    // do nothing
                    break;
                case 3: // description
                    {
                        frmLargeInputBox ib = new frmLargeInputBox();
                        frmLargeInputBox.prompt = "Enter new text below";
                        frmLargeInputBox.defaultText = objectives_primary.Rows[e.RowIndex][3].ToString();
                        ib.ShowDialog();
                        if (frmLargeInputBox.text != null)
                        {
                            Globals.StringsEN.Set(objectives_primary.Rows[e.RowIndex][2].ToString(), frmLargeInputBox.text);
                            objectives_primary.Rows[e.RowIndex][3] = frmLargeInputBox.text;
                        }
                    }
                    break;
            }
        }

        private void dgvObjectivesSecondary_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            switch (e.ColumnIndex)
            {
                case 0: // visible
                    if (objectives_secondary.Rows[e.RowIndex][0].ToString() == "true")
                        objectives_secondary.Rows[e.RowIndex][0] = "false";
                    else
                        objectives_secondary.Rows[e.RowIndex][0] = "true";
                    break;
                case 1: // points
                    {
                        frmInputBox ib = new frmInputBox();
                        frmInputBox.prompt = "Enter new points value below";
                        frmInputBox.text = objectives_secondary.Rows[e.RowIndex][1].ToString();
                        ib.ShowDialog();
                        if (frmInputBox.text != null)
                            objectives_secondary.Rows[e.RowIndex][1] = frmInputBox.text;
                    }
                    break;
                case 2: // name
                    // do nothing
                    break;
                case 3: // description
                    {
                        frmLargeInputBox ib = new frmLargeInputBox();
                        frmLargeInputBox.prompt = "Enter new text below";
                        frmLargeInputBox.defaultText = objectives_secondary.Rows[e.RowIndex][3].ToString();
                        ib.ShowDialog();
                        if (frmLargeInputBox.text != null)
                        {
                            Globals.StringsEN.Set(objectives_secondary.Rows[e.RowIndex][2].ToString(), frmLargeInputBox.text);
                            objectives_secondary.Rows[e.RowIndex][3] = frmLargeInputBox.text;
                        }
                    }
                    break;
            }
        }

        private void dgvDescriptions_CellDoubleClick(object sender, DataGridViewCellEventArgs e)
        {
            switch (e.ColumnIndex)
            {
                case 0: // name
                    // do nothing
                    break;
                case 1: // description
                    frmLargeInputBox ib = new frmLargeInputBox();
                    frmLargeInputBox.prompt = "Enter new text below";
                    frmLargeInputBox.defaultText = descriptions.Rows[e.RowIndex][1].ToString();
                    ib.ShowDialog();
                    if (frmLargeInputBox.text != null)
                    {
                        Globals.StringsEN.Set(descriptions.Rows[e.RowIndex][0].ToString(), frmLargeInputBox.text);
                        descriptions.Rows[e.RowIndex][1] = frmLargeInputBox.text;
                    }
                    break;
            }
        }

        private void dgvDescriptions_KeyUp(object sender, KeyEventArgs e)
        {
            string[] splits = editFile.Split('\\');
            string baseName = splits[1].Substring(0, splits[1].Length - 4);
            switch (e.KeyCode)
            {
            case Keys.Insert:
                // Add a new description
                DataRow row = descriptions.NewRow();
                descriptions.Rows.Add(row);
                row["Name"] = baseName.ToUpper() + "_DESCRIPTIONS_DESCRIPTION" + (descriptions.Rows.Count-1);
                Globals.StringsEN.Set(row["Name"].ToString(), "New description");
                row["Text"] = Globals.StringsEN.Get(row["Name"].ToString());
                break;
            case Keys.Delete:
                // Delete the selected description
                int rowIndex = -1;
                if (dgvDescriptions.SelectedCells.Count == 1)
                    rowIndex = dgvDescriptions.SelectedCells[0].RowIndex;
                else if (dgvDescriptions.SelectedRows.Count == 1)
                    rowIndex = dgvDescriptions.SelectedRows[0].Index;
                if (rowIndex > -1)
                {
                    // Iterate through rows after it, moving their text down
                    for (int i = rowIndex; i < descriptions.Rows.Count - 1; i++)
                    {
                        string setid = baseName.ToUpper() + "_DESCRIPTIONS_DESCRIPTION" + (i);
                        string getid = baseName.ToUpper() + "_DESCRIPTIONS_DESCRIPTION" + (i + 1);
                        Globals.StringsEN.Set(setid, Globals.StringsEN.Get(getid));
                        descriptions.Rows[i]["Text"] = Globals.StringsEN.Get(setid);
                    }
                    // Delete the last row
                    descriptions.Rows.RemoveAt(descriptions.Rows.Count - 1);
                }
                break;
            }
        }

        private void dgvObjectivesPrimary_KeyUp(object sender, KeyEventArgs e)
        {
            string[] splits = editFile.Split('\\');
            string baseName = splits[1].Substring(0, splits[1].Length - 4);
            switch (e.KeyCode)
            {
                case Keys.Insert:
                    // Add a new objective
                    DataRow row = objectives_primary.NewRow();
                    objectives_primary.Rows.Add(row);
                    row["Name"] = baseName.ToUpper() + "_OBJECTIVES_PRIMARY_PRIMARY" + (objectives_primary.Rows.Count - 1);
                    Globals.StringsEN.Set(row["Name"].ToString(), "New objective");
                    row["Text"] = Globals.StringsEN.Get(row["Name"].ToString());
                    row["Visible"] = "true";
                    break;
                case Keys.Delete:
                    // Delete the selected objective
                    int rowIndex = -1;
                    if (dgvObjectivesPrimary.SelectedCells.Count == 1)
                        rowIndex = dgvObjectivesPrimary.SelectedCells[0].RowIndex;
                    else if (dgvObjectivesPrimary.SelectedRows.Count == 1)
                        rowIndex = dgvObjectivesPrimary.SelectedRows[0].Index;
                    if (rowIndex > -1)
                    {
                        // Iterate through rows after it, moving their text down
                        for (int i = rowIndex; i < objectives_primary.Rows.Count - 1; i++)
                        {
                            string setid = baseName.ToUpper() + "_OBJECTIVES_PRIMARY_PRIMARY" + (i);
                            string getid = baseName.ToUpper() + "_OBJECTIVES_PRIMARY_PRIMARY" + (i + 1);
                            Globals.StringsEN.Set(setid, Globals.StringsEN.Get(getid));
                            objectives_primary.Rows[i]["Text"] = Globals.StringsEN.Get(setid);
                            objectives_primary.Rows[i]["Visible"] = objectives_primary.Rows[i + 1]["Visible"];
                            objectives_primary.Rows[i]["Points"] = objectives_primary.Rows[i + 1]["Points"];
                        }
                        // Delete the last row
                        objectives_primary.Rows.RemoveAt(objectives_primary.Rows.Count - 1);
                    }
                    break;
            }
        }

        private void dgvObjectivesSecondary_KeyUp(object sender, KeyEventArgs e)
        {
            string[] splits = editFile.Split('\\');
            string baseName = splits[1].Substring(0, splits[1].Length - 4);
            switch (e.KeyCode)
            {
                case Keys.Insert:
                    // Add a new objective
                    DataRow row = objectives_secondary.NewRow();
                    objectives_secondary.Rows.Add(row);
                    row["Name"] = baseName.ToUpper() + "_OBJECTIVES_SECONDARY_SECONDARY" + (objectives_secondary.Rows.Count - 1);
                    Globals.StringsEN.Set(row["Name"].ToString(), "New objective");
                    row["Text"] = Globals.StringsEN.Get(row["Name"].ToString());
                    row["Visible"] = "true";
                    break;
                case Keys.Delete:
                    // Delete the selected objective
                    int rowIndex = -1;
                    if (dgvObjectivesSecondary.SelectedCells.Count == 1)
                        rowIndex = dgvObjectivesSecondary.SelectedCells[0].RowIndex;
                    else if (dgvObjectivesSecondary.SelectedRows.Count == 1)
                        rowIndex = dgvObjectivesSecondary.SelectedRows[0].Index;
                    if (rowIndex > -1)
                    {
                        // Iterate through rows after it, moving their text down
                        for (int i = rowIndex; i < objectives_secondary.Rows.Count - 1; i++)
                        {
                            string setid = baseName.ToUpper() + "_OBJECTIVES_SECONDARY_SECONDARY" + (i);
                            string getid = baseName.ToUpper() + "_OBJECTIVES_SECONDARY_SECONDARY" + (i + 1);
                            Globals.StringsEN.Set(setid, Globals.StringsEN.Get(getid));
                            objectives_secondary.Rows[i]["Text"] = Globals.StringsEN.Get(setid);
                            objectives_secondary.Rows[i]["Visible"] = objectives_secondary.Rows[i+1]["Visible"];
                            objectives_secondary.Rows[i]["Points"] = objectives_secondary.Rows[i + 1]["Points"];
                        }
                        // Delete the last row
                        objectives_secondary.Rows.RemoveAt(objectives_secondary.Rows.Count - 1);
                    }
                    break;
            }
        }

        private void dgvHints_KeyUp(object sender, KeyEventArgs e)
        {
            string[] splits = editFile.Split('\\');
            string baseName = splits[1].Substring(0, splits[1].Length - 4);
            switch (e.KeyCode)
            {
                case Keys.Insert:
                    // Add a new hint
                    DataRow row = hints.NewRow();
                    hints.Rows.Add(row);
                    row["Name"] = baseName.ToUpper() + "_HINTS_HINT" + (hints.Rows.Count - 1);
                    Globals.StringsEN.Set(row["Name"].ToString(), "New hint");
                    row["Text"] = Globals.StringsEN.Get(row["Name"].ToString());
                    row["Visible"] = "true";
                    break;
                case Keys.Delete:
                    // Delete the selected hint
                    int rowIndex = -1;
                    if (dgvHints.SelectedCells.Count == 1)
                        rowIndex = dgvHints.SelectedCells[0].RowIndex;
                    else if (dgvHints.SelectedRows.Count == 1)
                        rowIndex = dgvHints.SelectedRows[0].Index;
                    if (rowIndex > -1)
                    {
                        // Iterate through rows after it, moving their text down
                        for (int i = rowIndex; i < hints.Rows.Count - 1; i++)
                        {
                            string setid = baseName.ToUpper() + "_HINTS_HINT" + (i);
                            string getid = baseName.ToUpper() + "_HINTS_HINT" + (i + 1);
                            Globals.StringsEN.Set(setid, Globals.StringsEN.Get(getid));
                            hints.Rows[i]["Text"] = Globals.StringsEN.Get(setid);
                            hints.Rows[i]["Visible"] = hints.Rows[i + 1]["Visible"];
                        }
                        // Delete the last row
                        hints.Rows.RemoveAt(hints.Rows.Count - 1);
                    }
                    break;
            }
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Globals.StringsEN.Load(null);
            Close();
        }

        private void btnOK_Click(object sender, EventArgs e)
        {
            SaveObjectives();
            Globals.StringsEN.Save();
            Close();
        }

        public void ExportObjectives(BinaryWriter bw)
        {
            int i;
            Int32 count;
            
            // Write the counts
            count = descriptions.Rows.Count;
            bw.Write(count);
            count = objectives_primary.Rows.Count;
            bw.Write(count);
            count = objectives_secondary.Rows.Count;
            bw.Write(count);
            count = hints.Rows.Count;
            bw.Write(count);

            for (i = 0; i < descriptions.Rows.Count; i++)
            {
                Globals.ExportString(bw, descriptions.Rows[i]["Text"].ToString());
            }
            
            for (i = 0; i < objectives_primary.Rows.Count; i++)
            {
                Globals.ExportString(bw, objectives_primary.Rows[i]["Text"].ToString());
            }

            for (i = 0; i < objectives_secondary.Rows.Count; i++)
            {
                Globals.ExportString(bw, objectives_secondary.Rows[i]["Text"].ToString());
            }

            for (i = 0; i < hints.Rows.Count; i++)
            {
                Globals.ExportString(bw, hints.Rows[i]["Text"].ToString());
            }
         
        }

    }
}