using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

namespace LegacyMissionEditor
{
    public partial class frmScriptEdit : Form
    {
        string missionspath = "missions\\";
        string editText;
        string realEditFile;

        bool isRuleFile = false;
        bool isScriptFile = false;
        bool noCloseVerify = false;

        public frmScriptEdit()
        {
            InitializeComponent();
        }

        string editFile;

        public void SetEditFile(string file)
        {
            editFile = file;
        }

        private void richTextBox1_TextChanged(object sender, EventArgs e)
        {

        }

        private void frmScriptEdit_Load(object sender, EventArgs e)
        {
            Text = editFile;
            
            // Load the source for this file
            string dir = editFile.Substring(0, editFile.Length - 5);
            isRuleFile = editFile.Contains(".rul");
            realEditFile = missionspath + dir + "\\" + (isRuleFile ? dir + "R.rul" : dir + "S.scp");

            try
            {
                TextReader tr = new StreamReader(realEditFile);
                String line;
                while ((line = tr.ReadLine()) != null)
                {
                    editText += line;
                    editText += "\n";
                }
                tr.Close();
                tbScriptText.Text = editText;
            }
            catch (Exception except)
            {
                MessageBox.Show("Failed to load file.");
                noCloseVerify = true;
                Close();
            }
        }

        private void btnCompile_Click(object sender, EventArgs e)
        {
            string path = editFile.Substring(0, editFile.Length - 5);
            string scriptfile = missionspath + path + "\\" + path + "S.scp";
            string rulefile = missionspath + path + "\\" + path + "R.rul";

            // Save to this file
            TextWriter tw = new StreamWriter(realEditFile);
            tw.Write(tbScriptText.Text);
            tw.Close();

            tw = new StreamWriter("compile.bat");
            // If building the rule file, we have to build the script file first.
            // Do this through a batch file so we have control over execution order.
            tw.WriteLine("ParseScr.exe " + scriptfile);
            if (isRuleFile)
            {
                tw.WriteLine("ParseRul.exe " + rulefile);
            }
            tw.Close();

            Process process = new Process();
            // Gonkulate/reticulate
            process = new Process();
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.RedirectStandardOutput = true;
            process.StartInfo.RedirectStandardError = true;
            process.StartInfo.CreateNoWindow = true;
            process.StartInfo.FileName = "compile.bat";
            process.Start();
            string soutput = process.StandardOutput.ReadToEnd();
            if ((soutput == null) || (soutput == ""))
            {
                if (isRuleFile)
                    MessageBox.Show("Problem parsing rule file. See log for details.", "Error");
                else
                    MessageBox.Show("Problem parsing script file. See log for details.", "Error");
                return;
            }
            else
            {
                frmResultOutput.text = soutput;
                new frmResultOutput().ShowDialog();
            }

            File.Delete("compile.bat");
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            // Save to this file
            TextWriter tw = new StreamWriter(realEditFile);
            tw.Write(tbScriptText.Text);
            tw.Close();
            noCloseVerify = true;
            Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            noCloseVerify = true;
            Close();
        }

        private void frmScriptEdit_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!noCloseVerify)
            {
                if (MessageBox.Show("Save changes before closing?", "Plase confirm", MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    btnSave_Click(sender, new EventArgs());
                    noCloseVerify = true;
                }
            }
        }

        private void btnTest_Click(object sender, EventArgs e)
        {
            string path = editFile.Substring(0, editFile.Length - 5);
            string tempfile = missionspath + path + "\\temp" + (isRuleFile ? ".rul" : ".scp");
            string scriptfile = missionspath + path + "\\" + path + "S.scp";

            // Save to a temporary file
            TextWriter tw = new StreamWriter(tempfile);
            tw.Write(tbScriptText.Text);
            tw.Close();

            tw = new StreamWriter("compile.bat");
            // If building the rule file, we have to build the script file first.
            // Do this through a batch file so we have control over execution order.
            tw.WriteLine("ParseScr.exe " + scriptfile);
            if (isRuleFile)
            {
                tw.WriteLine("ParseRul.exe " + tempfile);
            }
            tw.Close();

            Process process = new Process();
            // Gonkulate/reticulate
            process = new Process();
            process.StartInfo.UseShellExecute = false;
            process.StartInfo.RedirectStandardOutput = true;
            process.StartInfo.RedirectStandardError = true;
            process.StartInfo.CreateNoWindow = true;
            process.StartInfo.FileName = "compile.bat";
            process.Start();
            string soutput = process.StandardOutput.ReadToEnd();
            if ((soutput == null) || (soutput == ""))
            {
                if (isRuleFile)
                    MessageBox.Show("Problem parsing rule file. See log for details.", "Error");
                else
                    MessageBox.Show("Problem parsing script file. See log for details.", "Error");
                return;
            }
            else
            {
                frmResultOutput.text = soutput;
                new frmResultOutput().ShowDialog();
            }

            // Delete the temp files
            try { File.Delete(missionspath + path + "\\temp.cpp"); }
            catch (Exception except) { }
            try { File.Delete(missionspath + path + "\\temp.def"); }
            catch (Exception except) { }
            try { File.Delete(missionspath + path + "\\temp.h"); }
            catch (Exception except) { }
            File.Delete(tempfile);
            File.Delete("compile.bat");
        }
    }
}