using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.IO;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace LegacyMissionImporter
{
    public partial class Form1 : Form
    {
        bool bunusedOnly = true;

        public Form1()
        {
            InitializeComponent();
            Globals.StringsEN = new LocalizedStrings();
            Globals.StringsEN.Load("Objectives_localized_Strings_EN.xml");
            Globals.LabelsEN = new LocalizedStrings();
            Globals.LabelsEN.Load("LabelMap_EN.xml");
            Missions.Load();
        }

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.InitialDirectory = "";
            ofd.Title = "Choose an mission file";
            ofd.Filter = "Mission files (*.mission)|*.mission";
            ofd.RestoreDirectory = true;
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                tbSource.Text = ofd.FileName;
            }
            else tbSource.Text = "";
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            bool isNew = false;
            if (System.Diagnostics.Process.GetProcessesByName("LegacyMissionEditor").Length > 0)
            {
                MessageBox.Show("You must close the editor before running the importer.");
                Close();
            }
            if (System.Diagnostics.Process.GetProcessesByName("LegacyMissionImporter").Length > 1)
            {
                MessageBox.Show("The Legacy Mission Importer is already running.");
                Close();
            }
            FillCombo();
        }

        public void FillCombo()
        {
            // Determine what missions are free
            cbMissionNumber.Items.Clear();
            string[] files = Directory.GetFiles("missions");
            for (int i = 12; i < 99; i++)
            {
                if ( (!bunusedOnly) || (Missions.FindMission("stl" + i,true) == -1) )
                    cbMissionNumber.Items.Add("" + i);
            }
            cbMissionNumber.SelectedIndex = -1;
            cbMissionNumber.Text = "Next";
        }

        public static void BinarySearchReplace(string file, string search, string replace)
        {
            BinaryReader br = null;
            BinaryWriter bw = null;
            try
            {
                FileInfo finf = new FileInfo(file);
                long flen = finf.Length;
                byte[] buf = new Byte[flen];
                byte[] outbuf = new Byte[flen * 2];
                FileStream fin = new FileStream(file, FileMode.Open, FileAccess.Read);
                try
                {
                    br = new BinaryReader(fin);
                    br.Read(buf, 0, (int)flen);
                    br.Close();
                }
                catch (Exception except) { return; }
                fin = null;

                // Do the search and replace
                int icur = 0;
                for (int i = 0; i < flen; i++)
                {
                    int j;
                    for (j = 0; j < search.Length; j++)
                    {
                        if (i + j >= flen) break;
                        if ((buf[i + j] != (byte)(search[j])) &&
                             (buf[i + j] != (byte)(search[j] - 'a' + 'A'))) break;
                    }
                    if (j == search.Length)
                    {
                        for (j = 0; j < replace.Length; j++)
                            outbuf[icur++] = (byte)(replace[j]);
                        i += (search.Length - 1);
                    }
                    else
                        outbuf[icur++] = buf[i];
                }

                bw = new BinaryWriter(new FileStream("sol\\temp.bin", FileMode.OpenOrCreate));
                bw.Write(outbuf, 0, icur);
                bw.Close();
                if (File.Exists(file)) File.Delete(file);
                File.Copy("sol\\temp.bin", file);
                File.Delete("sol\\temp.bin");
            }
            catch (Exception except)
            {
                MessageBox.Show(except.ToString());
            }
            finally
            {
                if (br != null) br.Close();
                if (bw != null) bw.Close();
            }
        }

        private string ImportString( BinaryReader br )
        {
            byte [] buffer = new byte[1024];
            string s = "";
            long size = br.ReadInt64();
            if (size > 0)
            {
                char[] c = br.ReadChars((int)size);
                s = new string(c);
            }
            return s;
        }

        private void ImportFile(BinaryReader br, string name )
        {
            byte[] buffer = new byte[1024];
            long size = br.ReadInt64();
            if (size > 0)
            {
                FileStream stream = new FileStream(name, FileMode.OpenOrCreate);
                BinaryWriter bw = new BinaryWriter(stream);
                for (int i = 0; i < size; i++)
                {
                    bw.Write(br.ReadByte());
                }
                bw.Close();
            }
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            // Read the objective text
            Globals.StringsEN = new LocalizedStrings();
            Globals.StringsEN.Load("Objectives_localized_Strings_EN.xml");
            Globals.LabelsEN = new LocalizedStrings();
            Globals.LabelsEN.Load("LabelMap_EN.xml");

            if (!Missions.Load())
            {
                MessageBox.Show("Can't load campaign file.");
                return;
            }

            int i;
            int origIndex = 0;
            int newIndex;
            string fileName = tbSource.Text;
            FileStream fs = null;
            try
            {
	            FileAttributes attr = FileAttributes.Normal;
	            File.SetAttributes(fileName, attr);
                fs = new FileStream(fileName, FileMode.Open);
            }
            catch (Exception except)
            {
                MessageBox.Show(except.ToString());
            }

            if ( fs == null) return;
            BinaryReader br = new BinaryReader(fs);

            string missionName = ImportString(br);
            string solFile = ImportString(br);
            string date = ImportString(br);
            string image = ImportString(br);
            string numShips = ImportString(br);
            string hero = ImportString(br);
            string skipPoints = ImportString(br);
            string freeCPs = ImportString(br);

            string origIndexS = solFile.Substring(3, 2);
            origIndex = int.Parse(origIndexS);
            for (i = 0; i < cbMissionNumber.Items.Count; i++ )
            {
//                if (cbMissionNumber.Items[i].ToString() == origIndexS)
//                    cbMissionNumber.SelectedIndex = i;
//                else
//                {
                    if (cbMissionNumber.SelectedIndex == -1)
                        cbMissionNumber.SelectedIndex = 0;
//                }
            }

            string newIndexS = cbMissionNumber.SelectedItem.ToString();
            newIndex = int.Parse(newIndexS);
            
            // Read the drl, dsl, and sol files
            if (File.Exists("temp.drl")) File.Delete("temp.drl");
            if (File.Exists("temp.dsl")) File.Delete("temp.dsl");
            if (File.Exists("temp.sol")) File.Delete("temp.sol");
            if (File.Exists("temp_objectives.xml")) File.Delete("temp_objectives.xml");

            ImportFile(br, "temp.drl");
            ImportFile(br, "temp.dsl");
            ImportFile(br, "temp.sol");
            ImportFile(br, "temp_objectives.xml");

            // Perform binary search and replace operations on these files
            BinarySearchReplace("temp.drl", "stl" + origIndexS, "STL" + newIndexS );
            BinarySearchReplace("temp.dsl", "stl" + origIndexS, "STL" + newIndexS );
            BinarySearchReplace("temp.sol", "stl" + origIndexS, "STL" + newIndexS );
            BinarySearchReplace("temp_objectives.xml", "stl" + origIndexS, "STL" + newIndexS );

            // Store the files
            string destFile;
            destFile = "missions\\stl" + newIndexS + "R.drl";
            if (File.Exists(destFile)) File.Delete(destFile);
            File.Copy("temp.drl", destFile);
            destFile = "missions\\stl" + newIndexS + "S.dsl";
            if (File.Exists(destFile)) File.Delete(destFile);
            File.Copy("temp.dsl", destFile);
            destFile = "sol\\stl" + newIndexS + ".sol";
            if (File.Exists(destFile)) File.Delete(destFile);
            if (File.Exists("temp.sol")) File.Copy("temp.sol", destFile);
            destFile = "objectives\\stl" + newIndexS + "_objectives.xml";
            if (File.Exists(destFile)) File.Delete(destFile);
            File.Copy("temp_objectives.xml", destFile);
            
            File.Delete("temp.drl");
            File.Delete("temp.dsl"); 
            File.Delete("temp_objectives.xml");

            // Read counts
            Int32 string_count = br.ReadInt32();
            for (i = 0; i < string_count; i++)
            {
                string id = ImportString(br);
                string newid = id.Replace("STL" + origIndexS, "STL" + newIndexS);
                string text = ImportString(br);
                Globals.StringsEN.Set(newid, text);
            }

            // Final couple of things we need to do:
            // Add the mission to the campaign_xml file
            // See if this mission is in the file
            DataRow row;
            int iat = 0;
            iat = Missions.FindMission("stl" + newIndexS,true);
            if (iat > -1)
                row = Missions.table.Rows[iat];
            else
            {
                row = Missions.table.NewRow();
                Missions.table.Rows.Add(row);
            }

            row["Mission Name"] = missionName;
            row["SOL File"] = "stl" + newIndexS + ".sol";
            row["Rule File"] = "stl" + newIndexS + "R.drl";
            row["Script File"] = "stl" + newIndexS + "S.dsl";
            row["Date"] = date;
            row["Image"] = image;
            row["Num Ships"] = numShips;
            row["Hero"] = hero;
            row["Skip Points"] = skipPoints;
            row["Free CPs"] = freeCPs;
            Missions.Save();
            Globals.LabelsEN.Set("mission_select" + row["SOL File"].ToString(), missionName + " ");

            br.Close();
            Globals.StringsEN.Save();
            Globals.LabelsEN.Save();

            MessageBox.Show("Mission \""+missionName+"\" has been imported as STL" + newIndexS +".");
            FillCombo();
        }

        private void cbUnused_CheckedChanged(object sender, EventArgs e)
        {
            bunusedOnly = cbUnused.Checked;
            FillCombo();
        }
    }
}