using System;
using System.Data;
using System.IO;
using System.Xml;
using System.Windows.Forms;

/// <summary>
/// Summary description for Class1
/// </summary>
public class LocalizedStrings
{
    public DataTable strings;
    public string myfile;

	public LocalizedStrings()
	{
        strings = new DataTable();
	}

    public void Load(string file)
    {
        if ( file != null ) myfile = file;
        strings = new DataTable();
        strings.Columns.Add("ID");
        strings.Columns.Add("Text");
        strings.PrimaryKey = new DataColumn[] { strings.Columns["ID"] };

        XmlTextReader reader = new XmlTextReader(myfile);
        DataRow row = null;
        while (reader.Read())
        {
            if (reader.NodeType == XmlNodeType.Element)
            {
                if (reader.Name == "Entry")
                {
                    row = strings.NewRow();
                    row["ID"] = reader.GetAttribute("id");
                    strings.Rows.Add(row);
                }
            }
            if (reader.NodeType == XmlNodeType.Text)
            {
                row["Text"] = reader.Value;
            }
        }
        reader.Close();
    }

    public void Save()
    {
        string tempfile = myfile + "x";
        TextWriter tw = new StreamWriter(tempfile);
        tw.WriteLine("<?xml version=\"1.0\" encoding=\"windows-1252\"?>");
        tw.WriteLine("<Root_Element>");
        for (int i = 0; i < strings.Rows.Count; i++)
        {
            tw.WriteLine("\t<Entry id=\"" + strings.Rows[i]["ID"] + "\">" + strings.Rows[i]["Text"] + "</Entry>");
        }
        tw.WriteLine("</Root_Element>");
        tw.Close();

        // Writing was successful. Copy the temp file to the main file.
        try
        {
            if ( File.Exists(myfile) )
                File.Delete(myfile);
        }
        catch (Exception except)
        {
            MessageBox.Show(except.ToString());
        }
        try
        {
            File.Copy(tempfile, myfile);
        }
        catch (Exception except)
        {
            MessageBox.Show(except.ToString());
        }
        try
        {
            File.Delete(tempfile);
        }
        catch (Exception except)
        {
            MessageBox.Show(except.ToString());
        }
    }

    // Set the label text
    public void Set(string id, string text)
    {
        // Add if it doesn't already exist
        int i;
        for (i = 0; i < strings.Rows.Count; i++)
        {
            if (strings.Rows[i]["ID"].ToString().ToUpper() == id.ToUpper())
            {
                strings.Rows[i]["Text"] = text;
                return;
            }
        }
        DataRow row = strings.NewRow();
        row["ID"] = id;
        row["Text"] = text;
        strings.Rows.Add(row);
    }

    // Return the label text
    public string Get(string id)
    {
        int i;
        for (i = 0; i < strings.Rows.Count; i++)
        {
            if (strings.Rows[i]["ID"].ToString() == id)
            {
                return strings.Rows[i]["Text"].ToString();
            }
        }
        return null;
    }
}
