using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Xml;
using System.Windows.Forms;

namespace LegacyMissionImporter
{
    class Missions
    {
        public static DataTable table;
        public const int DEFAULT_MISSION_COUNT = 0;

        public static bool Load()
        {
            table = new DataTable();
            table.TableName = "table";
            DataColumn missionIndex = new DataColumn("MissionIndex", typeof(int));
            missionIndex.AutoIncrement = true;
            missionIndex.AutoIncrementSeed = 1;
            missionIndex.AutoIncrementStep = 1;
            table.Columns.Add("Mission Name");
            table.Columns.Add("SOL File");
            table.Columns.Add("Rule File");
            table.Columns.Add("Script File");
            table.Columns.Add("Date");
            table.Columns.Add("Image");
            table.Columns.Add("Num Ships");
            table.Columns.Add("Hero");
            table.Columns.Add("Skip Points");
            table.Columns.Add("Free CPs");

            // Add the default missions
            DataRow row;

            // Populate the table list from Legacytable.xml
            string filename = "campaign\\Legacy\\LegacyMissions.xml";
            if (!File.Exists(filename)) return false;
            XmlTextReader reader = new XmlTextReader(filename);
            if (reader == null)
            {
                MessageBox.Show("Could not open missions XML file.");
                return false;
            }

            while (reader.Read())
            {
                if (reader.Name.Length > 0 && reader.Name != "ShellSettings")
                {
                    row = table.NewRow();
                    row["Mission Name"] = reader.Name;
                    table.Rows.Add(row);
                    row["SOL File"] = reader.GetAttribute("file");

                    // Correct the name; grab the label
                    string stest = Globals.LabelsEN.Get("mission_select" + row["SOL File"]);
                    if (stest != null)
                    {
                        if (stest[stest.Length - 1] == ' ') stest = stest.Substring(0, stest.Length - 1);
                        row["Mission Name"] = stest;
                    }
                    row["Rule File"] =
                        reader.GetAttribute("file").Substring(0,
                            reader.GetAttribute("file").Length - 4) + "R.rul";
                    row["Script File"] =
                        reader.GetAttribute("file").Substring(0,
                            reader.GetAttribute("file").Length - 4) + "S.scp";
                    row["Date"] = reader.GetAttribute("date");
                    row["Image"] = reader.GetAttribute("image");
                    row["Num Ships"] = reader.GetAttribute("numShips");
                    row["Hero"] = reader.GetAttribute("hero");
                    row["Skip Points"] = reader.GetAttribute("skipPoints");
                    row["Free CPs"] = reader.GetAttribute("freeCPs");
                }
            }
            reader.Close();
                return true;
        }

        public static int FindMission(string name, bool baseNameOnly)
        {
            for ( int i = 0; i < table.Rows.Count; i++ )
            {
                string missionName = table.Rows[i]["Mission Name"].ToString();
                missionName = missionName.ToLower();
                string solName = table.Rows[i]["SOL File"].ToString();
                solName = solName.ToLower();

                if (!baseNameOnly)
                    if (missionName == name.ToLower()) return i;
                if (solName == (name+".sol").ToLower()) return i;
            }
            return -1;
        }

        public static void Save()
        {
            // Save to a temporary file
            string tempfile = "campaign\\Legacy\\LegacyMissions.xmlx";
            string realfile = "campaign\\Legacy\\LegacyMissions.xml";
            TextWriter tw = new StreamWriter(tempfile);

            tw.WriteLine("<ShellSettings sequence=\"false\" name=\"LegacyMissions\">");
            for ( int i = DEFAULT_MISSION_COUNT; i < table.Rows.Count; i++ )
            {
                bool bhero = ( table.Rows[i]["Hero"].ToString() != "" );
                bool bskip = ( table.Rows[i]["Skip Points"].ToString() != "" );
                bool bfreeCPs = (table.Rows[i]["Free CPs"].ToString() != "");
                
                string therest = "\"";
                if ( bhero ) therest += " hero=\""+table.Rows[i]["Hero"]+"\"";
                if ( bskip ) therest += " skipPoints=\""+table.Rows[i]["Skip Points"]+"\"";
                if ( bfreeCPs ) therest += " freeCPs=\""+table.Rows[i]["Free CPs"]+"\"";
                string missionName = table.Rows[i]["Mission Name"].ToString();
                missionName = Globals.RemoveSpaces(missionName);

                tw.WriteLine("\t<"+missionName+
                    " file=\"" + table.Rows[i]["SOL File"] + 
                    "\" date=\"" + table.Rows[i]["Date"] + 
                    "\" image=\"" + table.Rows[i]["Image"] + 
                    "\" numShips=\"" + table.Rows[i]["Num Ships"] +
                    therest + "/>");
            }
            tw.WriteLine("</ShellSettings>");
            tw.Close();
            
            // This was successful. Save it over the main file
            try
            {
                if ( File.Exists(realfile) ) File.Delete(realfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            try
            {
                File.Copy(tempfile, realfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
            try
            {
                File.Delete(tempfile);
            }
            catch (Exception e)
            {
                MessageBox.Show(e.ToString());
            }
        }
    }
}
